#include "params.h"
      subroutine posrftsb(indx,npts,isrfty,fnt,dfntdt,sndpth,
     $                  sst, tsbsf   ,alpha ,beta  ,
     $                  gamma  ,tpthck ,fbt)
c----------------------------------------------------------------------
c
c computes surface/sub surface temperatures assuming thermal
c diffusion with non-constant conductivity, solved by backward implicit 
c time stepping procedure with linearized top flux boundary condition 
c and specified bottom flux boundary condition.
c
c assumes 7 surface types, along with their geometric and thermal
c properties, as follows:
c
c   1  ocean
c   2  sea ice
c   3  permanent land (or sea) ice
c   4  equatorial forest
c   5  mid/lat and high lat forest
c   6  shrubland/tundra
c   7  desert
c
c assumes plevmx number of surface/sub-surface levels, and
c ascribes physical properties for these levels for each of the
c 7 surface types above
c
c-----------------------------------------------------------------------
      implicit none
c-----------------------------------------------------------------------
#include "pmgrid.com"
c-----------------------------------------------------------------------
#include "pagrid.com"
c-----------------------------------------------------------------------
#include "comctl.com"
c-----------------------------------------------------------------------
#include "comtim.com"
c-----------------------------------------------------------------------
#include "comtsc.com"
c-----------------------------------------------------------------------
#include "pcoupled.com"
c-----------------------------------------------------------------------
c
c input arguments
c

      integer isrfty(uwlon)    ! surface type index (1..7)
      real fnt(uwlon),         ! net top surface flux
     $     dfntdt(uwlon),      ! srfc tmp partial derivative net srf flx
     $     sndpth(uwlon),       ! snow depth
     $     sst(uwlon)        !  SST from ocean
      integer indx(uwlon)       ! Longitude index array (land)
      integer npts              ! Number of land points
      integer ii              ! Longitude indices 

c
c input/output arguments
c
      real tsbsf(uwlon,plevmx),! initial/final surface/sub-surface tmprs
     $     alpha(uwlon),       ! ts  term from top layer linear equation
     $     beta(uwlon),        ! rhs term from top layer linear equation
     $     gamma(uwlon),       ! t2  term from top layer linear equation
     $     tpthck(uwlon),      ! top layer geometrical thickness
     $     fbt(uwlon)          ! heat flux into bottom layer
c
c local workspace
c
      integer      i,          ! longitude index
     $		   j,		! lat index
     $		   m,
     $             n,          ! sub-surface level index
     $          jndx,          ! surface type index
     $           ier           ! error return on tridiagonal solver
c
      real z(uwlon,0:plevmx),  !  level geometrical depth in m
     $     cmass(uwlon,plevmx),!  specific heat of soil in J/kg/K
     $     rho(uwlon,plevmx),  !  mass density (kg/m3)
     $     tk(uwlon,plevmx)    !  thermal conductivity (watts/m/k)
c
c
      integer    nsrfty        !  number of surface types
      parameter (nsrfty=7)
c
      real thckly(nsrfty,plevmx),  !  layer thicknesses (m)
     $     cmtype(nsrfty,plevmx),  !  mass specific heat (J/kg/K)
     $     rhtype(nsrfty,plevmx),  !  mass density (kg/m3)
     $     tktype(nsrfty,plevmx)   !  thermal conductivity (J/m/s)
c
      data thckly / 2., .500, .250, .050, .090, .080, .120,
     $              5., .500, .500, .366, .390, .435, .492,
     $             10., .500, .500,1.369,1.459,1.628,1.841,
     $             33.,.500,8.500,6.990,7.450,8.310,9.400/
c
      data cmtype /4.20e3,2.07e3,2.07e3,1.04e3,7.20e2,5.60e2,4.16e2,
     $             4.20e3,2.07e3,2.07e3,1.04e3,7.20e2,5.60e2,4.16e2,
     $             4.20e3,2.07e3,2.07e3,1.04e3,7.20e2,5.60e2,4.16e2,
     $             4.20e3,2.07e3,2.07e3,1.04e3,7.20e2,5.60e2,4.16e2/
c
      data rhtype /1.00e3,9.20e2,9.20e2,2.50e3,2.50e3,2.50e3,2.50e3,
     $             1.00e3,9.20e2,9.20e2,2.50e3,2.50e3,2.50e3,2.50e3,
     $             1.00e3,9.20e2,9.20e2,2.50e3,2.50e3,2.50e3,2.50e3,
     $             1.00e3,9.20e2,9.20e2,2.50e3,2.50e3,2.50e3,2.50e3/
c
      data tktype /15.0 ,2.200 ,2.200 ,1.408 ,1.104 ,1.071 ,1.019 ,
     $             15.0 ,2.200 ,2.200 ,1.408 ,1.104 ,1.071 ,1.019 ,
     $             15.0 ,2.200 ,2.200 ,1.408 ,1.104 ,1.071 ,1.019 ,
     $             15.0 ,2.200 ,2.200 ,1.408 ,1.104 ,1.071 ,1.019 /
c
c properties of ice and air used to determine
c snow thermal properties in 'tmpsub' mks units)
c
      real cmair,     ! mass specific heat of air
     $     rhair,     ! mass density of surface air
     $     tkair,     ! thermal conductivity of air
     $     cmice,     ! mass specific heat of ice
     $     rhice,     ! mass density of ice
     $     tkice,     ! thermal conductivity of ice
     $     frcair     ! fraction of air assumed in mix of ice and air
c
      data cmair  /1.00e3/
      data rhair  /1.25 /
      data tkair  /0.025/
      data cmice  /2.07e3/
      data rhice  /9.20e2/
      data tkice  /2.200/
      data frcair /0.90/   
C
      real htsrc(uwlon,plevmx),   ! external heat source (W/m3)
     $     msnow,                 ! mass path of snow
     $     msoil,                 ! mass path of soil
     $     zsnow,                 ! snow geometric depth
     $     diag(uwlon,plevmx),    ! diagonal matrix elements
     $     sbdiag(uwlon,plevmx),  ! sub-diagonal matrix elements
     $     spdiag(uwlon,plevmx),  ! super-diagonal matrix elements
     $     rhs(uwlon,plevmx),     ! rhs of tri-diagonal matrix eqn.
     $     wws(uwlon*plevmx)       ! working storage for omtdlss
      real tin(uwlon,plevmx)      ! initial sub-surface temperatures
      real thck(uwlon),           ! level thickness
     $     cmty(uwlon),           ! level mass heat capacity
     $     rhty(uwlon),           ! level mass density
     $     tkty(uwlon)            ! level thermal conductivity
      logical scvr(uwlon)         ! true if surface snow covered
c
      real rdtime,    ! inverse model time step
     $     cmsnow,    ! snow mass heat capacity
     $     rhsnow,    ! snow mass density
     $     tksnow     ! snow thermal conducitivity
c
      real  crt,      ! cmass*rho*rdtime
     $      ztop,     ! top level thickness
     $      rztop,    ! 1/ztop
     $      tktop,    ! top level bottom interface thermal conductivity
     $      zmpl,     ! depth of top level bottom interface
     $      zm,       ! mid depth of level
     $      delzpl,   ! top level lower half thickness
     $      tmp       ! crt - dfntdt(i)*rztop
c
      real  delz,     ! level thickness
     $      zmmn,     ! level bottom interface depth
     $      delzmn,   ! level bottom half thickness
     $      fpls,     ! 1/(delz*delzpl)
     $      fmns,     ! 1/(delz*delzmn)
     $      tkpls,    ! level top half thermal conductivity
     $      tkmns     ! level bottom half thermal conductivity
c
      real  zbot,     ! bottom level thickness
     $      tkbot     ! bottom level top interface thermal conductivity
c
c-----------------------------------------------------------------------
c
      if(npts .eq. 0) return

      rdtime = 1./dtime
      cmsnow = (1.-frcair)*cmice + frcair*cmair
      rhsnow = (1.-frcair)*rhice + frcair*rhair
      tksnow = (1.-frcair)*tkice + frcair*tkair
c
c no external heat source currently:
c
      do 22 ii=1,npts
          i=indx(ii)
      do 10 n=1,plevmx
            htsrc(i,n) = 0.0
   10   continue
   22    continue
c
c define logical for snow covered surfaces:
c
      do ii=1,npts
          i=indx(ii)
         scvr(i) = sndpth(i).gt.0.
      enddo
c
c define thermal properities for each sub/surface level, starting
c with the top level
c
      do ii=1,npts
          i=indx(ii)
         jndx    = isrfty(i)
         thck(i) = thckly(jndx,1)
         cmty(i) = cmtype(jndx,1)
         rhty(i) = rhtype(jndx,1)
         tkty(i) = tktype(jndx,1)
      enddo
c
      do ii=1,npts
          i=indx(ii)
c
c initialize fields for no snow cover
c
         z(i,0)     = 0.0
         z(i,1)     = thck(i)
         cmass(i,1) = cmty(i)
         rho(i,1)   = rhty(i)
         tk(i,1)    = tkty(i)
c
c modify level 1 fields for snow cover present, mixing snow
c if present.
c
         if(scvr(i)) then
c
c snow equivalent depth times snow liquid water depth gives
c the physical depth of snow for thermal conduction computation.
c
           zsnow     = sndpth(i)*snwedp
           msnow     = rhsnow*zsnow
           msoil     = rhty(i)*thck(i)
           rho(i,1)  = (msnow*rhsnow + msoil*rhty(i))/(msnow+msoil)
           cmass(i,1)= (msnow*cmsnow + msoil*cmty(i))/(msnow+msoil)
           tk(i,1)   = (msnow*tksnow + msoil*tkty(i))/(msnow+msoil)
           z(i,1)    = (msnow+msoil) / rho(i,1)
         endif
c
      enddo
c
c set surface thermal properties for the lower sub/surface levels:
c
      do n=2,plevmx
      do ii=1,npts
          i=indx(ii)
            jndx       = isrfty(i)
            thck(i)    = thckly(jndx,n)
            cmass(i,n) = cmtype(jndx,n)
            rho(i,n)   = rhtype(jndx,n)
            tk(i,n)    = tktype(jndx,n)
            z(i,n)  = z(i,n-1) + thck(i)
      enddo
      enddo
c
c define set of linear equations for temperature
c
      do n=1,plevmx
      do ii=1,npts
          i=indx(ii)
            tin(i,n) = tsbsf(i,n)
      enddo
      enddo
c
c if sea ice, compute heat flux from underlying ocean
c
      do ii=1,npts
          i=indx(ii)
         fbt(i) = 0.0
         if(isrfty(i).eq.2) then
            zbot    = 0.5*(z(i,plevmx) - z(i,plevmx-1))
#ifdef ICE_MODEL
          fbt(i)= -tk(i,plevmx)*(sst(i) - tin(i,plevmx))/zbot
#else
            fbt(i)= -tk(i,plevmx)*(271.16 - tin(i,plevmx))/zbot
#endif
         endif
      enddo
c
c set up linear equations
c
      do ii=1,npts
          i=indx(ii)
         sbdiag(i,1)      = 0.
         spdiag(i,plevmx) = 0.
      enddo
c
c single level
c
      if(plevmx.eq.1) then
        do ii=1,npts
          i=indx(ii)
            tpthck(i) = (z(i,1) - z(i,0))
            rztop     = 1./(z(i,1) - z(i,0))
            crt       = (cmass(i,1)*rho(i,1)*rdtime)
            diag(i,1) = crt - dfntdt(i)*rztop
            rhs(i,1)  = diag(i,1)*tin(i,1) + fnt(i)*rztop -
     $                  fbt(i)*rztop + htsrc(i,1)
        enddo
c
c more than one level
c
      else if(plevmx.gt.1) then
c
c top level
c
        do ii=1,npts
          i=indx(ii)
            crt         = cmass(i,1)*rho(i,1)*rdtime
            ztop        = z(i,1) - z(i,0)
            tpthck(i)   = ztop
            rztop       = 1./ztop
            tktop       = 0.5*(tk(i,1) + tk(i,2))
            zmpl        = 0.5*(z(i,2) + z(i,1))
            zm          = 0.5*(z(i,1) + z(i,0))
            delzpl      = zmpl - zm
            fpls        = 1./(ztop*delzpl)
            tmp         = crt - dfntdt(i)*rztop
            diag(i,1)   = tmp + tktop*fpls
            spdiag(i,1) = -tktop*fpls
            rhs(i,1)    = tmp*tin(i,1) + fnt(i)*rztop + 
     $	    htsrc(i,1)
        enddo
c
c intermediate level
c
         do n=2,plevmx-1
           do ii=1,npts
            i=indx(ii)
               crt         = cmass(i,n)*rho(i,n)*rdtime
               delz        = z(i,n) - z(i,n-1)
               zmpl        = 0.5*(z(i,n+1) + z(i,n))
               zm          = 0.5*(z(i,n)   + z(i,n-1))
               zmmn        = 0.5*(z(i,n-1) + z(i,n-2))
               delzpl      = zmpl - zm
               delzmn      = zm - zmmn
               fpls        = 1./(delz*delzpl)
               fmns        = 1./(delz*delzmn)
               tkpls       = 0.5*(tk(i,n+1)+tk(i,n))
               tkmns       = 0.5*(tk(i,n)+tk(i,n-1))
               sbdiag(i,n) =  -tkmns*fmns
               diag(i,n)   =  crt + (tkpls*fpls + tkmns*fmns)
               spdiag(i,n) =  -tkpls*fpls
               rhs(i,n)    = crt*tin(i,n) + htsrc(i,n)
	   enddo
	 enddo
c
c bottom level
c
         do ii=1,npts
           i=indx(ii)
            crt        = cmass(i,plevmx)*rho(i,plevmx)*rdtime
            zbot       = z(i,plevmx) - z(i,plevmx-1)
            zm         = 0.5*(z(i,plevmx)   + z(i,plevmx-1))
            zmmn       = 0.5*(z(i,plevmx-1) + z(i,plevmx-2))
            delzmn     = zm - zmmn
            tkbot      = 0.5*(tk(i,plevmx-1) + tk(i,plevmx))
            fmns       = 1./(zbot*delzmn)
            sbdiag(i,plevmx) = -tkbot*fmns
            diag(i,plevmx)   = crt + (tkbot*fmns)
            rhs(i,plevmx)    = crt*tin(i,plevmx) - fbt(i)/zbot
     $                          + htsrc(i,plevmx)
	 enddo
      endif
c
c for the linear equation ax = b,  a and b arrays are now set; 
c solve for temperatures (x):
c
      if(plevmx.eq.1) then
         do ii=1,npts
           i=indx(ii)
            tsbsf(i,1) = rhs(i,1)/diag(i,1)
	 enddo    
      else
         call pomtdlss(sbdiag ,diag  ,spdiag ,rhs  ,tsbsf ,
     $               plevmx ,uwlon ,npts,indx ,wws ,
     $		     uwlon*plevmx,ier)
         if(ier.ne.0) then
           write(6,*) 'srftsb: Error returned from omtdlss'
         endif
      endif
c
c set top linear equation terms:
c
      if(plevmx.gt.1) then
         do ii=1,npts
           i=indx(ii)
            gamma(i) = spdiag(i,1)*tsbsf(i,2)
      enddo
      else
         do ii=1,npts
           i=indx(ii)
            gamma(i) = 0.0
         enddo
      end if
c
      do ii=1,npts
        i=indx(ii)
         alpha(i) = diag(i,1)
         beta(i)  = rhs(i,1)
      enddo
c
c done
c 
      return
      end
