c
#include "params.h"
c
c  The Semtner 3-layer ice model
c
c  This routine is based on tice3l.F from
c  NCAR's CSIM 2.2.6
c
c  rewritten and adapted for FOAM by R. Jacob, March, 2000
c
      subroutine tice3l(i,j,
c surface input
     $hflx,ifntdt,solin,snowfx,Fb,
c surface input/output
     $ tsurf,tinter2,tmidsnow,TI,
     $ hice,hsnow,qstor2,
c surface output
     $ fbt,snwmlt)


      implicit none
c------------------------------------------------
#include "pmgrid.com"
c------------------------------------------------
#include "comtim.com"
c------------------------------------------------
#include "comtsc.com"
c------------------------------------------------
#include "pcoupled.com"
c------------------------------------------------
#include "pcplrstr.com"
c------------------------------------------------

c input
      integer indx(uwlon)
      integer npts
      real hflx    ! surface heat flux forcing
      real ifntdt(uwlon,uwlat)  ! derivative of fnt
      real snowfx
      real TI(2)
      real solin ! surface absorbed solar flux
      real tsurf	      ! surface temp
      real tinter2	      ! snow/ice interface
      real tmidsnow	      ! snow layer midpoint temp
      real hice		      ! ice thickness
      real hsnow	      ! snow thickness
      real qstor2
      real Fb         ! heat flux from ocean (always 0.)

c input/output
      real ts(uwlon,uwlat)    ! surface temp
c output
      real fbt(uwlon,uwlat)    ! flux between ice/ocean
      real snwmlt(uwlon,uwlat)     ! snow melt in meters

c physical variables
      real rhoice     ! density of ice
      real rhoh2o     ! density of water
      real snwmax     ! maximum snow depth in liquid water equiv
      real Qi         ! latent heat of fusion, surface ice
      real Qs         ! latent heat of fusion, snow
      real Ci         ! specific heat of ice
      real Cs         ! specific heat of snow
      real Qb         ! latent heat of fusion, bottom ice
      real Civ	      ! volume heat capacity of ice
      real Csv	      ! volume heat capacity of snow
      real minice     ! minimum ice thickness
      real minsnow     ! minimum snow thickness on ice
      real minsnowf   ! minimum snow thickness for constant heat flux
      real Yk         ! conductivity of ice
      real Sk         ! conductivity of snow
      real Tb         ! temp at bottom of ice
      real tfrez      ! freezing temp in K
      real fpensol    ! fraction of solar radiation penetrating ice
      real solout     ! net solar exiting bottom
      real coptic     ! bulk shortwave extinction
      real sdf        ! snow density factor for thermo calculations
      real idf        ! ice density factor for thermo calculations
      real rhos       ! density of snow
      real dh,hi,fa,fo,fs,dhi,dhb
      real dhtotal
      real con,xh,dhx,dfi
      real meltw,oldthick,oldthick2
      real dthick,hsold,dsnow
      real denom,dtemp
      real timelt     ! temperature to hold ice at when its melting.
      real hmin   ! minimum ice thickness for 3-layer model
      real rlayr
      real rhin
      real hinew
      real fact
      real fpensolx
      real qmax   ! maximum allowed heat storage in brine reservoir
      real qneed
      real htup,htdown,con1,con2
      real iceflux(0:2)
      real Zp(0:2)
      real Z(0:2)
      real Tp(3)
      real tpast


      integer iw,jw,i,j,ii,k
      integer nlayr
      integer nlayrmx
      integer nlayrm1



      nlayr = 2
      nlayrmx = 2
      rhoh2o= 1.e3
      idf = .905
      rhoice = idf*rhoh2o
      rhos  = 333.0
      sdf = rhoh2o/rhos
      snwmax = 1.0
      Qi = latice*rhoice !  J/ m3
      Qs = 110.2e6  ! J/m3
      Ci = 2.1e3  ! J/Kg/K
      Cs = 2.09e3 ! J/Kg/K
      Qb =Qi
      Yk = 2.03  ! J/(m s K)
      Sk = 0.31  ! J/(m s K)
c     Fb = 0.  !
      Civ = Ci*rhoice ! J/m3 K
      Csv = Cs*rhos
      minice = .01  ! m
      minsnow = .0001  !m
      minsnowf = .15  !m
      hmin = .25
      fpensol = .3
      coptic = 1.5  ! m-1
      tfrez  = 271.36  ! -1.8 C
      Tb = tfrez
      timelt = tmelt - 0.1
      iw=0
      jw=0

      fo=0.
      dhi=0.
      dhb=0.
      Zp(0)=0.
      Z(0)=0.
      Tp(1)=0.
      Tp(2)=0.
      Tp(3)=0.

      if(hice .lt. 2.*hmin) nlayr = 1
      nlayr=min(nlayr,nlayrmx)
      rlayr=float(nlayr)
      nlayrm1 = nlayr - 1
      rhin = rlayr/hice
      qmax = Qi*0.50*(hice - hmin)


      tpast = tsurf

ccccccccccccccccccccccccccccccccccccccccc
c            snow cover on thick enough ice
ccccccccccccccccccccccccccccccccccccccccc
c if surface has snow cover, all energy goes to changing
c snow temp.
       if(hsnow*sdf .gt. minsnow) then

c forcing terms on temperature
           con=2.*Sk/(hsnow*sdf)

c for snow lt 15cm assume vertically constant heat flux
           if(hsnow*sdf .lt. minsnowf) then
	    con=Sk/(hsnow*sdf + ((hice*Sk)/(Yk*rlayr*2.)))
	    tmidsnow=TI(1)
	   endif

	   hflx=hflx+ solin
	   dtemp=(-hflx + con*(tmidsnow-tpast))/
     $	     (con - ifntdt(i,j))

           tsurf=tpast+dtemp

c  hold snow temp to freezing for melting
	   if(tsurf .gt. tmelt) tsurf = tmelt

c find temperature at snow/ice interface
	   tinter2=(hice*Sk*tmidsnow + 
     $         rlayr*hsnow*sdf*Yk*TI(1) )/
     $		(hice*Sk + rlayr*sdf*hsnow*Yk)

c  compute fluxes
c change heat flux to positive upward
	   solout = 0.
	   fa= hflx
	   fs= con*(tmidsnow - tsurf)
	   iceflux(0)=2.*rhin*Yk*(TI(1)-tinter2)
	   iceflux(1)=Yk*(TI(2)-TI(1))*rhin

	   if(hsnow*sdf.lt. minsnowf) then
	     tinter2=(hice*Sk*tsurf + 
     $         2.*rlayr*hsnow*sdf*Yk*TI(1) )/
     $		(hice*Sk + 2.*rlayr*sdf*hsnow*Yk)
	     fs=Sk*(tinter2-tsurf)/(hsnow*sdf)
	     iceflux(0)=fs
	     tmidsnow=(tinter2+tsurf)*0.5
           endif

c update temp and thickness of snow

c  to cold to melt so add snow.

c  use lt instead of ne since tsurf will
c  never be gt tmelt -rlj
           if(tsurf .lt. tmelt) then
	      if(hsnow*sdf .ge. minsnowf) then
		 tmidsnow=tmidsnow+dtime*(iceflux(0)-fa)/
     $		 (hsnow*sdf*Csv)
	      endif

	      hsnow=hsnow+snowfx*dtime
	      
c  get rid of snow if there's too little
	      if(hsnow*sdf.lt.minsnow) then
		snwmlt(i,j)=snwmlt(i,j)+hsnow
		hsnow=0.
		tmidsnow=tmelt
              endif

	   else

c  snow is melting
           snwmlt(i,j)=snwmlt(i,j)+ snowfx*dtime

c see if snow actually melts
	     hsold=hsnow
	     if(fs.gt.fa) then
		dsnow=(dtime*(fs-fa)/Qs)/sdf
c  see if all snow melts
		if(dsnow.ge.hsnow) then
		  snwmlt(i,j)=snwmlt(i,j)+hsnow
		  hsnow=0.
		  tmidsnow=tmelt
                else
	          hsnow=hsnow - dsnow
	          snwmlt(i,j)=snwmlt(i,j)+ dsnow
		endif
             endif
	     if(hsold*sdf .ge. minsnowf) then
		 tmidsnow=tmidsnow+dtime*(iceflux(0)-fs)/
     $		 (hsnow*sdf*Csv)
	     endif

	     if(hsnow*sdf.lt.minsnow) then
		snwmlt(i,j)=snwmlt(i,j)+hsnow
		hsnow=0.
		tmidsnow=tmelt
             endif
	   endif
c restrict snow layer temp to be melting point
	   if(tmidsnow.gt.tmelt)tmidsnow=tmelt

c make sure upper decay of ice is zero
	   dhi=0

	   if(nlayr.gt.1) then
c update temperature in top layer of ice.
	     Tp(1)=TI(1) + 
     $	     dtime*(iceflux(1)-iceflux(0))*rhin/Civ

	     if(Tp(1).gt. timelt)Tp(1)=timelt
c heat stored in brine pockets
	     if(qstor2.ne.0. .and. Tp(1).lt.timelt) then
c		use heat stored to keep temp at timelt
	       qneed = (timelt - Tp(1))*Civ*hice/rlayr
	       if(qstor2.le.qneed) then
c     not enough heat to keep temp at timelt
		  fact=qstor2/qneed
		  Tp(1)= Tp(1) + fact*(timelt - Tp(1))
		  qstor2=0.
               else
		  qstor2=qstor2-qneed
		  Tp(1)=timelt
	       endif
	     endif
	   endif

           if((i.eq.iw).and.(j.eq.jw)) then
 	     write(0,*)'snow,thick',nstep,tsurf,tinter2,tmelt,dtemp,
     $	     hice,hsnow,snwmlt(i,j),snowfx,fs,fa,
     $       hflx,ifntdt(i,j)
           endif

       else
cccccccccccccccccccccccccccccccccccccccccccccc
c else calculate new ts and let snow accumulate
cccccccccccccccccccccccccccccccccccccccccccccc
        
c  allow penetration solar radiation to be stored up to
c   a maximum value
	     solout = solin*fpensol*
     $	        exp(-coptic*hice)

	     fpensolx=fpensol
	     if(qstor2 .gt. qmax) fpensolx=0.
	     if(fpensolx.ne.0. .and. nlayr.gt.1) then
	       qstor2=qstor2- dtime*(fpensolx*solin - solout)
             endif

 	     con = Yk*2.0*rhin
	     hflx=hflx + (1.-fpensolx)*solin
             if((i.eq.iw).and.(j.eq.jw)) then
	     debb(24)=(1.-fpensolx)*solin
	     debb(25)=hflx
	     endif

	     dtemp=(-hflx + con*(TI(1)-tpast))/
     $	       (con - ifntdt(i,j) + 4.*stebol*tpast*tpast*tpast)

c add solar out to flux between sea ice and ocean
	     fbt(i,j) = fbt(i,j)- solout*fice(i,j)

             tinter2=tpast+dtemp

c  hold ice temp to 273.15 (-0.1 C) for melting
	     if(tinter2 .gt. timelt) then
	       tinter2 = timelt
	     endif

	     tsurf=tinter2
             if((i.eq.iw).and.(j.eq.jw)) then
	     debb(26)=tsurf
	     debb(27)=dtemp
	     endif

c  compute fluxes
	     fa= hflx
c  
c heat flux through ice; positive upward
	     iceflux(0)=2.*rhin*Yk*(TI(1) - tinter2)
	     iceflux(1)=Yk*(TI(2)-TI(1))*rhin
             if((i.eq.iw).and.(j.eq.jw)) then
	     debb(28)=iceflux(0)
	     debb(29)=iceflux(1)
	     endif

c if to cold to melt
c then form snow on ice

c  use lt instead of ne here  rlj
c  tinter2 will never be gt then timelt by
c  check above
	     if(tinter2 .lt. timelt) then
	       dhi=0.
               if(snowfx.gt.0.)then
	        hsnow= hsnow + snowfx*dtime
		tmidsnow=tmelt
               endif
	       if(nlayr.gt.1) then
	         Tp(1)=TI(1) + dtime*(iceflux(1) - fa)*rhin/Civ
	       endif
c if melting from top
c figure out how much ice melts and let precip pass
c  through.
             else

	       if(snowfx.gt.0.) then
	         snwmlt(i,j)= snwmlt(i,j) + snowfx*dtime
	       endif

c note: CSIM assumed dhi would always be negative here.
c  it may not be so check.  -rlj
 	       if(fa.lt.iceflux(0)) then
	         dhi= dtime*(fa-iceflux(0))/Qi
	         if(qstor2.ge.qmax)then
c  stored heat used to melt ice
		   qstor2=qstor2 + dhi*Qi
		   if(qstor2 .lt. 0.) qstor2 = 0.
		   dhi = 2.*dhi
	         endif
	         if(nlayr.eq.1) then
		   hice = hice+dhi
		   rhin = 1./hice
		   dhi=0.
	         else
		   Tp(1)=TI(1)+dtime*(iceflux(1)-iceflux(0))/
     $		     ((hice/rlayr + dhi)*Civ)
	         endif
C  Make sure Tp1 is defined for case when fa ge iceflux(0)
C  RLJ  12/31/2001
               else
                 if(nlayr.gt.1) then
                   Tp(1)=TI(1)+dtime*(iceflux(1)-iceflux(0))/
     $               ((hice/rlayr + dhi)*Civ)
                 endif
               endif

	     endif

             if((i.eq.iw).and.(j.eq.jw)) then
 	        write(0,*)'thick,no snow',nstep,tsurf,tinter2,
     $	        hice,hsnow,snowfx,fa,fo
	     debb(30)=dhi
             endif

ccccccccccccccccccccccc
c  end of snow-free, ice thickn enough temp calc
ccccccccccccccccccccccc

       endif
ccccccccccccccccccccccc
c  end of temp, seaice and snow update,
c    snowfree or not
ccccccccccccccccccccccc

c brine pocket heat reservoir
       if(nlayr.gt.1) then
	  if(qstor2.ne.0. .and. Tp(1).lt.timelt) then
c possibly use heat to keep temp at timelt
	    qneed= (timelt - Tp(1))*Civ*hice/rlayr
	    if(qstor2.le.qneed) then
c     not enough heat to keep temp at timelt
		  fact=qstor2/qneed
		  Tp(1)= Tp(1) + fact*(timelt - Tp(1))
		  qstor2=0.
            else
		  qstor2=qstor2-qneed
		  Tp(1)=timelt
	    endif
	  endif
	  if(Tp(1).gt.timelt) Tp(1)=timelt

	  if(nlayr .gt. 2) then
	    write(0,*)'Dont know how to do nlayr gt 2'
	    call abortf
	  endif

       endif


c ablation or accretion at bottom
       iceflux(nlayr)=Yk*(Tb - TI(nlayr))*2.*rhin
       dhb = dtime*(iceflux(nlayr) - Fb)/Qb
       if((i.eq.iw).and.(j.eq.jw)) then
	debb(31)=iceflux(nlayr)
	debb(32)=dhb
       endif

       if(dhb.lt.0.) then
c ablation. use excess stored heat to melt ice
          if(qstor2.ge.qmax) then
	    qstor2=qstor2 + dhb*Qb
	    if(qstor2.lt.0.)qstor2=0.
	    dhb=2.0*dhb
          endif
       endif

       dhtotal=dhi+dhb
       if(dhtotal  .gt. 1.) then
	 write(0,*)'dhwatch',nstep,i,j,dhi,dhb,tsurf,sst(i,j),
     $  hsnow,fo
       endif
c  new ice thickness
       oldthick=hice

       hinew=hice+dhi+dhb

       if((i.eq.iw).and.(j.eq.jw)) then
	debb(33)=hice
	debb(34)=hinew
	debb(35)=TI(1)
	debb(36)=TI(2)
       endif

c determine depths below surface of interim and final interfaces
       con=hinew/rlayr
       do k=1,nlayr
	 Z(k)=Z(k-1) - con
       enddo

       con=hice/rlayr
       Zp(1)= -con -dhi

       if(nlayr.gt.2) then
	 do k=2,nlayrm1
	   Zp(k)=Zp(k-1) - con
	 enddo
       endif

       if(dhb.gt.0.) then
c bottom accreation (ice growing)
	  Zp(nlayr)=Zp(nlayrm1)-con
	  Tp(nlayr)=TI(nlayr) + 
     $	   dtime*(iceflux(nlayr)-iceflux(nlayrm1))*rhin/Civ
          Tp(nlayr+1)=Tb
	  do k=1,nlayr
	    TI(k) = ( (Z(k-1)-Zp(k))*Tp(k) + 
     $			(Zp(k)-Z(k))*Tp(k+1) )*rlayr/hinew
	  enddo

       else
c bottom ablation (ice melting)
	  Zp(nlayr)=Z(nlayr)
	  Tp(nlayr)=TI(nlayr) +
     $	   dtime*(iceflux(nlayr)-iceflux(nlayrm1))/
     $	   (((hice/rlayr) + dhb)*Civ)
	  do k=1,nlayr
	     if(Z(k).lt.Zp(k) .and. Zp(k).lt.Z(k-1))then
	       TI(k) = ( (Z(k-1)-Zp(k))*Tp(k) + 
     $			(Zp(k)-Z(k))*Tp(k+1) )*rlayr/hinew

	     elseif(Z(k).lt.Zp(k-1) .and. Zp(k-1).lt.Z(k-1))then
	       TI(k) = ( (Z(k-1)-Zp(k-1))*Tp(k-1) + 
     $			(Zp(k-1)-Z(k))*Tp(k) )*rlayr/hinew
	     else
	       TI(k)=Tp(k)
	     endif
	  enddo
       endif

       hice=hinew

c change number of layers?
       htup=float(nlayr+1)*hmin
       if(hice .lt. htup) then
c  possibly decrease number of layers
 	  htdown=float(nlayr)*hmin
	  if(hice .le. htdown) then
c   decrease number of layers
	      nlayr=nlayr-1
	      nlayrm1=nlayr-1
	      rlayr=float(nlayr)
	      if(nlayr.ne.0) then
		 con1= hice/float(nlayr+1)
		 con2= hice/rlayr
		 do k=1,nlayr
		    Z(k)=Z(k-1) - con1
		    Zp(k)=Zp(k-1) - con2
		 enddo
		 do k=1,nlayr
		    Tp(k)=((Zp(k-1) - Z(k))*TI(k) +
     $			   (Z(k) - Zp(k))*TI(k+1) )/con2
		 enddo
c  set the new ice temps
		 do k=1,nlayr
		    TI(k) = Tp(k)
		 enddo
              endif	
	  endif
       else
c  possibly increase number of layers
	  if(nlayr.ne.nlayrmx) then
c   increase number of layers
	      nlayr=nlayr+1
	      nlayrm1=nlayr-1
	      rlayr=float(nlayr)
	      Tp(1)=TI(1)
	      Tp(nlayr)=TI(nlayr)

	      if(nlayr .ne. 2) then
		 con1=hice/float(nlayrm1)
		 con2=hice/rlayr
		 do k=1,nlayr
		   Z(k)=Z(k-1) - con1
		   Zp(k) = Zp(k-1) - con2
		 enddo
		 do k=2,nlayrm1
		   Tp(k) = ((Zp(k-1) - Z(k-1))*TI(k-1) +
     $			    (Z(k-1) - Zp(k))*TI(k) )/con2
		 enddo
              endif	

	      do k=1,nlayr
		TI(k)=Tp(k)
              enddo
	
	  endif
       endif

       if((i.eq.iw).and.(j.eq.jw)) then
 	 write(0,*)'vert',nstep,tsurf,hice,
     $	 hsnow,dhi,dhb,snowfx
       debb(37)=TI(1)
       debb(38)=TI(2)
       endif

cccccccccccccccccccccccccccccccccccccc
c  end of vertical temperature balance
cccccccccccccccccccccccccccccccccccccc

       return
       end
