#include "params.h"

/* These are the routines used by the coupler to perform the communication
   between the models-CMS */

#include <stdio.h>
#include "mpi.h"
#include <stdlib.h>
#include <sys/time.h>

extern MPI_Comm split_comm, coupler_ocean;

#ifdef SIMPLE_OUTPUT
extern char storage[100];
extern char prefix[100];
extern char lastoutname[256];
#ifdef MONTHLY
extern int finished;
extern int hsfrq;
#endif
#endif

#ifdef PROFILE
extern FILE *profile_file;
#endif

#   ifdef WORD4
      typedef float real_type;
#   else
      typedef double real_type;
#   endif


#ifdef PROFILE
mark_event_C(evennum, onoff)
int evennum, onoff;

{
mark_event(&evennum,&onoff);
}

mark_event(evennum, onoff)
int *evennum, *onoff;

{
int proc, zonecnt;
static int event, base;
static int times[EVENTS], eventid[EVENTS], mode[EVENTS];
int milliclock();

  if (*onoff < 2) {
    if (event == 0) {
      base = milliclock();
      times[event] = 0;
    }
    else {
    times[event]=milliclock() - base;
    }
    eventid[event]=*evennum;
    mode[event]=*onoff;
    event = event + 1;
  }
  if (*onoff == 2) {
    for (zonecnt=0; zonecnt<event; zonecnt++) {
      fprintf(profile_file,"%7.7d %3.3d %d\n", times[zonecnt], eventid[zonecnt], mode[zonecnt]);
    }
  }
/*
  *evennum=*evennum-1;
  if (*onoff == 1)  
    timers[*evennum]=timers[*evennum] - milliclock();
  if (*onoff == 0)
    timers[*evennum]=timers[*evennum] + milliclock();
  if (*onoff == 2) {
    for (zonecnt=0; zonecnt<ZONES; zonecnt++) {
      fprintf(profile_file,"ZONE %3.3d: %d\n", zonecnt+1, timers[zonecnt]);
    }
  }
    
fprintf(profile_file,"PROFILE: %3.3d %4.4d %d %d\n", proc, *evennum, *onoff, milliclock());
fflush(profile_file); */

}

#endif
/*-----------------------------------------------------------------------
   Procedure receive_from_ocean
      This procedure takes messages sent from the individual ocean 
      nodes and combines them into a large array, placing them where
      they belong geographically.
-----------------------------------------------------------------------*/

#ifdef RS6000
void receive_from_ocean(nlat,nlon,nplat,nplon,nfields,entire,tag)
#else
# ifdef T3E
void RECEIVE_FROM_OCEAN(nlat,nlon,nplat,nplon,nfields,entire,tag)
# else
void receive_from_ocean_(nlat,nlon,nplat,nplon,nfields,entire,tag)
# endif
#endif

int *nlat, *nlon, *nplat, *nplon, *nfields, *tag;
real_type *entire;
{
   int loc2,first,loc,size_array, ierr, procs, i, nlon_proc, nlat_proc;
   int field,lat,lon,wordsz;
   MPI_Status status;
   real_type *data;

#   ifdef WORD4
      wordsz=4;
#   else
      wordsz=8;
#   endif

   nlon_proc=(*nlon)/(*nplon);
   nlat_proc=(*nlat)/(*nplat);

   procs=(*nplat)*(*nplon);
   size_array=nlon_proc*nlat_proc*(*nfields)*wordsz;
   data=malloc(size_array);

   for (i=0; i<procs; i++) {
#ifdef TCP_TEST
nexus_enable_tcp_poll();
#endif
      ierr=MPI_Recv(data,size_array,MPI_CHAR,i,*tag,coupler_ocean,&status);
#ifdef TCP_TEST
nexus_disable_tcp_poll();
#endif

      if (ierr != MPI_SUCCESS) {
         fprintf(stderr,"Error in receive_from_ocean\n");
      }

/* Insert received piece into entire array. Accounts for differences
    in array orderings */ 

      first=(i%(*nplon))*nlon_proc*(*nlat)+((i/(*nplon)))*nlat_proc;

      for (field=1; field<=(*nfields); field++) {
         for (lat=1; lat<=nlat_proc; lat++) {
            for (lon=1; lon<=nlon_proc; lon++) {
               loc=first+(lat-1)+(lon-1)*(*nlat)+(field-1)*(*nlon)*(*nlat); 
               loc2=(lon-1)+(lat-1)*nlon_proc+(field-1)*nlat_proc*nlon_proc;
               *(entire+loc)=*(data+loc2);
            }
         }
      }
   }
free(data);
}



/*-----------------------------------------------------------------------
   Procedure send_to_ocean
      This procedure takes the large array and breaks it up so the
      each ocean processor receives the correct piece.
-----------------------------------------------------------------------*/

#ifdef RS6000
void send_to_ocean(nlat,nlon,nplat,nplon,nfields,entire,tag)
#else
# ifdef T3E
void SEND_TO_OCEAN(nlat,nlon,nplat,nplon,nfields,entire,tag)
# else
void send_to_ocean_(nlat,nlon,nplat,nplon,nfields,entire,tag)
# endif
#endif


int *nlat, *nlon, *nplat, *nplon, *nfields, *tag;
real_type *entire;

{
   int first,procs, field, size_array, ierr, nlon_proc, nlat_proc;
   int lat,lon,i,loc,loc2,wordsz;
   real_type *data;

#   ifdef WORD4
      wordsz=4;
#   else
      wordsz=8;
#   endif

   nlon_proc=*nlon/(*nplon);
   nlat_proc=*nlat/(*nplat);
   procs=(*nplat)*(*nplon);
   data=malloc(wordsz*nlon_proc*nlat_proc*(*nfields));

   /* Pull out piece of entire array. */ 

   for (i=0; i<procs; i++) {
      first=(i%(*nplon))*nlon_proc*(*nlat)+((i/(*nplon)))*nlat_proc;
      for (field=1; field<=(*nfields); field++) {
         for (lat=1; lat<=nlat_proc; lat++) {
            for (lon=1; lon<=nlon_proc; lon++) {
               loc=first+(lat-1)+(lon-1)*(*nlat)+(field-1)*(*nlon)*(*nlat);
               loc2=(lon-1)+(lat-1)*nlon_proc+(field-1)*nlat_proc*nlon_proc;
               *(data+loc2)=*(entire+loc);
            }
         }
      }

      size_array=nlon_proc*nlat_proc*(*nfields)*wordsz;

#ifdef TCP_TEST
nexus_enable_tcp_poll();
#endif
#ifdef COLOCATED_OCEAN
      ierr=MPI_Bsend(data,size_array,MPI_CHAR,i,*tag,coupler_ocean);
#else
      ierr=MPI_Send(data,size_array,MPI_CHAR,i,*tag,coupler_ocean);
#endif
#ifdef TCP_TEST
nexus_disable_tcp_poll();
#endif
      if (ierr != MPI_SUCCESS) {
         fprintf(stderr,"Error in send_to_ocean\n");
      }
   }
free(data);
}




/*----------------------------------------------------------------------
   Procedure reduce_sum
      This procedure uses the call MPI_Reduce with the operator
      MPI_SUM.
-----------------------------------------------------------------------*/
#ifdef RS6000
void reduce_sum(inbuf,outbuf,size)
#else
# ifdef T3E
void REDUCE_SUM(inbuf,outbuf,size)
# else
void reduce_sum_(inbuf,outbuf,size)
# endif
#endif
float *inbuf,*outbuf;
int *size;

{
  int ierr;

  ierr=MPI_Reduce(inbuf,outbuf,(*size),MPI_FLOAT,MPI_SUM,0,split_comm);

  if (ierr != MPI_SUCCESS) {
    fprintf(stderr,"Error in MPI_Reduce\n");
  }

}


/*----------------------------------------------------------------------
    Procedure ireduce_sum
      Does the same as the above, but is for integer arrays.
--------------------------------------------------------------------*/
#ifdef RS6000
void ireduce_sum(inbuf,outbuf,size)
#else
# ifdef T3E
void IREDUCE_SUM(inbuf,outbuf,size)
# else
void ireduce_sum_(inbuf,outbuf,size)
# endif
#endif

float *inbuf,*outbuf;
int *size;

{
  int ierr;

  ierr=MPI_Reduce(inbuf,outbuf,(*size),MPI_INT,MPI_SUM,0,split_comm);

  if (ierr != MPI_SUCCESS) {
    fprintf(stderr,"Error in MPI_Reduce\n");
  }

}




/*--------------------------------------------------------------------
    Procedure barrier0
       This procedure uses the MPI_Barrier command to sync the
       processors.
---------------------------------------------------------------------*/
#ifdef RS6000
void barrier0()
#else
# ifdef T3E
void BARRIER0()
# else
void barrier0_()
# endif
#endif
{
  MPI_Barrier(split_comm);
}

#ifdef RS6000
void barrier1()
#else
# ifdef T3E
void BARRIER1()
# else
void barrier1_()
# endif
#endif
{
  MPI_Barrier(MPI_COMM_WORLD);
}




/*-------------------------------------------------------------------
     Procedure send0
         This procedure is a mapping down to MPI_Send
-------------------------------------------------------------------*/
#ifdef RS6000
void send0( buf, len, type, dest )
#else
# ifdef T3E
void SEND0( buf, len, type, dest )
# else
void send0_( buf, len, type, dest )
# endif
#endif
  char *buf ;
  int *len, *type, *dest ;
{
  int ierr;

  if ( buf == NULL )
  {
    if ( *len == 0 )
    {
      buf = "" ;
    }
    else
    {
      fprintf(stderr,"send0 with null buffer but non zero length\n") ;
      exit(2) ;
    }
  }

  ierr=MPI_Send(buf,*len,MPI_CHAR,*dest,*type,split_comm);

  if (ierr != MPI_SUCCESS)  {
    fprintf(stderr,"error in MPI_Send\n");
    exit(2);
  }
}




/*-------------------------------------------------------------------
     Procedure recv0
        This is a mapping down to MPI_Recv
-------------------------------------------------------------------*/

#ifdef RS6000
void recv0( buf, len, type )
#else
# ifdef T3E
void RECV0( buf, len, type )
# else
void recv0_( buf, len, type )
# endif
#endif
  char *buf ;
  int *len, *type ;
{
  int ierr;
  char xxx[32] ;                /* just a "safe" place */
  MPI_Status status;

  if ( buf == NULL )
  {
    if ( *len == 0 )
    {
      buf = xxx ;
    }
    else
    {
      fprintf(stderr,"recv0 with null buffer but non zero length\n") ;
      exit(2) ;
    }
  }

  ierr=MPI_Recv(buf,*len,MPI_CHAR,MPI_ANY_SOURCE,*type,split_comm,&status);

  if (ierr != MPI_SUCCESS)  {
    fprintf(stderr,"error in MPI_Recv\n");
    exit(2);
  }
}



/*------------------------------------------------------------------
      Procedure bcast0
        This is a mapping down to MPI_Bcast
-------------------------------------------------------------------*/
#ifdef RS6000
void bcast0( buf, len, type, source)
#else
# ifdef T3E
void BCAST0_( buf, len, type, source)
# else
void bcast0_( buf, len, type, source)
# endif
#endif
  char * buf ;
  int *len, *type, *source;
{

  MPI_Bcast(buf,*len,MPI_CHAR,*source,split_comm);

}




#ifdef UPSHOT

/*--------------------------------------------------------------------
    Procedure Log_event
       This procedure is used by the logging mechanism. Used for
       performance studies.
--------------------------------------------------------------------*/
#ifdef RS6000
void log_event(evno,id,name,strlen)
#else
# ifdef T3E
VOID LOG_EVENT(evno,id,name,strlen)
# else
void log_event_(evno,id,name,strlen)
# endif
#endif

int *evno,*id,*strlen;
char name[];

{
  name[*strlen]='\0';
  MPE_Log_event(*evno,*id,name);
}

#endif

/*------------------------------------------------------------------
      Function milliclock
          This function returns the current value of a system clock,
          given in milliseconds.
-----------------------------------------------------------------*/
#include <sys/time.h>
#ifdef RS6000
milliclock()
#else
# ifdef T3E
MILLICLOCK()
# else
milliclock_()
# endif
#endif
{
#ifdef RS6000
    struct timestruc_t tb ;
    int isec ;
    int nsec ;
    int msecs ;

    gettimer(TIMEOFDAY, &tb) ;

    isec = tb.tv_sec ;
    nsec = tb.tv_nsec ;

    msecs = 1000 * isec + nsec / 1000000 ;

    return(msecs) ;
#else
    struct timeval tb ;
    struct timezone tzp ;
    int isec ;  /* seconds */
    int usec ;  /* microseconds */
    int msecs ;

    gettimeofday( &tb, &tzp ) ;
    isec = tb.tv_sec ;
    usec = tb.tv_usec ;

    msecs = 1000 * isec + usec / 1000 ;

    return(msecs) ;

#endif
}


/*------------------------------------------------------------
     Procedure move_atmos_his
        This procedure moves the atmosphere history file to
        unitree, and at the same time gives the file its
        proper name.
-----------------------------------------------------------*/     

#ifdef RS6000
void move_atmos_his(char filname[],int *tape,int *stepno)
#else
# ifdef T3E
void MOVE_ATMOS_HIS(char filname[],int *tape,int *stepno)
# else
void move_atmos_his_(char filname[],int *tape,int *stepno)
# endif
#endif
{
  int suffix, hid;
  char buff[256];
  char dstfile[256];
  char srcfile[256];

#ifdef MONTHLY
  int number, count;

  number = 0;
  for (count=1; count<=4; count = count + 1)
  {
    number = 10 * number + (filname[count] - '0');
  }
  suffix = finished + (number * hsfrq); 
#else
  suffix = ((*stepno - 1)/TSPDAY);
#endif


#ifdef SIMPLE_OUTPUT
  filname[5]='\0';
/*  sprintf(srcfile,"history/atmos/%s",filname);
  sprintf(dstfile,"history/atmos/history.atmos.%7.7d",suffix);
*/
sprintf(srcfile,"history/atmos/%s",filname);
sprintf(dstfile,"history/atmos/history.atmos.%7.7d",suffix);
  sprintf(buff,"mv %s %s",srcfile,dstfile);
  system(buff); 
#endif

}

#ifdef SIMPLE_OUTPUT

/*----------------------------------------------------------------
   Procedure restart_clean
      This procedure moves the completed restarts to unitree if
      it is time. Also, a file is created which tells the script
      that a restart is possible at this point. Finally, the
      atmosphere restarts are catted together and properly renamed.
------------------------------------------------------------------*/

#ifdef RS6000
void restart_clean(int *suffix)
#else
# ifdef T3E
void RESTART_CLEAN(int *suffix)
# else
void restart_clean_(int *suffix)
# endif
#endif
{
char buff[256];
char buff2[256];
char srcfile[256];
char dstfile[256];
FILE *fileno;


/* Cat and correctly name restart files for pccm */

sprintf(buff,"cat ./restart/atmos/*.A.[0-9]* > ./restart/atmos/restart.atmos.%7.7d.A",*suffix);
system(buff);

sprintf(buff,"rm ./restart/atmos/*.A.[0-9]*");
system(buff); 

sprintf(buff,"mv ./restart/atmos/r0* ./restart/atmos/restart.atmos.%7.7d",*suffix);
system(buff);

sprintf(buff,"%s/Restart.%7.7d",prefix,*suffix);
fileno=fopen(buff,"w");
close(fileno);

}

#endif

