#include "params.h"
      subroutine dyndrv(grlps1  ,grt1    ,grq1    ,grz1    ,grd1    ,
     $                  grfu1   ,grfv1   ,grvt1   ,grvq1   ,grrh1   ,
     $                  grlps2  ,grt2    ,grq2    ,grz2    ,grd2    ,
     $                  grfu2   ,grfv2   ,grvt2   ,grvq2   ,grrh2)
C-----------------------------------------------------------------------
C
C Driving routine for Gaussian quadrature, semi-implicit equation
C solution and linear part of horizontal diffusion.
C The need for this interface routine is to have a multitasking
C driver for the spectral space routines it invokes.
C
C-----------------------------------------------------------------------
      implicit none
#include "pmgrid.com"
C---------------------------------------------------------------------
#include "pagrid.com"
C---------------------------------------------------------------------
#include "pspect.com"
C---------------------------------------------------------------------
#include "commap.com"
C---------------------------------------------------------------------
#include "comctl.com"
C---------------------------------------------------------------------
#include "comtim.com"
C---------------------------------------------------------------------
#include "comspe.com"
#ifdef PP_PCCM
#include "comparll.com"
C---------------------------------------------------------------------
#endif
#ifdef INTERVAL_TIMER
#include "timers.com"
#endif
C
C Input arguments
C
#ifdef PP_PCCM
c parallel fft needs full size
      real grlps1(plon,plat/2)
      real grt1(plon,plev,plat/2),
     $     grq1(plon,plev,plat/2),
     $     grz1(plon,plev,plat/2),
     $     grd1(plon,plev,plat/2),
     $     grfu1(plon,plev,plat/2),
     $     grfv1(plon,plev,plat/2),
     $     grvt1(plon,plev,plat/2),
     $     grvq1(plon,plev,plat/2),
     $     grrh1(plon,plev,plat/2)
      real grlps2(plon,plat/2)
      real grt2(plon,plev,plat/2),
     $     grq2(plon,plev,plat/2),
     $     grz2(plon,plev,plat/2),
     $     grd2(plon,plev,plat/2),
     $     grfu2(plon,plev,plat/2),
     $     grfv2(plon,plev,plat/2),
     $     grvt2(plon,plev,plat/2),
     $     grvq2(plon,plev,plat/2),
     $     grrh2(plon,plev,plat/2)
#else
      real grlps1(2*pmmax,plat/2)
      real grt1(2*pmmax,plev,plat/2),
     $     grq1(2*pmmax,plev,plat/2),
     $     grz1(2*pmmax,plev,plat/2),
     $     grd1(2*pmmax,plev,plat/2),
     $     grfu1(2*pmmax,plev,plat/2),
     $     grfv1(2*pmmax,plev,plat/2),
     $     grvt1(2*pmmax,plev,plat/2),
     $     grvq1(2*pmmax,plev,plat/2),
     $     grrh1(2*pmmax,plev,plat/2)
      real grlps2(2*pmmax,plat/2)
      real grt2(2*pmmax,plev,plat/2),
     $     grq2(2*pmmax,plev,plat/2),
     $     grz2(2*pmmax,plev,plat/2),
     $     grd2(2*pmmax,plev,plat/2),
     $     grfu2(2*pmmax,plev,plat/2),
     $     grfv2(2*pmmax,plev,plat/2),
     $     grvt2(2*pmmax,plev,plat/2),
     $     grvq2(2*pmmax,plev,plat/2),
     $     grrh2(2*pmmax,plev,plat/2)
#endif
C
C-----------------------------Local workspace-----------------------------
C
      real ztdtsq(pnmax)
      real zdt,ztdt
      integer k,m,n
C
C Multiply alp,dalp,by gaussian weight and modify alpn and dalpn
C
      zdt = twodt*0.5
      if (nstep.eq.0) zdt = .5*zdt
      ztdt = 2.*zdt
      do 10 n=1,pnmax
         ztdtsq(n) = ztdt*sq(n)
   10 continue
c
c Gaussian quadrature
c
CMIC$ DO ALL SHARED(DOSLT, ZDT, ZTDTSQ, GRLPS1, GRLPS2, GRT1, GRQ1, GRZ1
CMIC$1   , GRD1, GRFU1, GRFV1, GRVT1, GRVQ1, GRRH1, GRT2, GRQ2, GRZ2, 
CMIC$2   GRD2, GRFU2, GRFV2, GRVT2, GRVQ2, GRRH2) PRIVATE(M)
#ifdef PP_PCCM
      do 170 m=m_fwavbeg,m_fwavend
         call quad(m       ,doslt   ,zdt     ,ztdtsq  ,grlps1  ,
     $             grlps2  ,grt1    ,grq1    ,grz1    ,grd1    ,
     $             grfu1   ,grfv1   ,grvt1   ,grvq1   ,grrh1   ,
     $             grt2    ,grq2    ,grz2    ,grd2    ,grfu2   ,
     $             grfv2   ,grvt2   ,grvq2   ,grrh2)
  170 continue
      if (p_nproc_lt.gt.1) then
#ifdef INTERVAL_TIMER
         call interval_timer( 1, TIMER_Dyncolsum, 0 )
#endif
c communication to finish Legendre transform
         call colsum0( alps, r_tmp, m_nspect2,      m_type)
         m_type=m_type+2*p_nproc
         call colsum0( d,    r_tmp, m_nspect2*plev, m_type)
         m_type=m_type+2*p_nproc
         call colsum0( t,    r_tmp, m_nspect2*plev, m_type)
         m_type=m_type+2*p_nproc
         if ( .not. doslt ) then    ! 931021, jm
            call colsum0( q,    r_tmp, m_nspect2*plev, m_type)
            m_type=m_type+2*p_nproc
         endif
         call colsum0( vz,   r_tmp, m_nspect2*plev, m_type)
         m_type=m_type+2*p_nproc
#ifdef INTERVAL_TIMER
         call interval_timer( 0, TIMER_Dyncolsum, 0 )
#endif
      endif

      do m=m_fwavbeg,m_fwavend
         call tstep(m,zdt,ztdtsq)
      enddo

#else
      do 170 m=1,pmmax
         call quad(m       ,doslt   ,zdt     ,ztdtsq  ,grlps1  ,
     $             grlps2  ,grt1    ,grq1    ,grz1    ,grd1    ,
     $             grfu1   ,grfv1   ,grvt1   ,grvq1   ,grrh1   ,
     $             grt2    ,grq2    ,grz2    ,grd2    ,grfu2   ,
     $             grfv2   ,grvt2   ,grvq2   ,grrh2)
 170  continue
c
c Perform time extrapolation: explicit and implicit time step
c
      do m = 1, pmmax
         call tstep(m,zdt,ztdtsq)
      enddo
#endif
C
C Linear horizontal diffusion
C
CMIC$ DO ALL SHARED(ZTDT) PRIVATE(K)
      do 180 k=1,plev
         call hordif(k,ztdt)
  180 continue
      return
      end
