#include "params.h"
      subroutine hdinti(rearth, deltat)
C-----------------------------------------------------------------------
C
C Time independent initialization for the horizontal diffusion.
C
C---------------------------Code history--------------------------------
C
C Original version:  D. Williamson
C Standardized:      J. Rosinski, June 1992
C Reviewed:          B. Boville, J. Hack, August 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pspect.com"
C------------------------------Commons----------------------------------
#include "comhd.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      real rearth,              ! radius of the earth
     $     deltat               ! time step
C
C---------------------------Local workspace-----------------------------
C
      integer
     $     k,                   ! level index
     $     n                    ! n-wavenumber index
C
C-----------------------------------------------------------------------
C
C Top level for del**4 diffusion, set for 18-level model
C
      kmnhd4 = 4
C
C Bottom level for increased del**2 diffusion (kmxhd2 < kmnhd4)
C
      kmxhd2 = 2
C
C Initialize physical constants for courant number based spect truncation
C
      nmaxhd = ptrk
      cnlim  = 0.999          ! maximum allowable Courant number
      cnfac  = deltat*float(nmaxhd)/rearth
C
C Initialize arrays used for courant number based spectral truncation
C
      do k=1,plev
         nindex(k) = 2*nmaxhd
      end do
C
C Set the Del^2 and Del^4 diffusion coefficients for each wavenumber
C
      hdfst2(1) = 0.
      hdfsd2(1) = 0.
C
      hdfst4(1) = 0.
      hdfsd4(1) = 0.
      do n=2,pnmax
         hdfst2(n) = dif2 * (n*(n-1)  ) / rearth**2
         hdfsd2(n) = dif2 * (n*(n-1)-2) / rearth**2

         hdfst4(n) = dif4 * (n*(n-1)*n*(n-1)  ) / rearth**4
         hdfsd4(n) = dif4 * (n*(n-1)*n*(n-1)-4) / rearth**4
      end do
C
      return
      end
