#include "params.h"
      subroutine inidat(doslt   ,ps      ,u3      ,v3      ,t3      ,
     $                  q3      )
C-----------------------------------------------------------------------
C
C Read initial dataset and spectrally truncate as appropriate.
C Initialize work files.
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          D. Williamson, B. Boville, J. Hack, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
C---------------------------Local parameters----------------------------
C
C------------------------------Commons----------------------------------
#include "comcon.com"
C-----------------------------------------------------------------------
#include "comfft.com"
C-----------------------------------------------------------------------
#include "comgrd.com"
C-----------------------------------------------------------------------
#include "comhed.com"
C-----------------------------------------------------------------------
#include "comhyb.com"
C-----------------------------------------------------------------------
#include "comlun.com"
C-----------------------------------------------------------------------
#include "commap.com"
C-----------------------------------------------------------------------
#include "comqfl.com"
C-----------------------------------------------------------------------
#include "comtrcnm.com"
#ifdef PP_PCCM
#include "comparll.com"
#endif
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      logical doslt                       ! slt or spectral transport
C
C Input/Output arguments
C
#ifdef PP_PCCM
      real ps(plond,plat,2),                ! surface pressure
     $     u3(plond,plevd,platd,2,2),       ! u-wind
     $     v3(plond,plevd,platd,2,2),       ! v-wind
     $     t3(plond,plevd,platd,2,2),       ! temperature (K)
     $     q3(plond,plev,3+pcnst,platd,2,2) ! moisture, constituents
#else
      real ps(plond,plat,2),              ! surface pressure
     $     u3(plond,plevd,platd,2),       ! u-wind
     $     v3(plond,plevd,platd,2),       ! v-wind
     $     t3(plond,plevd,platd,2),       ! temperature (K)
     $     q3(plond,plev,3+pcnst,platd,2) ! moisture, constituents
#endif
C
C---------------------------Local workspace-----------------------------
C
      real b2(pflena)             ! out-of-core "a" buffer
      real b1(pflenb),            ! out-of-core "b" buffer
     $     pdelb(plond,plev),     ! pressure diff between interfaces
C                                 ! (pressure defined using the "B" part 
C                                 ! of the hybrid grid only)
     $     hyad (plev)            ! del (A)
#ifdef CRAY
      real work1((plon+1)*pcray)  ! Workspace for fft (Cray)
      real work2(1)
#else
# if (defined(PP_PCCM))
#  if (defined(PP_FFTX))
c     transpose parallel fft work space
      real work1(plond,(3+pcnst)*plev)
      real work2(p_glon+2,(((3+pcnst)*plev)/p_nproc_ln + 1)*2) 
#  else
c     distributed parallel fft work space
      real work1(plond,((3+pcnst)*plev))
      real work2(1)
#  endif
# else
      real work1((plon+1)*(3 + pcnst)*plev)   ! sequential definition
      real work2(1)
# endif
#endif
      real fldsin(phtbf),         ! Initial data
     $     xlat,                  ! latitude index from header
     $     xnlon,                 ! number of longitudes from header
     $     zsqcs                  ! sqrt(cs(irow)), cos(lat)
      integer recl,               ! hist file data record length (words)
     $     i,j,k,                 ! longitude, latitude, level indices
     $     lat,                   ! latitude index (S->N)
     $     irow,                  ! latitude pair index
     $     m,                     ! constituent index
     $     inc,                   ! increment between elements of fft
     $     isign,                 ! (+) or (-) indicts direction of transform
     $     ilev                   ! level index into 1-d array
C
C Space needed in 3 dimensions to store the initial data. This space is
C required because the input data file does not have a predetermined 
C ordering of the latitude records. A  specific order is imposed in the 
C transforms so that the results will be reproducible.
C
      real phis(plond,plat),      ! surface geopotential
     $     pm(plond,plat),        ! meridional derivative of sfc pressure
     $     pl(plond,plat),        ! longitudinal derivative of sfc pressure
     $     vort(plond,plev,plat), ! vorticity
     $     div(plond,plev,plat)   ! divergence
      real pssum,                 ! surface pressure sum
     $     dotproda,              ! dot product
     $     dotprodb               ! dot product
#ifdef PP_PCCM
c differentiate between hemispheres  2=n, 1=s
      integer ih
c other indexing variables
      integer g_lat             ! number of latitude in global grid
                                ! 1 <= g_lat <= p_glat, 1=South, p_glat=North
      integer lath              !
      integer jc                ! index into extended grid for hemisphere
c space needed for reproducible accumulations of tmass, qmass, etc.
      real q3_x_pdela( plon, plat )
      real q3_x_pdelb( plon, plat )

      integer plat3, repro_size
      parameter ( plat3 = 3*plat )
      parameter ( repro_size = max(plat3,plon))
      real repro_work( repro_size )
      real repro_work2( repro_size )

      integer ipack,nrow
      real f_locsum
      external f_locsum
#endif
C
C------------------------------Externals--------------------------------
C
      external spetru             ! Spectrally truncate data

C-----------------------------------------------------------------------
C
C Compute pdel from "A" portion of hybrid vertical grid
C
      do k=1,plev
         hyad(k) = hyai(k+1) - hyai(k)
      end do
      do k=1,plev
         do i=1,plon
            pdela(i,k) = hyad(k)*ps0
         end do
      end do
C
C Zero buffers which get written to ssd
C
      call resetr(b1,pflenb,0.)
      call resetr(b2,pflena,0.)
C
C Test to see if the history file buffer is large enough to hold maxsiz
C the length of a history file record.
C
      recl = maxsiz - 2
      if (phtbf.lt.recl) then
         write(6,*)'INIDAT: Local space not large enough for',
     $             ' initial dataset record'
         write(6,*)'Local parameter phtbf must be at least ',recl
         call endrun
      end if
C
C Loop through initial (grid point) data, saving the data which
C need to be spectrally truncated. Spectrally truncated q will be
C overwritten later if appropriate.
C
      do j=1,plat
#ifdef PP_PCCM
         call rdharr(ninit,recl,xlat,xnlon,fldsin,j,plat)
         g_lat = nint(xlat)
         lath = mod( g_lat-1, p_lato2 ) + 1
         jc = j1m + lath
         if( g_lat .gt. p_glat/2 ) then
            lat = lath + p_lato2
            ih = 2
         else
            lat = lath
            ih = 1
         end if
         irow = lat
         if(lat.gt.plat/2) irow = plat - lat + 1
         call mkslic ('PHIS', phis(1,lat), 1,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
         call mkslic ('U', u3(1,1,jc,ih,1), plev,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
         call mkslic ('V', v3(1,1,jc,ih,1), plev,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
         call mkslic ('T', t3(1,1,jc,ih,1), plev,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
         call mkslic ('Q', q3(1,1,1,jc,ih,1), plev,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
#else
         call rdharr(ninit   ,recl    ,xlat    ,xnlon   ,fldsin)
         lat = nint(xlat)
         irow = lat
         if (lat.gt.plat/2) irow = plat - lat + 1
         call mkslic ('PHIS', phis(1,lat), 1,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
         call mkslic ('U', u3(1,1,lat,1), plev,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
         call mkslic ('V', v3(1,1,lat,1), plev,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
         call mkslic ('T', t3(1,1,lat,1), plev,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
         call mkslic ('Q', q3(1,1,1,lat,1), plev,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
#endif
C
C Set non-water tracers to zero to keep fft happy.  These will get
C initialized later after the fft gets done.
C
         do m=2,pcnst
#ifdef PP_PCCM
            call resetr(q3(1,1,m,jc,ih,1),plndlv,0.)
#else
            call resetr(q3(1,1,m,lat,1),plndlv  ,0.)
#endif
         end do
C
C Compute the quantities which are transformed to spectral space:
C   1. u = u*sqrt(1-mu*mu),   u * cos(phi)
C   2. v = v*sqrt(1-mu*mu),   v * cos(phi)
C   3. t = t-t0,              T prime 
C   4. ps= ln(ps). 
C
         zsqcs = sqrt(cs(irow))
         do k=1,plev
            do i=1,plon
#ifdef PP_PCCM
               u3(i,k,jc,ih,1) = u3(i,k,jc,ih,1)*zsqcs
               v3(i,k,jc,ih,1) = v3(i,k,jc,ih,1)*zsqcs
c              t3(i,k,jc,ih,1) = t3(i,k,jc,ih,1) - t0(k)
#else
               u3(i,k,lat,1) = u3(i,k,lat,1)*zsqcs
               v3(i,k,lat,1) = v3(i,k,lat,1)*zsqcs
c              t3(i,k,lat,1) = t3(i,k,lat,1) - t0(k)
#endif
            end do
         end do
         call mkslic ('PS', ps(1,lat,1), 1,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
         do i=1,plon
            ps(i,lat,1) = alog(ps(i,lat,1))
         end do
      end do         ! j=1,plat
C
C Set some FFT consts (spacing of data points, direction of transform)
C
      inc = 1        ! increment in words between successive data
      isign = -1     ! -1 => transform from gridpoint to Fourier coeff.
C
C Transform grid -> fourier
C 1st transform: U,V,T,Q: note contiguity assumptions
C 2nd transform: LN(PS).  3rd transform: surface geopotential
C
c 8/4/92 loop over latitudes for ffts added.
c allows much smaller working space in fft code.
c the other ffts in the main part of the model are only latitude
c at a time, so it doesn t make much sense to use up this much
c storage here, during initialization.
c
# if defined( PP_PCCM)
      do j  = 1, p_lato2
        call rftlon(u3(1,1,j1m+j,2,1), work1, work2,       !northern
     &              plond, plon, (3+pcnst)*plev, isign)
        call rftlon(u3(1,1,j1m+j,1,1), work1, work2,       !southern
     &              plond, plon, (3+pcnst)*plev, isign)
        call rftlon(ps(1,2*j-1,1), work1, work2, plond, plon, 2, isign)
        call rftlon(phis(1,2*j-1), work1, work2, plond, plon, 2, isign)
      enddo
#   else
      do j  = 1, plat
        call rftlon(u3(1,1,j,1), work1, work2, plond, plon, 
     &              (3+pcnst)*plev, isign)
        call rftlon(ps(1,j,1), work1, work2, plond, plon, 1, isign)
        call rftlon(phis(1,j), work1, work2, plond, plon, 1, isign)
      enddo
#   endif
C
C Spectrally truncate ps and its derivatives, phis, u, v,
C vorticity, divergence, t, and q. Transform phis, ps, vorticity
C and divergence from Fourier space to spectral. Transform back
C from spectral space to obtain Fourier coefficients of desired
C variables.
C
      call spetru(ps      ,phis    ,u3      ,v3      ,t3    ,
     $            q3      ,vort    ,div     ,pl      ,pm    )
C
C Transform Fourier -> grid, obtaining spectrally truncated
C grid point values.
C 1st transform: U,V,T,Q: note contiguity assumptions
C 2nd: ln(PS). 3rd: PHIS. 4th: longitudinal derivative of ln(PS)
C 5th: meridional derivative of ln(PS)
C 6th: vorticity. 7th: divergence
C
      isign = +1      ! +1 => transform from Fourier coeff to gridpoint
C
c 8/4/92 loop over latitudes for ffts added.
c allows much smaller working space in fft code.
c the other ffts in the main part of the model are only latitude
c at a time, so it doesn t make much sense to use up this much
c storage here, during initialization.
#ifdef PP_PCCM
      do j = 1, p_lato2
        call rftlon(u3(1,1,j1m+j,2,1), work1, work2,    
     &              plond, plon, (3+pcnst)*plev, isign)
        call rftlon(u3(1,1,j1m+j,1,1), work1, work2,    
     &              plond, plon, (3+pcnst)*plev, isign)
        call rftlon(ps(1,2*j-1,1), work1, work2, plond, plon, 2, isign)
        call rftlon(phis(1,2*j-1), work1, work2, plond, plon, 2, isign)
        call rftlon(pl(1,2*j-1), work1, work2, plond, plon, 2, isign)
        call rftlon(pm(1,2*j-1), work1, work2, plond, plon, 2, isign)
        call rftlon(vort(1,1,2*j-1), work1, work2, plond, plon, 2*plev,
     &              isign)
        call rftlon(div(1,1,2*j-1), work1, work2, plond, plon, 2*plev,
     &              isign)
      enddo
#else
      do j = 1, plat
        call rftlon(u3(1,1,j,1), work1, work2,    
     &              plond, plon, (3+pcnst)*plev, isign)
        call rftlon(ps(1,j,1), work1, work2, plond, plon, 1, isign)
        call rftlon(phis(1,j), work1, work2, plond, plon, 1, isign)
        call rftlon(pl(1,j), work1, work2, plond, plon, 1, isign)
        call rftlon(pm(1,j), work1, work2, plond, plon, 1, isign)
        call rftlon(vort(1,1,j), work1, work2, plond, plon, plev,
     &              isign)
        call rftlon(div(1,1,j), work1, work2, plond, plon, plev,
     &              isign)
      enddo
#endif
C
C 2nd pass through initial data to obtain and merge all untruncated
C fields:read in and store the data which do not need to be spectrally
C truncated, skipping over header records first.  Also complete
C initialization of /com3d/
C
#ifdef PP_PCCM
      if(m_pin.eq.mapmesh(0,0)) then
         rewind ninit
         do i=1,nrbd
            read(ninit)
         end do
      endif
C
      do 370 j=1,plat
         call rdharr(ninit,recl,xlat,xnlon,fldsin,j,plat)
         g_lat = nint(xlat)
         lath = mod( g_lat-1, p_lato2 ) + 1
         jc = j1m + lath
         if( g_lat .gt. p_glat/2 ) then
            lat = lath + p_lato2
            ih = 2
         else
            lat = lath
            ih = 1
         end if
#else
      rewind ninit
      do i=1,nrbd
         read(ninit)
      end do
C
      do 370 j=1,plat
         call rdharr(ninit   ,recl    ,xlat    ,xnlon   ,fldsin)
         lat = nint(xlat)
#endif
         irow = lat
         if (lat.gt.plat/2) irow = plat - lat + 1
C
C doslt => Start from initial Q, overwriting spectrally truncated Q
C
         if (doslt) then
#ifdef PP_PCCM
            call mkslic('Q',q3(1,1,1,jc,ih,1),plev,
     $           mflds, mcflds, nfldh, fldsin, phtbf,
     $           plon, nlonw, plond, plev)
#else
            call mkslic('Q',q3(1,1,1,lat,1),plev,
     $           mflds, mcflds, nfldh, fldsin, phtbf,
     $           plon, nlonw, plond, plev)
#endif
         end if
C
C Convert U,V to u,v
C Convert perturbation temperature to temperature.
C
         zsqcs = sqrt(cs(irow))
         do k=1,plev
            do i=1,plon
#ifdef PP_PCCM
               u3(i,k,jc,ih,1) = u3(i,k,jc,ih,1)/zsqcs
               v3(i,k,jc,ih,1) = v3(i,k,jc,ih,1)/zsqcs
c              t3(i,k,jc,ih,1) = t3(i,k,jc,ih,1) + t0(k)
#else
               u3(i,k,lat,1) = u3(i,k,lat,1)/zsqcs
               v3(i,k,lat,1) = v3(i,k,lat,1)/zsqcs
c              t3(i,k,lat,1) = t3(i,k,lat,1) + t0(k)
#endif
            end do
         end do
C
C Initialize non-h2o tracers.
C
         do m=2,pcnst
#ifdef PP_PCCM
            call mkslic(tracnam(m),q3(1,1,m,jc,ih,1),plev,
     $           mflds, mcflds, nfldh, fldsin, phtbf,
     $           plon, nlonw, plond, plev)
#else
            call mkslic(tracnam(m),q3(1,1,m,lat,1),plev,
     $           mflds, mcflds, nfldh, fldsin, phtbf,
     $           plon, nlonw, plond, plev)
#endif
C
C To put h2o in tracer fields, use the following "mkslic" call instead of
C the former.
C
Cbds add parallel data structure call here too.
C#ifdef PP_PCCM
C            call mkslic('Q',q3(1,1,m,jc,ih,1),plev,
C     $           mflds, mcflds, nfldh, fldsin, phtbf,
C     $           plon, nlonw, plond, plev)
C#else
C            call mkslic('Q',q3(1,1,m,lat,1),plev,
C     $           mflds, mcflds, nfldh, fldsin, phtbf,
C     $           plon, nlonw, plond, plev)
C#endif
         end do
C
C Ensure sufficient constituent concentration at all gridpoints before 
C calculating global integral.
C
cbds added pccm call to qneg3. plx17 version does not have this call
#ifdef PP_PCCM
         call qneg3('INIDAT  ',lat,q3(1,1,1,jc,ih,1))
#else
         call qneg3('INIDAT  ',lat,q3(1,1,1,lat,1))
#endif
C
C Make both time levels of /com3d/ contain identical data.
C Fields to be convectively adjusted only *require* n3 time
C level since copy gets done in linems.
C
#ifdef PP_PCCM
         call copyac(u3(1,1,jc,ih,1),u3(1,1,jc,ih,2),plndlv)
         call copyac(v3(1,1,jc,ih,1),v3(1,1,jc,ih,2),plndlv)
         call copyac(t3(1,1,jc,ih,1),t3(1,1,jc,ih,2),plndlv)
         do m=1,pcnst
            call copyac(q3(1,1,m,jc,ih,1),q3(1,1,m,jc,ih,2),plndlv)
         enddo
#else
         call copyac(u3(1,1,lat,1),u3(1,1,lat,2),plndlv)
         call copyac(v3(1,1,lat,1),v3(1,1,lat,2),plndlv)
         call copyac(t3(1,1,lat,1),t3(1,1,lat,2),plndlv)
         do m=1,pcnst
            call copyac(q3(1,1,m,lat,1),q3(1,1,m,lat,2),plndlv)
         end do
#endif
C
C Initialization of in-core arrays complete.  Now transfer required
C physical fields into model buffer for nra1 and nrb1 units
C
         do i=1,plon
            b1(ndpslm1+i) = pl(i,lat)
            b1(ndpsmm1+i) = pm(i,lat)
            b2(ndpslp1+i) = pl(i,lat)
            b2(ndpsmp1+i) = pm(i,lat)
         end do
         do k=1,plev
            ilev = (k-1)*plond
            do i=1,plon
               b1(nzm1+ilev+i) = vort(i,k,lat)
               b1(ndm1+ilev+i) = div(i,k,lat)
               b2(nzp1+ilev+i) = vort(i,k,lat)
               b2(ndp1+ilev+i) = div(i,k,lat)
C
               b1(nztl+ilev+i) = vort(i,k,lat)
               b1(ndtl+ilev+i) = div(i,k,lat)
            end do
         end do
         call copyac(phis(1,lat),b1(nphis+1),plon)
         call mkslic ('ORO', b1(noro+1), 1,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
         call mkslic ('TS1', b1(nts+1), 1,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
C
C Set the 4 (sub)surface temperature fields
C
         call mkslic ('TS1', b1(ntssub+1+0*plond), 1,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
         call mkslic ('TS2', b1(ntssub+1+1*plond), 1,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
         call mkslic ('TS3', b1(ntssub+1+2*plond), 1,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
         call mkslic ('TS4', b1(ntssub+1+3*plond), 1,
     $        mflds, mcflds, nfldh, fldsin, phtbf,
     $        plon, nlonw, plond, plev)
C
C Write data to work file units
C
         call writeric(nra1,b2(1),pflena,lat)
         call writeric(nrb1,b1(1),pflenb,lat)
  370 continue                                          ! j=1,plat
C
C Initialize mass and moisture integrals for summation
C in a third calculation loop (assures bit-for-bit compare
C with non-random history tape).
C
      tmassf = 0.
      qmass1 = 0.
      qmass2 = 0.
c
c Loop through old "nrow" so integrals are identical
c Otherwise can just loop lat=1,plat
c
#ifdef PP_PCCM
      call resetr(repro_work,3*plat,   0.)
      call resetr(q3_x_pdela,plon*plat,0.)
      call resetr(q3_x_pdelb,plon*plat,0.)
      ipack = 0
      do  nrow=1,plat

         lat = nrow
         if ( lat .gt. plat/2 ) then
            ih = 2
            jc = j1m + lat - plat/2
         else
            jc = j1m + lat
            ih = 1
         endif

         irow = lat
         if(lat.gt.plat/2) irow = plat - lat + 1
C
C Accumulate average mass
C
         do  i=1,plon
            ps(i,lat,1) = exp(ps(i,lat,1))
         enddo

c        call plevs0(ps(1,lat,1),pint,pmid,pdel)
         call pdelb0(ps(1,lat,1),pdelb)

         ipack = ipack + 1
         repro_work(ipack) =
     $      repro_work(ipack) + f_locsum(ps(1,lat,1),plon,repro_work2)
c         tmassf = tmassf + w(irow)*ssum(plon,ps(1,lat,1),1)
C
C Calculate global integral
C
         do  k=1,plev
            do i=1,plon
                q3_x_pdela(i,lat) = q3_x_pdela(i,lat) +
     $                           q3(i,k,1,jc,ih,1)*pdela(i,k)
                q3_x_pdelb(i,lat) = q3_x_pdelb(i,lat) +
     $                           q3(i,k,1,jc,ih,1)*pdelb(i,k)
c            qmass1 = qmass1 + w(irow)*
c     $              sdot(plon,q3(1,k,1,lat,1),1,pdela(1,k),1)
c            qmass2 = qmass2 + w(irow)*
c     $              sdot(plon,q3(1,k,1,lat,1),1,pdelb(1,k),1)
            enddo
         enddo
         do  i=1,plon
            ps(i,lat,1) = ps(i,lat,1) - psr
         enddo
         call copyac(ps(1,lat,1),ps(1,lat,2),plond)
      enddo

      do nrow=1,plat
          ipack = ipack + 1
          repro_work(ipack) =
     $        f_locsum(q3_x_pdela(1,nrow),plon,repro_work2)
      enddo
      do nrow=1,plat
          ipack = ipack + 1
          repro_work(ipack) =
     $        f_locsum(q3_x_pdelb(1,nrow),plon,repro_work2)
      enddo

      call rowsum0( repro_work, repro_work2, 3*plat, m_type )
      m_type=m_type+2*p_nproc

c multiply latitude weightings in now that all processors have
c contributed in e/w direction
      ipack=0
      do i=1,3
          do lat=1,plat
             irow = lat
             if(lat.gt.plat/2) irow = plat - lat + 1

             ipack=ipack+1
             repro_work(ipack)=repro_work(ipack)*w(irow)
          enddo
      enddo

      repro_work(1) =
     $    f_locsum( repro_work(1)       , plat, repro_work2 )
      repro_work(2) =
     $    f_locsum( repro_work(plat+1), plat, repro_work2 )
      repro_work(3) =
     $    f_locsum( repro_work(2*plat+1)  , plat, repro_work2 )

      call colsum0( repro_work, repro_work2, 3, m_type )
      m_type=m_type+2*p_nproc
      tmassf = repro_work(1)
      qmass1 = repro_work(2)
      qmass2 = repro_work(3)

#else
C
C Compute integrals of mass and moisture
C
      do lat=1,plat
         irow = lat
         if (lat.gt.plat/2) irow = plat - lat + 1
C
C Convert from ln(ps) to ps
C
         do i=1,plon
            ps(i,lat,1) = exp(ps(i,lat,1))
         end do
C
C Accumulate average mass of atmosphere
C
         call pdelb0(ps(1,lat,1),pdelb)
         pssum = 0.
         do i=1,plon
            pssum = pssum + ps(i,lat,1)
         end do
         tmassf = tmassf + w(irow)*pssum
C
C Calculate global integrals needed for water vapor adjustment
C
         do k=1,plev
            dotproda = 0.
            dotprodb = 0.
            do i=1,plon
               dotproda = dotproda + q3(i,k,1,lat,1)*pdela(i,k)
               dotprodb = dotprodb + q3(i,k,1,lat,1)*pdelb(i,k)
            end do
            qmass1 = qmass1 + w(irow)*dotproda
            qmass2 = qmass2 + w(irow)*dotprodb
         end do
C
C Convert to pertubation ps
C
         do i=1,plon
            ps(i,lat,1) = ps(i,lat,1) - psr
         end do
         call copyac(ps(1,lat,1),ps(1,lat,2),plond)
C
      end do      ! latitude loop
#endif
C
C Normalize average mass
C
#ifdef PP_PCCM
      tmassf = tmassf*.5/float(p_glon)
      qmass1 = qmass1*.5/float(p_glon)
      qmass2 = qmass2*.5/float(p_glon)
      qmassf = qmass1 + qmass2
      if ( m_pin .eq. mapmesh(0,0) ) then
         rewind ninit
         close (ninit)
      endif
#else
      tmassf = tmassf*.5/float(plon)
      qmass1 = qmass1*.5/float(plon)
      qmass2 = qmass2*.5/float(plon)
      qmassf = qmass1 + qmass2
      rewind ninit
      close (ninit)
#endif
C
C Set dry mass to which model atmosphere will be constrained.
C
      tmass0 = 98222.
      write(6,800) tmassf,tmass0,qmassf
      return
C
C Formats
C
  800 format('INIDAT: DRY MASS OF INITIAL DATA BEFORE CORRECTION = '
     $        ,1p,e20.10,/,' MASS WILL BE HELD = ',e20.10
     $        ,/,' MASS OF MOISTURE AFTER REMOVAL OF NEGATIVES = '
     $        ,e20.10)
      end
