#include "params.h"
      subroutine mfinti(rair    ,cpair   ,gravit  ,latvap  , rhowtr )
C-----------------------------------------------------------------------
C
C Initialize moist convective mass flux procedure common block, cmfmca 
C 
C----------------------------Code History-------------------------------
C
C Original version:  J. Hack
C Standardized:      J. Rosinski, June 1992
C Reviewed:          J. Hack, G. Taylor, August 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Commons----------------------------------
#include "pmgrid.com"
#include "comcmf.com"
#include "comhyb.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      real rair,             ! gas constant for dry air
     $     cpair,            ! specific heat of dry air
     $     gravit,           ! acceleration due to gravity
     $     latvap,           ! latent heat of vaporization
     $     tmeltd,           ! freezing point of water
     $     wssatd,           ! soil moisture capacity
     $     rhowtr            ! density of liquid water (STP)
      integer k              ! vertical level index

C
C-----------------------------------------------------------------------
C
C Initialize physical constants for moist convective mass flux procedure
C
      cp     = cpair         ! specific heat of dry air     
      hlat   = latvap        ! latent heat of vaporization  
      grav   = gravit        ! gravitational constant       
      rgas   = rair          ! gas constant for dry air
      rhoh2o = rhowtr        ! density of liquid water (STP)
      wssat  = wssatd        ! bucket capacity
      tmelt  = tmeltd        ! freezing point (req-d for rain vs snow)
C
C Initialize free parameters for moist convective mass flux procedure
C
      c0     = 1.0e-4        ! rain water autoconversion coeff (1/m)
      dzmin  = 0.0           ! minimum cloud depth to precipitate (m)
      betamn = 0.10          ! minimum overshoot parameter

C  TIME STEP, convective time scale, 
C  might want to change to 5400 for a time step of 1800 secs
C  as suggested in Williamson, Kielh, and Hack 1994
C  RLJ  Jul 1, 1996

      cmftau = 3600.         ! characteristic adjustment time scale
#if(PP_TRIANG_TRUNC==170)
      cmftau=750.0          ! for t-170
#endif

c
c Limit convection to regions below 40 mb
c
      if (hypi(1) .ge. 4.e3) then
        limcnv = 1
      else
        do k=1,plev
          if (hypi(k).lt.4.e3 .and. hypi(k+1).ge.4.e3) then
            limcnv = k
            goto 10
          end if
        end do
        limcnv = plevp
      end if
   10 write(6,*)'MFINTI: Convection will be capped at intfc ',limcnv,
     $            ' which is ',hypi(limcnv),' pascals'


      tpmax  = 1.50          ! maximum acceptable t perturbation (deg C)
      shpmax = 1.50e-3       ! maximum acceptable q perturbation (g/g)
      rlxclm = .true.        ! logical variable to specify that relaxation
c                                time scale should applied to column as
c                                opposed to triplets individually  
C
C Initialize miscellaneous (frequently used) constants
C
      rhlat  = 1.0/hlat      ! reciprocal latent heat of vaporization  
      rcp    = 1.0/cp        ! reciprocal specific heat of dry air     
      rgrav  = 1.0/grav      ! reciprocal gravitational constant       
C
C Initialize diagnostic location information for moist convection scheme
C
      iloc   = 1             ! longitude point for diagnostic info
      jloc   = 1             ! latitude  point for diagnostic info
      nsloc  = 1             ! nstep value at which to begin diagnostics
C
C Initialize other miscellaneous parameters
C
      tiny   = 1.0e-36       ! arbitrary small number (scalar transport)
      eps    = 1.0e-13       ! convergence criteria (machine dependent) 
C
      return
      end



