#include "params.h"
      subroutine oznini(iunit)
C-----------------------------------------------------------------------
C
C Do initial read of time-variant ozone boundary dataset, containing
C ozone mixing ratios as a function of latitude and pressure.  Read two
C consecutive months between which the current date lies.  Routine
C RADOZ2 then evaluates the two path length integrals (with and without
C pressure weighting) from zero to the interfaces between the input
C levels.  It also stores the contribution to the integral from each
C layer.
C
C---------------------------Code history--------------------------------
C
C Original version:  L. Bath
C Standardized:      L. Buja, June 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pagrid.com"
C------------------------------Commons----------------------------------
#include "comctl.com"
C-----------------------------------------------------------------------
#include "comhdt.com"
C-----------------------------------------------------------------------
#include "comhed.com"
C-----------------------------------------------------------------------
#include "comozp.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#include "crdcon.com"
C-----------------------------------------------------------------------
#ifdef PP_PCCM
#include "pspect.com"
#include "comparll.com"
#endif
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer iunit ! Logical unit number for time-variant ozone dataset
C
C---------------------------Local parameters----------------------------
C
      integer ptvolenr
#ifdef PP_PCCM
      parameter(ptvolenr=3*(2*pnoz + 1) + 2*p_glat)
#else
      parameter(ptvolenr=3*(2*pnoz + 1) + 2*plat)
#endif
C
C---------------------------Local variables-----------------------------
C
      real fldsin(ptvofld*pnoz),    ! Array for data record
     $     hdr(ptvolenr)            ! Array for real header record
      character*8 hdc(ptolencs)     ! Character header record scalars
      integer hdi(ptolenis),        ! Integer header record scalars
     $        mtvfl(3,ptvofld)      ! Mflds for boundary dataset header
      character*8 mctvfl(2,ptvofld) ! Mcflds for boundary dataset header
      integer readl,                ! Length of data buffer - 2  (local)
     $        nlonwl,               ! Number of longitude points (local)
     $        nfldhl,               ! Number of fields on header (local)
     $        ncdatel,              ! Current date               (local)
     $        ncsecl,               ! Current seconds            (local)
     $        mpsigl                ! Pointer to list of levels  (local)
      integer ier,                  ! Error return from rdhdr
     $        n,                    ! Latitude index
     $        k,                    ! Vertical index
     $        tyr,                  ! Year number from current date
     $        lat                   ! Index
      real cday,                    ! Calendar day
     $     tdoy,                    ! Temporary for day of year
     $     flat,                    ! Lat,lon pair read from boundary
     $     flon,                    !  dataset
     $     ndoya                    ! Add to next day after end of year
      logical oznflag               ! Ozone dataset rewind flag
#ifdef PP_PCCM
      character*4 savetype
#endif

C
C------------------------------Externals--------------------------------
C
      external rdhdr,               ! Read dataset header
     $         rdharr,              ! Read dataset record
     $         bnddyi,              ! Day of year calculation
     $         mkslic,              ! "Make a slice" of data
     $         prnthd,              ! Print dataset header
     $         endrun               ! Abnormal termination
C
C-----------------------------------------------------------------------
C
C Assure that a previous month-s data gets read in
C
      ldoyoz = -999.
      ndoya = 0.
      cday = calday
      oznflag = .false.         ! initialize ozone rewind flag
C
C Use year information only if a multiyear dataset
C
      if (.not.ozncyc) cday = float(mcdate/10000)*real(YEAR_LENGTH) + cday
  100 continue
      call rdhdr(iunit   ,ptvoleni,ptolenis,ptvolenc,ptolencs,
     $           ptvolenr,hdi     ,hdc     ,hdr     ,mtvfl   ,
     $           mctvfl  ,ier      )
      if (ier.ne.0) then
         if (ozncyc) then
            if (oznflag) then
C
C  Trap possible infinite rewind loop for improper ozone datasets
C
               write(6,*)'OZNINI: Consecutive rewinds requested on ',
     $                   'ozone dataset.' 
               call prnthd(0   ,hdi(1) ,hdi(31) ,hdi(32)  ,hdi    ,
     $                     hdc ,hdr    ,mtvfl   ,mctvfl   ,.true.)
               call endrun
            else
#ifdef PP_PCCM
            if ( m_pin .eq. mapmesh(0,0) ) then
#endif
               write(6,*)'OZNINI:Rewinding file'
               rewind iunit
#ifdef PP_PCCM
            endif
#endif
               ndoya = real(YEAR_LENGTH)
               oznflag = .true.
               go to 100
            end if
         else
            write(6,*)'OZNINI:Multiyear file assumed. EOF read.'
            call prnthd(0       ,hdi(1)  ,hdi(31) ,hdi(32) ,hdi     ,
     $                  hdc     ,hdr     ,mtvfl   ,mctvfl  ,.true.)
            call endrun
         end if
      end if
c -- need to comment out for our old (PLX17) ozn data set.
c     if (mod(hdi(2),100).ne.43) then  ! not a ccm2 hybrid tape
c        write(6,*)'OZNINI: Ozone dataset is NOT a type 43 dataset.'
c        call prnthd(0   ,hdi(1) ,hdi(31) ,hdi(32)  ,hdi    ,
c    $               hdc ,hdr    ,mtvfl   ,mctvfl   ,.true.)
c        call endrun
c     endif
      oznflag = .false.
      ncdatel = hdi(26)
      write(6,*)'OZNINI:Read ozone data for date (yymmdd) ',ncdatel
      ncsecl = hdi(27)
      tyr = ncdatel/10000
C
C Find day of year from header information
C
      call bnddyi(ncdatel ,ncsecl  ,tdoy)
      if (.not.ozncyc) tdoy = float(tyr)*real(YEAR_LENGTH) + tdoy
  200 continue
#ifdef PP_PCCM
C We need to broadcast the value of ndoya to all of the processors
      call getdataf(savetype)
      call setdataf('real')
      call bcast0f(ndoya, R_SIZE, m_type, mapmesh(0,0) )
      m_type = m_type + 1
      call setdataf(savetype)
#endif
      if (cday.ge.tdoy+ndoya) then
         ndoya = 0.
         ldoyoz = tdoy
         readl  = hdi(6) - 2
         nlonwl = hdi(10)
         nfldhl = hdi(16)
         koz    = hdi(12)
         do n=1,plat
#ifdef PP_PCCM
            call rdharr(iunit,ptvofld*pnoz,flat,flon,fldsin,n,plat)
#else
            call rdharr(iunit   ,readl  ,flat    ,flon    ,fldsin)
#endif
            lat = nint(flat)
#ifdef PP_PCCM
            if ( lat .gt. p_glat/2 ) then  ! north
               lat = plat/2 + mod(lat-1,plat/2) + 1
            else                           ! south
               lat = mod(lat-1,plat/2) + 1
            endif
            call mkslic('OZONE   ',ozmixm(1,1,lat,1),koz   ,
     $        mtvfl   ,mctvfl  ,
     $        nfldhl  ,fldsin  ,ptvofld*pnoz  ,nlonwl  ,nlonwl  ,
     $        nlonwl  ,koz     )
#else
            call mkslic('OZONE   ',ozmixm(1,1,lat,1),koz   ,
     $        mtvfl   ,mctvfl  ,
     $        nfldhl  ,fldsin  ,readl   ,nlonwl  ,nlonwl  ,
     $        nlonwl  ,koz     )
#endif
         end do
         go to 100
      end if
C
C Did previous month get set?
C
      if (ldoyoz.gt.0.) then
C
C Save dataset information for output header
C
         ldhstvos = hdc(22)
         lthstvos = hdc(23)
         lshstvos = hdc(24)
         nyroz    = tyr
         ndoyoz   = tdoy
C
C Test to see if consecutive time slices are too far apart,
C fatal error.
C
         if (ndoyoz.gt.ldoyoz+45.) then
            write(6,*)'OZNINI: Error>>>>>Two consecutive time slices'
            write(6,*)'on ozone dataset are more than 45 days apart.'
            call prnthd(0       ,hdi(1)  ,hdi(31) ,hdi(32) ,hdi     ,
     $                  hdc     ,hdr     ,mtvfl   ,mctvfl  ,.true.  )
            call endrun
         end if
         readl  = hdi(6) - 2
         nlonwl = hdi(10)
         nfldhl = hdi(16)
         do n=1,plat
#ifdef PP_PCCM
            call rdharr(iunit,ptvofld*pnoz,flat,flon,fldsin,n,plat)
            lat = nint(flat)
            if ( lat .gt. p_glat/2 ) then  ! north
               lat = plat/2 + mod(lat-1,plat/2) + 1
            else                           ! south
               lat = mod(lat-1,plat/2) + 1
            endif
            call mkslic('OZONE   ',ozmixm(1,1,lat,2),koz   ,
     $        mtvfl   ,mctvfl  ,
     $        nfldhl  ,fldsin  ,ptvofld*pnoz ,nlonwl  ,nlonwl  ,
     $        nlonwl  ,koz     )
#else
            call rdharr(iunit   ,readl   ,flat    ,flon    ,fldsin  )
            lat = nint(flat)
            call mkslic('OZONE   ',ozmixm(1,1,lat,2),koz   ,
     $        mtvfl   ,mctvfl  ,
     $        nfldhl  ,fldsin  ,readl   ,nlonwl  ,nlonwl  ,
     $        nlonwl  ,koz     )
#endif
         end do
      else
         if (ozncyc) then
            cday = cday + real(YEAR_LENGTH)
            go to 200
         else
            write(6,*)'OZNINI: ',
     $                'All data on ozone dataset is beyond current date'
            call prnthd(0       ,hdi(1)  ,hdi(31) ,hdi(32) ,hdi     ,
     $                  hdc     ,hdr     ,mtvfl   ,mctvfl  ,.true.  )
            call endrun
         end if
      end if
C
C Define the interface pressures
C *** WARNING *** Assume pressure values are TIME-INVARIANT
C
C Change hybrid pressure coefficient values to Pascals.
C
C Double precision is necessary in the following multiplication to ensure
C a bit-for-bit match with the 414 control run.
C
C Ozone data are on the *LEVELS* of the header, not the interfaces.
C
      pin(1) = 0.0

c  This is the PLX22 version
      mpsigl = hdi(33) - 1
      do k=2,koz+1
        pin(k) = hdr(mpsigl+k*2-1)*1.d5
      end do
C
c     do k=1,koz+1
c     write(0,*) k,pin(k)
c     enddo
      return
      end
