#include "params.h"
      subroutine oznint(iunit)
C-----------------------------------------------------------------------
C
C Time interpolate ozone mixing ratios to current time, reading
C in new monthly data if necessary
C
C---------------------------Code history--------------------------------
C
C Original version:  L. Bath
C Standardized:      L. Buja, June 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
#include "comparll.com"
C------------------------------Commons----------------------------------
#include "comctl.com"
C-----------------------------------------------------------------------
#include "comhed.com"
C-----------------------------------------------------------------------
#include "comozp.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer iunit                 ! Logical unit number
C
C---------------------------Local parameters----------------------------
C
      integer ptvolenr
      parameter(ptvolenr=3*(2*pnoz + 1) + 2*plat)
C
C---------------------------Local variables-----------------------------
C
      real fldsin(ptvofld*pnoz),    ! Array for data record
     $     hdr(ptvolenr)            ! Array for real header record
      character*8 hdc(ptolencs)     ! Character header record scalars
      integer hdi(ptolenis),        ! Integer header record scalars
     $        mtvfl(3,ptvofld)      ! Mflds for boundary dataset header
      integer readl,                ! Length of array read
     $        nlonwl,               ! Local copies of header variables
     $        nfldhl,               ! Number of fields in input record
     $        ncdatel,              ! Current date as yymmdd
     $        ncsecl                ! Seconds of day for current date
      integer n,i,j,l,k,            ! Indices
     $        ier,                  ! Error return from rdhdr
     $        lat                   ! Latitude index
      real flat,                    ! Lat,lon pair from boundary dataset
     $     flon,                    ! Lon read from boundary dataset
     $     cday,                    ! Calendar day
     $     fact                     ! Interpolation factor
      character*8 mctvfl(2,ptvofld) ! Mcflds for boundary dataset header
C
C------------------------------Externals--------------------------------
C
      external rdharr,              ! Read dataset record
     $         prnthd,              ! Print dataset header
     $         rdhdr,               ! Read dataset header
     $         bnddyi               ! Day of year calculation
C
C-----------------------------------------------------------------------
C
C Use year information only if a multiyear dataset
C
      cday = calday
      if (.not.ozncyc) cday = float(mcdate/10000)*real(YEAR_LENGTH) + cday
C
C Check to see if we need to read another month-s data
C
      if (ldoyoz.gt.ndoyoz) ndoyoz = ndoyoz + real(YEAR_LENGTH)
      if (cday.lt.ldoyoz) cday = cday + real(YEAR_LENGTH)
      if (cday.gt.ndoyoz) then
         ldoyoz = ndoyoz
         if (ozncyc) then
            ldoyoz = amod(ndoyoz,real(YEAR_LENGTH))
            cday = amod(cday,real(YEAR_LENGTH))
         end if
         do i=1,pnoz
            do j=1,plat
               do l=1,pozlon
                  ozmixm(l,i,j,1) = ozmixm(l,i,j,2)
               end do
            end do
         end do
   10    continue
         call rdhdr(iunit   ,ptvoleni ,ptolenis ,ptvolenc ,ptolencs,
     $              ptvolenr  ,hdi    ,hdc      ,hdr      ,mtvfl    ,
     $              mctvfl    ,ier     )
         if (ier.ne.0) then
            if (ozncyc) then
#ifdef PP_PCCM
               if ( m_pin .eq. mapmesh(0,0) ) then
                 write(6,*)'OZNINT: Rewinding file'
                 rewind iunit
               endif
#else
               write(6,*)'OZNINT: Rewinding file'
               rewind iunit
#endif
               go to 10
            else
               write(6,*)'OZNINT: Multiyear file assumed. EOF read.'
               call prnthd(0       ,hdi(1)  ,hdi(31) ,hdi(32)  ,hdi    ,
     $                     hdc     ,hdr     ,mtvfl   ,mctvfl   ,.true. )
               call blowup
            end if
         end if
         ncdatel = hdi(26)
         write(6,*)'OZNINT: Read ozone data for date (yymmdd) ',ncdatel
         ncsecl = hdi(27)
         nyroz = ncdatel/10000
         call bnddyi(ncdatel  ,ncsecl  ,ndoyoz)
         if (.not.ozncyc) then
            ndoyoz = float(nyroz)*real(YEAR_LENGTH) + ndoyoz
         else if (ndoyoz.lt.ldoyoz) then
            ndoyoz = ndoyoz + real(YEAR_LENGTH)
         end if
C
C Test to see if consecutive time slices are too far apart, fatal error.
C
         if (ndoyoz.gt.ldoyoz+45.) then
            write(6,*)'OZNINT: Error>>>>>Two consecutive time slices ',
     $                'on ozone dataset are more than 45 days apart.'
            call prnthd(0       ,hdi(1)  ,hdi(31) ,hdi(32) ,hdi     ,
     $                  hdc     ,hdr     ,mtvfl   ,mctvfl  ,.true.  )
            call blowup
         end if
         readl = hdi(6) - 2
         nlonwl = hdi(10)
         nfldhl = hdi(16)
         koz = hdi(12)
         do n=1,plat
	    call rdharr(iunit,ptvofld*pnoz,flat,flon,fldsin,n,plat)
            lat = nint(flat)
            if ( lat .gt. p_glat/2 ) then  ! north
               lat = plat/2 + mod(lat-1,plat/2) + 1
            else                           ! south
               lat = mod(lat-1,plat/2) + 1
            endif
            call mkslic('OZONE   ',ozmixm(1,1,lat,2),
     $                                    koz     ,mtvfl   ,mctvfl  ,
     $                  nfldhl  ,fldsin  ,ptvofld*pnoz     ,nlonwl  ,
     $                  nlonwl  ,nlonwl  ,koz     )
         end do
      end if
C
C Linear time interpolation of ozone mixing ratios
C
      fact = (cday - ldoyoz)/(ndoyoz - ldoyoz)
      do k=1,koz
         do j=1,plat
            ozmix(j,k) = ozmixm(1,k,j,1) + fact*
     $                (ozmixm(1,k,j,2) - ozmixm(1,k,j,1))
         end do
      end do
C
C Evaluate the ozone path length integrals to interfaces.
C
      do j=1,plat
         plos0(j,1) = 0.0
         plol0(j,1) = 0.0
      end do
      do k=2,koz+1
         do j=1,plat
            plos0(j,k) = plos0(j,k-1) + cplos*ozmix(j,k-1)*
     $                  (pin(k) - pin(k-1))
            plol0(j,k) = plol0(j,k-1) + cplol*ozmix(j,k-1)*
     $                  (pin(k)*pin(k) - pin(k-1)*pin(k-1))
         end do
      end do
C
      return
      end
