#include "params.h"
      subroutine points
C-----------------------------------------------------------------------
C
C Initialize displacements of grid point variables in Model buffer
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      L. Bath, June 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
C------------------------------Commons----------------------------------
#include "com3d.com"
C-----------------------------------------------------------------------
#include "comgrd.com"
C---------------------------Local variables-----------------------------
C
      integer nbflnb,     ! Total length of nprgtl
     $        nbflna,     ! Length of nprg
     $        nlnbuf1,    ! Length of buffer 1
     $        nlnbuf2,    ! Length of buffer 2
     $        nprgnd,     ! End of nprg
     $        nprgtlnd    ! End of nprgtl
C
C------------------------------Externals--------------------------------
C
      external endrun     ! Abnormal termination
C
C-----------------------------------------------------------------------
C
C Initialize 3-d array pointers
C
      n3m1 = 1
      n3   = 2
C
C Set buffer block pointers using parameter lengths
C
      nprgtl = 0
      nprgm1 = pbflnb
      nprg   = 0
C
C nprgm1, nprgtl and nprg: surface pressure transform first
C
      npsm1 = nprgm1
      npsp1 = nprg
C
      nzm1 = npsm1 + plond
      nztl = nprgtl
      nzp1 = npsp1 + plond
C
C Divergence
C
      ndm1 = nzm1 + plndlv
      ndtl = nztl + plndlv
      ndp1 = nzp1 + plndlv
C
C Horizontal diffusion tendency
C
      nduhm1 = ndm1 + plndlv
      nduhp1 = ndp1 + plndlv
C
      ndvhm1 = nduhm1 + plndlv
      ndvhp1 = nduhp1 + plndlv
C
      ndthm1 = ndvhm1 + plndlv
      ndthp1 = ndvhp1 + plndlv
C
      ndpslm1 = ndthm1 + plndlv
      ndpsmm1 = ndpslm1 + plond
      ndpsm1  = ndpsmm1 + plond
C
C d(ln(pstar))/dlambda (nprg)
C
      ndpslp1 = ndthp1 + plndlv
C
C (1-mu**2)*d(ln(pstar))/dmu (nprg)
C
      ndpsmp1 = ndpslp1 + plond
      ndpsp1  = ndpsmp1 + plond
C
      ndqhm1 = ndpsm1 + plond
      ndqhp1 = ndpsp1 + plond
C
C Set extra pointer to end of nprg for use in finding length nbfl
C
      nprgnd = ndqhp1 + plndlv
C
C nprgtl: orography first
C
      noro = ndtl + plndlv
C
C Surface values for temperature,water,snow
C
      nts = noro + plond
C
C Surface geopotential so no conflict w/ndpsl,ndpsm in INIDAT
C
      nphis = nts + plond
      nws = nphis + plond
      nsn = nws + plond
C
C z0 and fluxes
C
      nrnfp1  = nsn + plond
      nqminus = nrnfp1 + plond
      npsm2   = nqminus + plndlv*pcnst
      num2    = npsm2 + plond
      nvm2    = num2 + plndlv
      ntm2    = nvm2 + plndlv
      nqm2    = ntm2 + plndlv
      npblht  = nqm2 + plndlv*pcnst
      nqpert  = npblht + plond
      ntpert  = nqpert + plond*pcnst
C
C Radiation quantities
C
      nflwds = ntpert + plond
      nfsns = nflwds + plond
      nqrs  = nfsns + plond
      nqrl  = nqrs + plndlv
C
C tssub = instantaneous sub-surface temperatures
C
      ntssub = nqrl + plndlv
C
C Ozone pathlengths
C
      nplol  = ntssub + plond*plevmx
      nplos  = nplol + plndlv + plond
C
C Set extra pointer to end of nprgtl for use in finding length
C
      nprgtlnd = nplos + plndlv + plond
      nbflna   = nprgnd - nprg
      nbflnb   = nprgtlnd - nprgtl
C
C Check buffer section lengths as defined by these pointers against
C lengths defined by parameters nlnbuf1, length of buffer 1
C (Parameter pflenb is padded for unblocked I/O)
C
      nlnbuf1 = ((nbflnb + nbflna)/512 + 1)*512
      if (nlnbuf1.ne.pflenb) then
         write(6,9000)
         write(6,9100) nlnbuf1,pflenb
         call endrun
      end if
C
C nlnbuf2, length of buffer 2 (Parameter pflenb is padded for
C unblocked I/O)
C
      nlnbuf2 = (nbflna/512 + 1)*512
      if (nlnbuf2.ne.pflena) then
         write(6,9000)
         write(6,9200)nlnbuf2,pflena
         call endrun
      end if
C
      return
 9000 format('POINTS: Mismatch between calculated length and parameter')
 9100 format(' NLNBUF1 = ',i10,', PFLENB = ',i10)
 9200 format(' NLNBUF2 = ',i10,', PFLENA = ',i10)
      end
