#include "params.h"
      subroutine sstini(iunit)
C-----------------------------------------------------------------------
C
C Initialize the procedure for specifying sea surface temperatures
C Do initial read of time-varying sst boundary dataset, reading two
C consecutive months on either side of the current model date.
C
C---------------------------Code history--------------------------------
C
C Original version:  L. Bath
C Standardized:      L. Buja, June 1992
C Reviewed:          J. Hack, B. Boville, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pagrid.com"
C------------------------------Commons----------------------------------
#include "comctl.com"
C-----------------------------------------------------------------------
#include "comhdt.com"
C-----------------------------------------------------------------------
#include "comhed.com"
C-----------------------------------------------------------------------
#include "comsst.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#ifdef PP_PCCM
#include "pspect.com"
#include "comparll.com"
#endif
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer iunit  !  Logical unit number for time-variant dataset
C
C---------------------------Local variables-----------------------------
C
      real fldsin(ptvsfld*plon),    ! Array for data record
     $     hdr(plenhr)              ! Real header record
      character*8 hdc(ptslencs)     ! Character header record scalars
      integer hdi(ptslenis),        ! Integer header record scalars
     $        mtvfl(3,ptvsfld)      ! mflds for sst dataset header
      character*8 mctvfl(2,ptvsfld) ! mcflds for sst dataset header
C
      integer readl,                ! Length of data buffer - 2  (local)
     $        nlonwl,               ! Number of longitude points (local)
     $        nfldhl,               ! Number of fields on header (local)
     $        ncdatel,              ! Current date               (local)
     $        ncsecl                ! Current seconds            (local)
      integer ier,                  ! Error flag from rdhdr
     $        n,                    ! Indices
     $        tyr,                  ! Year number from current date
     $        lat                   ! Index
      real cday,                    ! Calendar day
     $     tdoy,                    ! Temporary for day of year
     $     flat,                    ! Lat,lon pair read from boundary
     $     flon,                    !  dataset
     $     ndoya                    ! Add to next day after end of year
      logical sstflag               ! Sst dataset rewind flag
#ifdef PP_PCCM
      character*4 savetype
#endif
C
C------------------------------Externals--------------------------------
C
      external rdhdr,   ! Read dataset header
     $         rdharr,  ! Read dataset record
     $         bnddyi,  ! Day of year calculation
     $         mkslic,  ! "Make a slice" of data
     $         prnthd,  ! Print dataset header
     $         endrun   ! Abnormal termination
C
C-----------------------------------------------------------------------
C
C Assure that a previous month-s data gets read in
C
      ldoy = -999.
      ndoya = 0.
      cday = calday
      sstflag = .false.          ! initialize sst rewind flag
C
C Use year information only if a multiyear dataset
C
      if (.not.sstcyc) cday = float(mcdate/10000)*
     $    real(YEAR_LENGTH) + cday
C
C Read the header for the next time sample on the input data file.
C Rewind the file if the eof is hit and a single year of sst data is
C being recycled.
C
   40 continue
      call rdhdr(iunit   ,ptvsleni,ptslenis,ptvslenc,ptslencs,
     $           plenhr  ,hdi     ,hdc     ,hdr     ,mtvfl   ,
     $           mctvfl  ,ier)
      if (ier.ne.0) then
         if (sstcyc) then
#ifdef PP_PCCM
           if ( m_pin .eq. mapmesh(0,0) ) then
#endif
            if (sstflag) then
C
C  Trap possible infinite rewind loop for improper sst datasets
C
               write(6,*)'SSTINI: Consecutive rewinds requested on sst',
     $                   ' dataset.' 
               call prnthd(0   ,hdi(1) ,hdi(31) ,hdi(32)  ,hdi    ,
     $                     hdc ,hdr    ,mtvfl   ,mctvfl   ,.true.)
               call endrun
            else
               write(6,*)'SSTINI:Rewinding file'
               rewind iunit
               ndoya = real(YEAR_LENGTH)
               sstflag = .true.
               go to 40
            end if
#ifdef PP_PCCM
           endif
#endif
         else
            write(6,*)'SSTINI:Multiyear file assumed. EOF read.'
            call prnthd(0       ,hdi(1)  ,hdi(31) ,hdi(32) ,
     $                  hdi     ,hdc     ,hdr     ,
     $                  mtvfl   ,mctvfl  ,.true.)
            call endrun
         end if
      end if
      sstflag = .false.
      ncdatel = hdi(26)
      write(6,*)'SSTINI:Read sst data for date (yymmdd) ',ncdatel
      ncsecl = hdi(27)
      tyr = ncdatel/10000
C
C Find day of year from header information
C
      call bnddyi(ncdatel ,ncsecl  ,tdoy)
      if (.not.sstcyc) tdoy = float(tyr)*real(YEAR_LENGTH) + tdoy
C
C Read data records and put the sst data into the array for the previous
C month if the current model day is later than the day read from the 
C input file. Then go back and read the next header.
C
   53 continue
#ifdef PP_PCCM
C We need to broadcast the value of ndoya to all of the processors
      call getdataf(savetype)
      call setdataf('real')
      call bcast0f(ndoya, R_SIZE, m_type, mapmesh(0,0) )
      m_type = m_type + 1
      call setdataf(savetype)
#endif
      if (cday.ge.tdoy+ndoya) then
         ndoya = 0.
         ldoy = tdoy
         readl = hdi(6) - 2
         nlonwl = hdi(10)
         nfldhl = hdi(16)
         do 65 n=1,plat
#ifdef PP_PCCM
            call rdharr(iunit,ptvsfld*plon,flat,flon,fldsin,n,plat)
#else
            call rdharr(iunit   ,readl  ,flat    ,flon    ,fldsin)
#endif
            lat = nint(flat)
#ifdef PP_PCCM
            if ( lat .gt. p_glat/2 ) then  ! north
               lat = plat/2 + mod(lat-1,plat/2) + 1
            else                           ! south
               lat = mod(lat-1,plat/2) + 1
            endif
#endif
            call mkslic('SST     ',sstm(1,lat,1),1       ,
     $                     mtvfl   ,mctvfl  ,
     $                  nfldhl  ,fldsin  ,ptvsfld*plon  ,plon    ,
     $                     nlonwl  ,
     $                  plond   ,plev)
   65    continue
         go to 40
      end if
C
C Did previous month get set?
C
      if (ldoy.gt.0.) then
C
C Save dataset information for output header
C
         ldhstvss = hdc(22)
         lthstvss = hdc(23)
         lshstvss = hdc(24)
         nyr = tyr
         ndoy = tdoy
C
C Test to see if consecutive time slices are too far apart, fatal error
C
         if (ndoy.gt.ldoy+45.) then
            write(6,*)'SSTINI:Error>>>>>Two consecutive time slices'
            write(6,*)'on sst dataset are more than 45 days apart'
            call prnthd(0,hdi(1),hdi(31),hdi(32),hdi,hdc,hdr,
     $                  mtvfl,mctvfl,.true.)
            call endrun
         end if
C
C Read the data records and put the sst data into the array for the 
C next month.
C
         readl = hdi(6) - 2
         nlonwl = hdi(10)
         nfldhl = hdi(16)
         do 80 n=1,plat
#ifdef PP_PCCM
            call rdharr(iunit,ptvsfld*plon,flat,flon,fldsin,n,plat)
#else
            call rdharr(iunit   ,readl  ,flat    ,flon    ,fldsin)
#endif
            lat = nint(flat)
#ifdef PP_PCCM
            if ( lat .gt. p_glat/2 ) then  ! north
               lat = plat/2 + mod(lat-1,plat/2) + 1
            else                           ! south
               lat = mod(lat-1,plat/2) + 1
            endif
#endif
            call mkslic('SST     ',sstm(1,lat,2),1       ,
     $                     mtvfl   ,mctvfl  ,
     $                  nfldhl  ,fldsin  ,ptvsfld*plon  ,plon    ,
     $                     nlonwl  ,
     $                  plond   ,plev)
   80    continue
      else
         if (sstcyc) then
            cday = cday + real(YEAR_LENGTH)
            go to 53
         else
            write(6,*)
     $       'SSTINI:All data on sst dataset is beyond current date'
            call prnthd(0,hdi(1),hdi(31),hdi(32),hdi,hdc,hdr,
     $               mtvfl,mctvfl,.true.)
            call endrun
         end if
      end if
C
      return
      end
