#include "params.h"
      subroutine bldfld
C-----------------------------------------------------------------------
C
C Build master field list of all possible fields on history tape
C
C---------------------------Code history--------------------------------
C
C Original version:  L. Bath, April 1992
C Standardized:      L. Bath, June 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pagrid.com"
C------------------------------Commons----------------------------------
#include "comhst.com"
C-----------------------------------------------------------------------
#include "comtrcnm.com"
C---------------------------Local variables-----------------------------
C
      integer if,           !  Field counter
     $        m,j           !  Index
      integer nsingl,       !  Flag for single level field
     $        nmulth,       !  Flag for multilevel field on interfaces
     $        nmultf        !  Flag for multilevel field on Model levels
      integer nactve,       !  Flag to dispose field to history tape
     $        ninact        !  Flag to not dispose field to history tape
      logical lprint        !  Flag for print of full table at end
C
      character*8 fieldns(2,pflds) ! Sorted list of field names
      integer isloccc(0:pichsum),  ! Current location in list
     $        islocdum(0:pichsum), ! Local work array for islocc
     $        nfldscdu(0:pichsum), ! Local work array for nfldsc
     $        ifldss(3,pflds),     ! Sorted list of "iflds"
     $        ipos,                ! Ichar sum
     $        i                    ! Index
C
C-----------------------------------------------------------------------
C
C Set lprint true to get a complete printout of the master field list
C
      lprint = .false.
      nsingl = 0
      nmulth = 1
      nmultf = 2
      nactve = 1
      ninact = 0
C
C Surface geopotential
C
      if = 0
      if = if + 1
      fieldn(1,if) = 'PHIS    '
      fieldn(2,if) = 'M2/S2   '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Surface pressure
C
      if = if + 1
      fieldn(1,if) = 'PS      '
      fieldn(2,if) = 'PA      '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Temperature
C
      if = if + 1
      fieldn(1,if) = 'T       '
      fieldn(2,if) = 'K       '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C U component (east-west) of horizontal velocity
C
      if = if + 1
      fieldn(1,if) = 'U       '
      fieldn(2,if) = 'M/S     '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C V component (north-south) of horizontal velocity
C
      if = if + 1
      fieldn(1,if) = 'V       '
      fieldn(2,if) = 'M/S     '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C Vertical velocity (vertical "eta" velocity)
C
      if = if + 1
      fieldn(1,if) = 'ETADOT  '
      fieldn(2,if) = '1/S     '
      iflds(1,if)  = nmulth
      iflds(3,if)  = ninact
C
C Constituent fields:
C First field is specific humidity
C
      do 10 m=1,pcnst
         if = if + 1
         fieldn(1,if) = tracnam(m)
         fieldn(2,if) = 'KG/KG   '
         iflds(1,if)  = nmultf
         iflds(3,if)  = nactve
C
C Remainder of fields in do 10 loop are SLT diagnostics
C Horizontal advection of constituent fields
C
         if = if + 1
         fieldn(1,if) = hadvnam(m)
         fieldn(2,if) = 'KG/KGS  '
         iflds(1,if)  = nmultf
         iflds(3,if)  = ninact
C
C Vertical advection of constituent fields
C
         if = if + 1
         fieldn(1,if) = vadvnam(m)
         fieldn(2,if) = 'KG/KGS  '
         iflds(1,if)  = nmultf
         iflds(3,if)  = ninact
C
C Conservation fixer for constituent fields
C
         if = if + 1
         fieldn(1,if) = fixcnam(m)
         fieldn(2,if) = 'KG/KGS  '
         iflds(1,if)  = nmultf
         iflds(3,if)  = ninact
C
C SLT constituent tendency (advection + fixer)
C
         if = if + 1
         fieldn(1,if) = tottnam(m)
         fieldn(2,if) = 'KG/KGS  '
         iflds(1,if)  = nmultf
         iflds(3,if)  = nactve
C
C Vertical diffusion of constituents
C
         if = if + 1
         fieldn(1,if) = vdiffnam(m)
         fieldn(2,if) = 'KG/KGS  '
         iflds(1,if)  = nmultf
         iflds(3,if)  = nactve
C
C Convective adjustment of constituents
C
         if = if + 1
         fieldn(1,if) = dcconnam(m)
         fieldn(2,if) = 'KG/KGS  '
         iflds(1,if)  = nmultf
         iflds(3,if)  = nactve
C
C Total constituent tendency
C
         if = if + 1
         fieldn(1,if) = tendnam(m)
         fieldn(2,if) = 'KG/KGS  '
         iflds(1,if)  = nmultf
         iflds(3,if)  = ninact
   10 continue
C
C Constituent sources/sinks
C
      do 20 m=2,pcnst
         if = if + 1
         fieldn(1,if) = srcnam(m)
         fieldn(2,if) = 'KG/KGS  '
         iflds(1,if)  = nmultf
         iflds(3,if)  = ninact
C
C Constituent surface fluxes
C
         if = if + 1
         fieldn(1,if) = sflxnam(m)
         fieldn(2,if) = 'KG/M2/S '
         iflds(1,if)  = nsingl
         iflds(3,if)  = ninact
   20 continue
C
C U horizontal diffusive heating
C
      if = if + 1
      fieldn(1,if) = 'DUH     '
      fieldn(2,if) = 'K/S     '
      iflds(1,if)  = nmultf
      iflds(3,if)  = ninact
C
C V horizontal diffusive heating
C
      if = if + 1
      fieldn(1,if) = 'DVH     '
      fieldn(2,if) = 'K/S     '
      iflds(1,if)  = nmultf
      iflds(3,if)  = ninact
C
C T horizontal diffusive heating
C
      if = if + 1
      fieldn(1,if) = 'DTH     '
      fieldn(2,if) = 'K/S     '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C Q horizontal diffusion
C
      if = if + 1
      fieldn(1,if) = 'NDQH    '
      fieldn(2,if) = 'KG/KGS  '
      iflds(1,if)  = nmultf
      iflds(3,if)  = ninact
C
C Land, ocean, sea ice mask
C
      if = if + 1
      fieldn(1,if) = 'ORO     '
      fieldn(2,if) = 'FLAG    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Soil moisture
C
      if = if + 1
      fieldn(1,if) = 'WET     '
      fieldn(2,if) = 'M       '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Water equivalent snow depth
C
      if = if + 1
      fieldn(1,if) = 'SNOWH   '
      fieldn(2,if) = 'M       '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Large-scale (stable) precipitation rate
C
      if = if + 1
      fieldn(1,if) = 'PRECL   '
      fieldn(2,if) = 'M/S     '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Convective precipitation rate

      if = if + 1
      fieldn(1,if) = 'PRECC   '
      fieldn(2,if) = 'M/S     '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Large-scale (stable) snow rate (water equivalent)
C
      if = if + 1
      fieldn(1,if) = 'PRECSL  '
      fieldn(2,if) = 'M/S     '
      iflds(1,if)  = nsingl
      iflds(3,if)  = ninact
C
C Convective snow rate (water equivalent)
C
      if = if + 1
      fieldn(1,if) = 'PRECSC  '
      fieldn(2,if) = 'M/S     '
      iflds(1,if)  = nsingl
      iflds(3,if)  = ninact
C
C Surface sensible heat flux
C
      if = if + 1
      fieldn(1,if) = 'SHFLX   '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Surface latent heat flux
C
      if = if + 1
      fieldn(1,if) = 'LHFLX   '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Surface water flux
C
      if = if + 1
      fieldn(1,if) = 'QFLX    '
      fieldn(2,if) = 'KG/M2/S '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C PBL height
C
      if = if + 1
      fieldn(1,if) = 'PBLH    '
      fieldn(2,if) = 'M       '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Surface friction velocity
C
      if = if + 1
      fieldn(1,if) = 'USTAR   '
      fieldn(2,if) = 'M/S     '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C 10-meter potential temperature
C
      if = if + 1
      fieldn(1,if) = 'T10     '
      fieldn(2,if) = 'K       '
      iflds(1,if)  = nsingl
      iflds(3,if)  = ninact
C
C 10-meter specific humidity
C
      if = if + 1
      fieldn(1,if) = 'Q10     '
      fieldn(2,if) = 'KG/KG   '
      iflds(1,if)  = nsingl
      iflds(3,if)  = ninact
C
C Counter-gradient term for heat in PBL
C
      if = if + 1
      fieldn(1,if) = 'CGH     '
      fieldn(2,if) = 'K/M     '
      iflds(1,if)  = nmulth
      iflds(3,if)  = ninact
C
C Counter-gradient term for moisture in PBL
C
      if = if + 1
      fieldn(1,if) = 'CGQ     '
      fieldn(2,if) = '1/M     '
      iflds(1,if)  = nmulth
      iflds(3,if)  = ninact
C
C Counter-gradient coefficient on surface kinematic fluxes
C
      if = if + 1
      fieldn(1,if) = 'CGS     '
      fieldn(2,if) = 'S/M2    '
      iflds(1,if)  = nmulth
      iflds(3,if)  = ninact
C
C Perturbation temperature (eddies in PBL)
C
      if = if + 1
      fieldn(1,if) = 'TPERT   '
      fieldn(2,if) = 'K       '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Perturbation specific humidity (eddies in PBL)
C
      if = if + 1
      fieldn(1,if) = 'QPERT   '
      fieldn(2,if) = 'KG/KG   '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Vertical diffusion diffusivities (heat/moisture)
C
      if = if + 1
      fieldn(1,if) = 'KVH     '
      fieldn(2,if) = 'M2/S    '
      iflds(1,if)  = nmulth
      iflds(3,if)  = ninact
C
C Vertical diffusion diffusivities (momentum)
C
      if = if + 1
      fieldn(1,if) = 'KVM     '
      fieldn(2,if) = 'M2/S    '
      iflds(1,if)  = nmulth
      iflds(3,if)  = ninact
C
C U vertical diffusion
C
      if = if + 1
      fieldn(1,if) = 'DUV     '
      fieldn(2,if) = 'M/S2    '
      iflds(1,if)  = nmultf
      iflds(3,if)  = ninact
C
C V vertical diffusion
C
      if = if + 1
      fieldn(1,if) = 'DVV     '
      fieldn(2,if) = 'M/S2    '
      iflds(1,if)  = nmultf
      iflds(3,if)  = ninact
C
C T vertical diffusion
C
      if = if + 1
      fieldn(1,if) = 'DTV     '
      fieldn(2,if) = 'K/S     '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C Net solar flux at surface
C
      if = if + 1
      fieldn(1,if) = 'FSNS    '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Net longwave flux at surface
C
      if = if + 1
      fieldn(1,if) = 'FLNS    '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Net longwave flux at top
C
      if = if + 1
      fieldn(1,if) = 'FLNT    '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Net solar flux at top
C
      if = if + 1
      fieldn(1,if) = 'FSNT    '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Cloud fraction
C
      if = if + 1
      fieldn(1,if) = 'CLOUD   '
      fieldn(2,if) = 'FRACTION'
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C Specified liquid water path lengths
C
      if = if + 1
      fieldn(1,if) = 'SETLWP  '
      fieldn(2,if) = 'G/M2    '
      iflds(1,if)  = nmultf
      iflds(3,if)  = ninact
C
C Actual cloud liquid water path length (accounting for cloud fraction)
C
      if = if + 1
      fieldn(1,if) = 'CLDLWP  '
      fieldn(2,if) = 'G/M2    '
      iflds(1,if)  = nmultf
      iflds(3,if)  = ninact
C
C "Effective" cloud fraction
C
      if = if + 1
      fieldn(1,if) = 'EFFCLD  '
      fieldn(2,if) = 'FRACTION'
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C Clearsky net longwave flux at top
C
      if = if + 1
      fieldn(1,if) = 'FLNTC   '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Clearsky net solar flux at top
C
      if = if + 1
      fieldn(1,if) = 'FSNTC   '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Clearsky net longwave flux at surface
C
      if = if + 1
      fieldn(1,if) = 'FLNSC   '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Clearsky net solar flux at surface
C
      if = if + 1
      fieldn(1,if) = 'FSNSC   '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Vertical pressure velocity
C
      if = if + 1
      fieldn(1,if) = 'OMEGA   '
      fieldn(2,if) = 'PA/S    '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C Specific humidity tendency due to precipitation
C
      if = if + 1
      fieldn(1,if) = 'DQP     '
      fieldn(2,if) = 'KG/KGS  '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C X-component (east-west) of surface stress
C
      if = if + 1
      fieldn(1,if) = 'TAUX    '
      fieldn(2,if) = 'N/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Y-component (north-south) of surface stress
C
      if = if + 1
      fieldn(1,if) = 'TAUY    '
      fieldn(2,if) = 'N/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Net radiative flux at surface
C
      if = if + 1
      fieldn(1,if) = 'SRFRAD  '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Solar heating rate
C
      if = if + 1
      fieldn(1,if) = 'QRS     '
      fieldn(2,if) = 'K/S     '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C Longwave heating rate
C
      if = if + 1
      fieldn(1,if) = 'QRL     '
      fieldn(2,if) = 'K/S     '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C Vertically-integrated, random overlap, total cloud amount
C
      if = if + 1
      fieldn(1,if) = 'CLDTOT  '
      fieldn(2,if) = 'FRACTION'
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Vertically-integrated, random overlap, low cloud amount
C
      if = if + 1
      fieldn(1,if) = 'CLDLOW  '
      fieldn(2,if) = 'FRACTION'
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Vertically-integrated, random overlap, mid-level cloud amount
C
      if = if + 1
      fieldn(1,if) = 'CLDMED  '
      fieldn(2,if) = 'FRACTION'
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Vertically-integrated, random overlap, high cloud amount
C
      if = if + 1
      fieldn(1,if) = 'CLDHGH  '
      fieldn(2,if) = 'FRACTION'
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Vertically-integrated actual liquid water path length
C
      if = if + 1
      fieldn(1,if) = 'TOTLWP  '
      fieldn(2,if) = 'FRACTION'
      iflds(1,if)  = nsingl
      iflds(3,if)  = ninact
C
C Subsoil temperatures - top level is surface temperature
C
      do 30 m=1,plevmx
         if = if + 1
         fieldn(1,if) = tsnam(m)
         fieldn(2,if) = 'K       '
         iflds(1,if)  = nsingl
         iflds(3,if)  = nactve
   30 continue
C
C Solar insulation
C
      if = if + 1
      fieldn(1,if) = 'SOLIN   '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C U tendency
C
      if = if + 1
      fieldn(1,if) = 'UTEND   '
      fieldn(2,if) = 'M/S2    '
      iflds(1,if)  = nmultf
      iflds(3,if)  = ninact
C
C V tendency
C
      if = if + 1
      fieldn(1,if) = 'VTEND   '
      fieldn(2,if) = 'M/S2    '
      iflds(1,if)  = nmultf
      iflds(3,if)  = ninact
C
C T tendency
C
      if = if + 1
      fieldn(1,if) = 'TTEND   '
      fieldn(2,if) = 'K/S     '
      iflds(1,if)  = nmultf
      iflds(3,if)  = ninact
C
C Surface pressure tendency
C
      if = if + 1
      fieldn(1,if) = 'LPSTEN  '
      fieldn(2,if) = 'PA/S    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = ninact
C
C U tendency - gravity wave drag
C
      if = if + 1
      fieldn(1,if) = 'UTGW    '
      fieldn(2,if) = 'M/S2    '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C V tendency - gravity wave drag
C
      if = if + 1
      fieldn(1,if) = 'VTGW    '
      fieldn(2,if) = 'M/S2    '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C East-west gravity wave drag surface stress
C
      if = if + 1
      fieldn(1,if) = 'TAUGWX  '
      fieldn(2,if) = 'N/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C North-south gravity wave drag surface stress
C
      if = if + 1
      fieldn(1,if) = 'TAUGWY  '
      fieldn(2,if) = 'N/M2    '
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C T tendency - convective adjustment
C
      if = if + 1
      fieldn(1,if) = 'DTCOND  '
      fieldn(2,if) = 'K/S     '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C T tendency - moist convection
C
      if = if + 1
      fieldn(1,if) = 'CMFDT   '
      fieldn(2,if) = 'K/S     '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C Q tendency - moist convection
C
      if = if + 1
      fieldn(1,if) = 'CMFDQ   '
      fieldn(2,if) = 'KG/KGS  '
      iflds(1,if)  = nmultf
      iflds(3,if)  = nactve
C
C Q tendency - moist convective rainout
C
      if = if + 1
      fieldn(1,if) = 'CMFDQR  '
      fieldn(2,if) = 'KG/KGS  '
      iflds(1,if)  = nmultf
      iflds(3,if)  = ninact
C
C Moist convection mass flux
C
      if = if + 1
      fieldn(1,if) = 'CMFMC   '
      fieldn(2,if) = 'KG/M2S  '
      iflds(1,if)  = nmulth
      iflds(3,if)  = nactve
C
C Moist convection liquid water static energy flux
C
      if = if + 1
      fieldn(1,if) = 'CMFSL   '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nmulth
      iflds(3,if)  = nactve
C
C Moist convection total water flux
C
      if = if + 1
      fieldn(1,if) = 'CMFLQ   '
      fieldn(2,if) = 'W/M2    '
      iflds(1,if)  = nmulth
      iflds(3,if)  = nactve
C
C Random overlap total convective cloud amount
C
      if = if + 1
      fieldn(1,if) = 'CNVCLD  '
      fieldn(2,if) = 'FRACTION'
      iflds(1,if)  = nsingl
      iflds(3,if)  = nactve
C
C Fields which go out on monthly average history tapes
C
C Meridional heat transport
C
      if = if + 1
      fieldn(1,if) = 'VT      '
      fieldn(2,if) = 'KM/S    '
      iflds(1,if)  = nmultf
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C Meridional transport
C
      if = if + 1
      fieldn(1,if) = 'VZ      '
      fieldn(2,if) = 'M3/S3   '
      iflds(1,if)  = nmultf
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C Meridional water transport
C
      if = if + 1
      fieldn(1,if) = 'VQ      '
      fieldn(2,if) = 'M/S     '
      iflds(1,if)  = nmultf
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C Kinetic Energy
C
      if = if + 1
      fieldn(1,if) = 'VVPUU   '
      fieldn(2,if) = 'M2/S2   '
      iflds(1,if)  = nmultf
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C Albedo
C
      if = if + 1
      fieldn(1,if) = 'ALB     '
      fieldn(2,if) = 'FRACTION'
      iflds(1,if)  = nsingl
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C Clear sky albedo
C
      if = if + 1
      fieldn(1,if) = 'ALBCLR  '
      fieldn(2,if) = 'FRACTION'
      iflds(1,if)  = nsingl
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = ninact
      else
         iflds(3,if)  = ninact
      end if
C
C Relative humidity
C
      if = if + 1
      fieldn(1,if) = 'RELHUM  '
      fieldn(2,if) = 'FRACTION'
      iflds(1,if)  = nmultf
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C Geopotential Height (above sea level)
C
      if = if + 1
      fieldn(1,if) = 'Z2      '
      fieldn(2,if) = 'M       '
      iflds(1,if)  = nmultf
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C water mass
C
      if = if + 1
      fieldn(1,if) = 'MQ      '
      fieldn(2,if) = 'KG/M2   '
      iflds(1,if)  = nmultf
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C Sea level pressure
C
      if = if + 1
      fieldn(1,if) = 'PSL     '
      fieldn(2,if) = 'PASCALS '
      iflds(1,if)  = nsingl
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C Average of Omega when omega is < 0 (up)
C
      if = if + 1
      fieldn(1,if) = 'OMEGAUP '
      fieldn(2,if) = 'PA/S    '
      iflds(1,if)  = nmultf
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C Percentage of time omega is < 0 (up)
C
      if = if + 1
      fieldn(1,if) = 'NUMOMGUP'
      fieldn(2,if) = 'FRACTION'
      iflds(1,if)  = nmultf
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C Average of Cloud when omega is < 0 (up)
C
      if = if + 1
      fieldn(1,if) = 'CLOUDUP '
      fieldn(2,if) = 'FRACTION'
      iflds(1,if)  = nmultf
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C Longitude Pressure Gradient
C
      if = if + 1
      fieldn(1,if) = 'DPSLON  '
      fieldn(2,if) = 'M/S2    '
      iflds(1,if)  = nmultf
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C Latitude Pressure Gradient
C
      if = if + 1
      fieldn(1,if) = 'DPSLAT  '
      fieldn(2,if) = 'M/S2    '
      iflds(1,if)  = nmultf
      if (ninavg(1).eq.'Q') then
         iflds(3,if)  = nactve
      else
         iflds(3,if)  = ninact
      end if
C
C Sort the master field list by summation of ichar-s of 1st 4 characters
C pichsum = 127*4.  Minimum ichar output is 0. Maximum is 127.
C
      islocdum(0) = 0
      isloccc(0) = 0
      do i=0,pichsum
         nfldscdu(i) = 0
      end do
C
C Count up number of fields with a given sum of 4 ichar-s
C
      do i=1,if
         ipos = ichar(fieldn(1,i)(1:1)) + ichar(fieldn(1,i)(2:2)) +
     $          ichar(fieldn(1,i)(3:3)) + ichar(fieldn(1,i)(4:4))
         nfldscdu(ipos) = nfldscdu(ipos) + 1
      end do
C
C Define starting location for each sorted sum
C
      do i=1,pichsum
         islocdum(i) = islocdum(i-1) + nfldscdu(i-1)
         isloccc(i) = islocdum(i)
      end do
C
C Sort the hashed field names by increasing sum
C
      do i=1,if
         ipos = ichar(fieldn(1,i)(1:1)) + ichar(fieldn(1,i)(2:2)) +
     $          ichar(fieldn(1,i)(3:3)) + ichar(fieldn(1,i)(4:4))
         isloccc(ipos) = isloccc(ipos) + 1
         fieldns(1,isloccc(ipos)) = fieldn(1,i)
         fieldns(2,isloccc(ipos)) = fieldn(2,i)
         ifldss(1,isloccc(ipos)) = iflds(1,i)
         ifldss(3,isloccc(ipos)) = iflds(3,i)
      end do
C
C Copy the local sorted list into the permanent list
C
      do i=1,if
         fieldn(1,i) = fieldns(1,i)
         fieldn(2,i) = fieldns(2,i)
         iflds(1,i) = ifldss(1,i)
         iflds(3,i) = ifldss(3,i)
      end do
C
C Check number of fields against max for master list.
C
      if (if.gt.pflds) then
         write(6,*)'BLDFLD: Too many fields -- pflds,if=',pflds,if
         call endrun
      end if
      if (lprint) then
         write(6,*)' ******* MASTER FIELD LIST *******'
         do j=1,if
            write(6,9000)j,fieldn(1,j),iflds(1,j),iflds(3,j),
     $                   fieldn(2,j)
         end do
      end if
C
      return
 9000 format (i5,8x,a8,i3,i3,5x,a8)
      end
