#include "params.h"
      subroutine fldlst(lhbuf)
C-----------------------------------------------------------------------
C
C Initialize each history tape field
C
C---------------------------Code history--------------------------------
C
C Original version:  L. Bath 
C Standardized:      L. Bath, June 1992
C Reviewed:          B. Boville, P. Rasch, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pagrid.com"
C------------------------------Commons----------------------------------
#include "comhed.com"
C-----------------------------------------------------------------------
#include "comhst.com"
C-----------------------------------------------------------------------
#include "comtrcnm.com"
C------------------------------Arguments--------------------------------
C
C Output argument
C
      integer lhbuf            ! Length of hbuf
C
C---------------------------Local variables-----------------------------
C
      integer it               !  Starting tape number
      character*1 ninst        !  Codes for instantaneous or averaged fields
      character*1 avg          !  Temporary for averaging flag
      character*8 fldtmp       !  Temporary for field name
      character*8 fldname(2)   !  8byte field names/units
      integer if,              !  Index into master field list for each field
     $        ifex,            !  Index into exclude fields
     $        ifin,            !  Index into include fields (primary)
     $        i,               !  Index
     $        kfld,            !  Index into mflds,mcflds,etc. for each field
     $        kkfld,           !  Index like kfld, but for aux. tapes
     $        kt               !  Index over mtapes
      integer ipos,iascii
      integer imlist,
     $        iaux,
     $        itape
C
C------------------------------Externals--------------------------------
C
      external inifld,         ! Initialize field list information for 1 field
     $         endrun,         ! Abnormal termination
     $         ljust
C
C Set flags used to specify whether a field should be output as an
C instantaneous value, time averaged over the output interval, or as the
C minimum (maximum) value over the output interval. These flags are used
C to explicitly specify the type for each individual field.
C
      ninst = 'I'
C
C Initialize counters
C

C Modification for parallel code: we always reinitialize from the
C namelist, regardless of what mode run it is.

      mtapes = 0

      if (mtapes.eq.0) then
C
C Initialize "number of fields per ichar sum" array to zero
C
         do it=1,ptapes
            do iascii=0,pichsum
               nfldsc(iascii,it) = 0
            end do
         end do
         mtapes = 1
         nflds(mtapes) = 0
         nrlen(mtapes) = 0
         nplen(mtapes) = 0
         kfld = 1
C
C Primary history tape. Call inifld for all active fields. The exclude 
C list is used to specify which fields on the master list of possible 
C output fields will not be written to the history file. All fields 
C except 'PHIS' and 'ORO' are time averaged.
C
         do 10 if=1,pflds
C
C Search the exclude list
C
            do ifex = 1,nexcl
               call ljust(exclude(ifex),fldtmp)
               if (fieldn(1,if).eq.fldtmp) go to 10
            end do
C
C Now process "primary" requested fields
C
            do ifin = 1,nincl
               call ljust(primary(ifin),fldtmp)
               if (fieldn(1,if).eq.fldtmp) then
                  if (fieldn(1,if).eq.'PHIS    '.or.
     $                fieldn(1,if).eq.'ORO     ') then
                     avg = ninst
                  else
                     avg = ninavg(mtapes)
                  end if
                  fldname(1)=fieldn(1,if)(1:8)
                  fldname(2)=fieldn(2,if)(1:8)
                  call inifld(fldname,iflds(1,if)  ,avg     ,
     $                          hbufpt  ,ndens(1),
     $                        mtapes  ,nrlen(mtapes) ,nplen(mtapes),
     $                          nflds(mtapes) ,mflds(1,kfld) ,
     $                        mcflds(1,kfld),nupnt(kfld,mtapes),
     $                          npnt(kfld,mtapes))
C
C Add one to the count of active field names starting with a given ichar sum
C
                  ipos = ichar(mcflds(1,kfld)(1:1)) +
     $                   ichar(mcflds(1,kfld)(2:2)) +
     $                   ichar(mcflds(1,kfld)(3:3)) +
     $                   ichar(mcflds(1,kfld)(4:4))
                  nfldsc(ipos,mtapes) = nfldsc(ipos,mtapes) + 1
                  kfld = kfld + 1
                  go to 10
               end if
            end do
C
C Search master list for default active fields
C
            if (iflds(3,if).eq.1) then
               if (fieldn(1,if).eq.'PHIS    '.or.
     $             fieldn(1,if).eq.'ORO     ') then
                  avg = ninst
               else
                  avg = ninavg(mtapes)
               end if
               fldname(1)=fieldn(1,if)(1:8)
               fldname(2)=fieldn(2,if)(1:8)
               call inifld(fldname,iflds(1,if)  ,avg     ,
     $                       hbufpt  ,ndens(1),
     $                     mtapes  ,nrlen(mtapes) ,nplen(mtapes),
     $                       nflds(mtapes) ,mflds(1,kfld),
     $                     mcflds(1,kfld),nupnt(kfld,mtapes),
     $                       npnt(kfld,mtapes))
C
C Add one to the count of active field names starting with a given ichar sum
C
               ipos = ichar(mcflds(1,kfld)(1:1)) +
     $                ichar(mcflds(1,kfld)(2:2)) +
     $                ichar(mcflds(1,kfld)(3:3)) +
     $                ichar(mcflds(1,kfld)(4:4))
               nfldsc(ipos,mtapes) = nfldsc(ipos,mtapes) + 1
               kfld = kfld + 1
            end if
   10    continue
      else
C
C Continuation run, primary tape already declared
C
         kfld = 1
         do i=1,mtapes
            kfld = kfld + nflds(i)
         end do
      end if
C
C Check for auxiliary tape declarations
C
      do 20 itape=mtapes,ptapes-1
         nflds(itape+1) = 0
         nrlen(itape+1) = 0
         nplen(itape+1) = 0
C
C If 1st "aux" is not blank, the user has specified an auxilliary tape
C for the current "itape"
C
         if (aux(1,itape).ne.'        ') then
            mtapes = mtapes + 1
            hbufpt(mtapes) = hbufpt(mtapes-1) + nrlen(mtapes-1)
         endif
C
C Make sure all auxilliary fields are valid
C
         do iaux=1,pflds
            if (aux(iaux,itape).eq.'        ') go to 200
            do imlist=1,pflds
               if (aux(iaux,itape).eq.fieldn(1,imlist)) go to 100
            end do
            write(6,*) 'FLDLST:cant find auxilliary field ',
     $                 aux(iaux,itape)
  100       continue
         end do
  200    continue
         kkfld = 1
         do 15 imlist=1,pflds
            do iaux=1,pflds
C
C Once we hit a blank "aux" entry, all auxilliary fields for this tape
C have been specified
C
               if (aux(iaux,itape).eq.'        ') go to 15
               if (aux(iaux,itape).eq.fieldn(1,imlist)) then
                  if (aux(iaux,itape).eq.'PHIS    '.or.
     $                aux(iaux,itape).eq.'ORO     ') then
                     avg = ninst
                  else
                     avg = ninavg(mtapes)
                  end if
                  fldname(1)=fieldn(1,imlist)(1:8)
                  fldname(2)=fieldn(2,imlist)(1:8)
                  call inifld(fldname,iflds(1,imlist),avg,
     $                          hbufpt  ,ndens(itape+1),
     $                        mtapes  ,nrlen(mtapes) ,nplen(mtapes),
     $                          nflds(mtapes) ,mflds(1,kfld) ,
     $                        mcflds(1,kfld),nupnt(kkfld,mtapes),
     $                          npnt(kkfld,mtapes))
C
C Add one to the count of active field names starting with a given ichar sum
C
                  ipos = ichar(mcflds(1,kfld)(1:1)) +
     $                   ichar(mcflds(1,kfld)(2:2)) +
     $                   ichar(mcflds(1,kfld)(3:3)) +
     $                   ichar(mcflds(1,kfld)(4:4))
                  nfldsc(ipos,mtapes) = nfldsc(ipos,mtapes) + 1
                  kfld = kfld + 1
                  kkfld = kkfld + 1
                  go to 15
               end if
            end do
   15    continue
   20 continue
C
C Set mflds(2) using array npnt, calculating length of hbuf as well
C Also: set the index array which points to the starting location for each 
C ASCII character
C
      lhbuf = 0
      kfld = 1
      do kt=1,mtapes
         islocc(0,kt) = 0
         do i=1,pichsum
            islocc(i,kt) = islocc(i-1,kt) + nfldsc(i-1,kt)
         end do
         do i=1,nflds(kt)
            mflds(2,kfld) = npnt(i,kt) + 2
            if (mod(mflds(1,kfld),10).eq.0) then
               lhbuf = lhbuf + plon
            else
               lhbuf = lhbuf + plon*plev
            end if
            kfld = kfld + 1
         end do
      end do
C
C Pad length of history tape buffer for unblocked I/O
C
      lhbuf = (lhbuf/512 + 1)*512
      return
      end
