#include "params.h"
      subroutine mkslic (field   ,slice   ,nlevf   ,mflds   ,mcflds  ,
     $                   nfldh   ,hrec    ,lhrec   ,nlon    ,nlonh   ,
     $                   nlond   ,nlev)
C-----------------------------------------------------------------------
C
C Mkslic extracts the data for a specified field from an input history
C tape record and returns it as a vertical slice.
C
C The input data record may be packed.
C
C---------------------------Code history--------------------------------
C
C Original version:  B. Boville, October 1990
C Standardized:      L. Bath, June 1992
C Reviewed:          B. Boville, August 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      character*(*) field       ! Desired field
      integer
     $     lhrec,               ! length of input record
     $     nfldh,               ! number of fields in input record
     $     nlon,                ! number of longitudes (distinct)
     $     nlond,               ! number of longitudes (dimension)
     $     nlonh,               ! number of longitudes (input record)
     $     nlev,                ! number of levels (multilevel fields)
     $     nlevf,               ! number of levels in current field
     $     mflds(3,nfldh)       ! integer field list from input header
      character*(*)
     $     mcflds(2,nfldh)      ! character field list (header)
C
C Output arguments
C
      real slice(nlond,nlevf),  ! unpacked data (vertical slice)
     $     hrec(lhrec)          ! packed data slice read
C
C---------------------------Local storage-------------------------------
C
      integer if, i, k, l,      ! counters
     $        ksiz,             ! packed size of data per level
     $        ndens,            ! packing of current field
     $        nlevfi,           ! number of levels in input field
     $        start             ! starting point of current field
C
C------------------------------Externals--------------------------------
C
      external unpkaf           ! unpacker for data (CRAY supplied)
      external endrun
C
C-----------------------------------------------------------------------
C
C Loop through fields on input record until the required field is found.
C The input pointer cannot be used because we allow for packed data.
C
#ifdef PP_PCCM
c in the t21 dataset we have, the first field, U, starts at offset 3.
      start = mflds(2,1) - 2
#else
      start = 1
#endif
      do 100 if = 1, nfldh
C
C Determine the packing density, and number of words per level for
C current field.
C
         ndens = mflds(3,if)
         if (ndens .gt. 1) then
            ksiz = 3 + (nlonh - 1) / ndens
         else
            ksiz = nlonh
         end if
C
C Determine the number of levels for the current field
C
         if (mod(mflds(1,if),10).lt.1) then
            nlevfi = 1
         else
            nlevfi = nlev
         end if
C
C Is this the required field?
C
         if (field .eq. mcflds(1,if)) then
C
C This is the required field, check the number of levels.
C
            if (nlevf .ne. nlevfi) then
               write(6,*)'MKSLIC:Level mismatch for field: ',field
               if (nlevf .eq. 1) then
                  write(6,*)'The array is single level but the'
     $                 //' data is multilevel'
               else
                  write(6,*)'The array is multilevel but the'
     $                 //' data is single level'
               end if
               write(6,*) 'Bad levels in mkslic'
               call endrun
            end if
C
C Unpack and load the data.
C
C           l = mflds(2,if)
            l = start
            do 20 k=1,nlevfi
               if (ndens.gt.1) then
                  call unpkaf(slice(1,k),nlon    ,hrec(l) ,ndens)
               else
                  do 50 i = 1, nlon
                     slice(i,k) = hrec(l+i-1)
   50             continue
               end if
               l = l + ksiz
   20       continue
            return
         else
C
C Not the required field, skip it.
C
            start = start + ksiz * nlevfi
         end if
  100 continue
C
C The required field was not found in the input record.
C
      write(6,*) 'MKSLIC:The required input field ', field,
     $           ' was not on the initial data file'
      write(6,*) 'The available fields are:'
      write(6,*) (mcflds(1,if), if = 1, nfldh)
      call endrun
      end
