#include "params.h"
      subroutine readrg(lhbuf)
C-----------------------------------------------------------------------
C
C Read master regeneration dataset (written from wsds)
C
C---------------------------Code history--------------------------------
C
C Original version:  B. Boville 
C Standardized:      L. Bath, June 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Argument---------------------------------
C
C Output argument
C
      integer lhbuf      ! Length of history tape buffer
C
C------------------------------Commons----------------------------------
#include "pmgrid.com"
C-----------------------------------------------------------------------
#include "pagrid.com"
C-----------------------------------------------------------------------
#include "pspect.com"
C-----------------------------------------------------------------------
#include "comparll.com"
C-----------------------------------------------------------------------
#include "comctl.com"
C-----------------------------------------------------------------------
#include "comhed.com"
C-----------------------------------------------------------------------
#include "comhst.com"
C-----------------------------------------------------------------------
#include "comloc.com"
C-----------------------------------------------------------------------
#include "comlun.com"
C-----------------------------------------------------------------------
#include "commss.com"
C-----------------------------------------------------------------------
#include "comqfl.com"
C-----------------------------------------------------------------------
#include "comtfc.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#include "crdalb.com"
C
C---------------------------Local workspace-----------------------------
C
      character  tcase*8          ! Read in previous case name
      integer ndenst(ptapes),     ! The following 5 variables hold
     $        nhtfrqt(ptapes),    !  history tape controls. These will
     $        mfiltt(ptapes),     !  be stored in their respective
     $        mtapest             !  common variables if this is not a
      logical nlfiltt             !  branch run.
      integer i,j,k               ! Indices
      integer irec                ! Record number for error processing
      integer ierr                ! Error status from read statement
      character*37 clabel         ! Error message

      integer rcurs               ! integer cursor for packing
      integer rbuf_sz             ! size of real buffer for bcst
      parameter( rbuf_sz =        ! determined by counting (plus a few)
     $    3  + 10 ! for safety
     $  + (2*(plev+1)*2)
     $  + (plev*2)
     $)
      real rbuf_bcst(rbuf_sz)     ! buffer for bdcstng to other procs

      integer icurs               ! integer cursor for packing
      integer ibuf_sz             ! size of int buffer for bcst
      parameter( ibuf_sz =        ! determined by counting (plus a few)
     $     15 + 10 ! for safety
     $  + (ptapes*(pichsum+1)*1)
     $  + (ptapes*(pflds+1)*2)
     $  + (ptapes*5)
     $  + (ptapes*3)
     $  + ((pnrg1+1)*2)
     $  + ((pnrg2+1)*2)
     $  + (ptapes*(pnrg3+1)*1)
     $  + (ptapes*(pnrg3)*1)
     $  + (ptapes*3)
     $)
      integer ibuf_bcst(ibuf_sz)  ! buffer for bdcstng to other procs

      integer ccurs               ! character cursor for packing
      integer cbuf_sz             ! size of char buffer for bcst
      parameter( cbuf_sz =        ! determined by counting (plus a few)
     $  + 3*ptapes*80             ! nfpath,ppath,cpath
     $  + ptapes*20               ! nhfil
     $  + ptapes*1	          ! ninavg
     $  + 80                      ! ctitle
     $  + 8                       ! caseid
     $  + (2+1+1+ptapes-1)*pflds*8 ! fieldn,exclude,primary,aux
     $  + 100                     ! safety
     $)
      character*80 tmpstr
      character cbuf_bcst(cbuf_sz)  ! buffer for bdcstng to other procs
      character*6 nhfil_tmp(ptapes)
      character strbuf(80)

C
C------------------------------Externals--------------------------------
C
      external endrun,            ! Abnormal termination
     $         iostop             ! Process I/O errors
C
C-----------------------------------------------------------------------
C
C Preset record number and label
C
      irec = 1
C
Cccccccccccccccccccc Code for processor zero cccccccccccccccccccc
C
      if ( m_pin .eq. mapmesh(0,0) ) then

C
C comtim, comtfc variables
C
        read (nrg,err=10,end=20,iostat=ierr) nstepr,twodt,eps,lhbuf
        irec = irec + 1
C
C Read comhst values
C

        if (lbrnch) then
          read(nrg,err=10,end=20,iostat=ierr) mtapest,nlfiltt
        else
          read(nrg,err=10,end=20,iostat=ierr) mtapest,nlfiltt,
     $         nfldsc,nupnt,npnt
        end if
#ifdef PP_ERROR
	write(0,*)'readrg: mtapest = ',mtapest
#endif
        irec = irec + 1
        read(nrg,err=10,end=20,iostat=ierr)
     $     (nhtfrqt(i) ,ndenst(i) ,
     $      nflds(i) ,nfils(i) ,hbufpt(i), i=1,mtapest)
        irec = irec + 1
        read(nrg,err=10,end=20,iostat=ierr) (nrlen(i),nplen(i),
     $     mfiltt(i),i=1,mtapest)
        irec = irec + 1
        read(nrg,err=10,end=20,iostat=ierr) tcase
        irec = irec + 1
c        if (lbrnch.and.tcase.eq.caseid) then
c         write(6,*) 'READRG:Must change case name on a branch run.'
c         write(6,*) 'Previous name = ',tcase,
c     $              ', current case name = ',caseid
c         call endrun
c        end if
        read(nrg,err=10,end=20,iostat=ierr) (nfpath(i),cpath(i) ,
     $     ppath(i) ,nhfil_tmp(i) ,i=1,mtapest)
	do i = 1, mtapest
	  nhfil(i) = nhfil_tmp(i)
	enddo
        irec = irec + 1
        read(nrg,err=10,end=20,iostat=ierr) (ndcurf(i),nscurf(i),
     $     ncdatf(i),ncsecf(i),i=1,mtapest)
        irec = irec + 1
C
C read comhed values for branch run
C
        read(nrg,err=10,end=20,iostat=ierr) lnhsti,ldhsti,lthsti,lshsti
        irec = irec + 1
        read(nrg,err=10,end=20,iostat=ierr) nbdate,nbsec,ndbase,nsbase
        irec = irec + 1
        read(nrg,err=10,end=20,iostat=ierr) mpsig,siga,sigb

        irec = irec + 1
C
C Read commss values
C
        read(nrg,err=10,end=20,iostat=ierr) nrfil
        irec = irec + 1
C
C Store history tape variables if not a branch run
C
C
C This was not modified from in the parallel code.  It is one of the
C few remaining distinctions that the parallel code makes between a
C restart and a branch run.  This is harmless, since the data in question
C is coming in from the restart file and does not require us to look in an
C old history tape (such as with other configuration information).
C JM 95/02/20.
C
        if (.not.lbrnch) then
           nlfilt = nlfiltt
           mtapes = mtapest
           do j=1,mtapest
              ndens(j) = ndenst(j)
              mfilt(j) = mfiltt(j)
              nhtfrq(j) = nhtfrqt(j)
           end do
        end if
        write(6,9050) nstepr,twodt,eps

c  Don't write out cpath because it contains a null character -RLJ
c       write(6,9060)
c       do j=1,mtapest
c          write(6,*)cpath(j)
c       end do

C
C Read the global integrals of moisture source
C
        read(nrg,err=10,end=20,iostat=ierr) tqf,dqf,fixmas
        irec = irec + 1
C
C note: parallel code does not store surface properties in the master
C restart data set.  These quantities are read in from the time-invariant
C boundary dataset, later, in intbnd.F .  JM 95/02/14
C

C
C Read history buffer regeneration variables - if this is a branch run the 
C history buffer regeneration variables (rg3...) will have already been 
C defined so don t read them in.
C
C Note for parallel code.  This data isn t used in the parallel implemetation
C and there is no such thing as secondary or tertiary regeneration data.
C Regeneration runs are not supported, only restart and branch.
C JM 95/02/20.
C
        if (lbrnch) then
         read(nrg,err=10,end=20,iostat=ierr)
     $        rg1lat  ,rg1siz  ,rg1buf  ,nnrg1   ,
     $        rg2lat  ,rg2siz  ,rg2buf  ,nnrg2   ,
     $        mxszrg,
     $        rg1ext  ,rg2ext
        else
         read(nrg,err=10,end=20,iostat=ierr)
     $        rg1lat  ,rg1siz  ,rg1buf  ,nnrg1   ,
     $        rg2lat  ,rg2siz  ,rg2buf  ,nnrg2   ,
     $        mxszrg,
     $        rg1ext  ,rg2ext,
     $        rg3lat,rg3siz,rg3buf,nnrg3,rg3ext,rgnht,
     $        rg3num
        end if
        irec = irec + 1
C
C Are absortivities and emissivities part of the regeneration data.
C      
        read(nrg,err=10,end=20,iostat=ierr) aeregen
C
C All information read, rewind the master regeneration dataset, since it
C may be overwritten if the last primary history file was not complete.
C
        rewind nrg
        
C
C Broadcast the information to the other processors
C
        icurs = 1
        rcurs = 1
c comtim, comtfc variables
        ibuf_bcst(icurs) = nstepr
        icurs = icurs+1
        rbuf_bcst(rcurs) = twodt
        rcurs = rcurs+1
        rbuf_bcst(rcurs) = eps
        rcurs = rcurs+1
        ibuf_bcst(icurs) = lhbuf
        icurs = icurs+1
c Read comhst values
        ibuf_bcst(icurs) = mtapest
        icurs = icurs+1
        if ( nlfiltt ) then
          ibuf_bcst(icurs) = 1
        else
          ibuf_bcst(icurs) = 0
        endif
        icurs = icurs+1
        do j = 1, ptapes
          do i = 0, pichsum
            ibuf_bcst(icurs) = nfldsc(i,j)
            icurs = icurs+1
          enddo
          do i = 1, pflds
            ibuf_bcst(icurs) = nupnt(i,j)
            icurs = icurs+1
            ibuf_bcst(icurs) = npnt(i,j)
            icurs = icurs+1
          enddo
        enddo
        do i = 1, ptapes
          ibuf_bcst(icurs) = nhtfrqt(i)
          icurs = icurs+1
          ibuf_bcst(icurs) = ndenst(i)
          icurs = icurs+1
          ibuf_bcst(icurs) = nflds(i)
          icurs = icurs+1
          ibuf_bcst(icurs) = nfils(i)
          icurs = icurs+1
          ibuf_bcst(icurs) = hbufpt(i)
          icurs = icurs+1
        enddo
        do i = 1, ptapes
          ibuf_bcst(icurs) = nrlen(i)
          icurs = icurs+1
          ibuf_bcst(icurs) = nplen(i)
          icurs = icurs+1
          ibuf_bcst(icurs) = mfiltt(i)
          icurs = icurs+1
        enddo
        ibuf_bcst(icurs) = nbdate
        icurs = icurs+1
        ibuf_bcst(icurs) = nbsec
        icurs = icurs+1
        ibuf_bcst(icurs) = ndbase
        icurs = icurs+1
        ibuf_bcst(icurs) = nsbase
        icurs = icurs+1
        ibuf_bcst(icurs) = mpsig
        icurs = icurs+1
        do i = 1, 2*plev+1
          rbuf_bcst(rcurs) = siga(i)
          rcurs = rcurs+1
          rbuf_bcst(rcurs) = sigb(i)
          rcurs = rcurs+1
        enddo
        do i = 1, plev
          rbuf_bcst(rcurs) = tqf(i)
          rcurs = rcurs+1
          rbuf_bcst(rcurs) = dqf(i)
          rcurs = rcurs+1
        enddo
          rbuf_bcst(rcurs) = fixmas
          rcurs = rcurs+1
c
        do i = 1, pnrg1
          ibuf_bcst(icurs) = rg1lat(i)
          icurs = icurs+1
          ibuf_bcst(icurs) = rg1siz(i)
          icurs = icurs+1
        enddo
        ibuf_bcst(icurs) = rg1lat(pnrg1+1)
        icurs = icurs + 1
        ibuf_bcst(icurs) = rg1buf
        icurs = icurs+1
        ibuf_bcst(icurs) = nnrg1
        icurs = icurs+1

        do i = 1, pnrg2
          ibuf_bcst(icurs) = rg2lat(i)
          icurs = icurs+1
          ibuf_bcst(icurs) = rg2siz(i)
          icurs = icurs+1
        enddo
        ibuf_bcst(icurs) = rg2lat(pnrg2+1)
        icurs = icurs + 1
        ibuf_bcst(icurs) = rg2buf
        icurs = icurs+1
        ibuf_bcst(icurs) = nnrg2
        icurs = icurs+1
        
        ibuf_bcst(icurs) = mxszrg
        icurs = icurs+1

        do j = 1, ptapes
          do i = 1, pnrg3+1
            ibuf_bcst(icurs) = rg3lat(i,j)
            icurs = icurs+1
          enddo
          do i = 1, pnrg3
            ibuf_bcst(icurs) = rg3lat(i,j)
            icurs = icurs+1
          enddo
          ibuf_bcst(icurs) = rg3buf(j)
          icurs = icurs+1
          ibuf_bcst(icurs) = nnrg3(j)
          icurs = icurs+1
          if ( rgnht(j) ) then
            ibuf_bcst(icurs) = 1
          else
            ibuf_bcst(icurs) = 0
          endif
          icurs = icurs+1
        enddo
        ibuf_bcst(icurs) = rg3num
        icurs = icurs+1

        if ( aeregen ) then
          ibuf_bcst(icurs) = 1
        else
          ibuf_bcst(icurs) = 0
        endif
c
c  character data
c
              ccurs = 1
	      do i = 1, ptapes
		tmpstr = nfpath(i)
		do j = 1,80
		  cbuf_bcst(ccurs) = tmpstr(j:j)
		  ccurs = ccurs + 1
		enddo
		tmpstr = ppath(i)
		do j = 1,80
		  cbuf_bcst(ccurs) = tmpstr(j:j)
		  ccurs = ccurs + 1
		enddo
		tmpstr = cpath(i)
		do j = 1,80
		  cbuf_bcst(ccurs) = tmpstr(j:j)
		  ccurs = ccurs + 1
		enddo
		tmpstr = nhfil(i)
		do j = 1,20
		  cbuf_bcst(ccurs) = tmpstr(j:j)
		  ccurs = ccurs + 1
		enddo
		tmpstr = ninavg(i)
		cbuf_bcst(ccurs) = tmpstr(1:1)
		ccurs = ccurs + 1
	      enddo
              do j = 1,80
		cbuf_bcst(ccurs) = ctitle(j:j)
		ccurs = ccurs + 1
	      enddo
              do j = 1,8
		cbuf_bcst(ccurs) = caseid(j:j)
		ccurs = ccurs + 1
	      enddo
	      do i = 1, pflds
		tmpstr = fieldn(1,i)
		do j = 1,8
		  cbuf_bcst(ccurs) = tmpstr(j:j)
		  ccurs = ccurs + 1
		enddo
		tmpstr = fieldn(2,i)
		do j = 1,8
		  cbuf_bcst(ccurs) = tmpstr(j:j)
		  ccurs = ccurs + 1
		enddo
		tmpstr = exclude(i)
		do j = 1,8
		  cbuf_bcst(ccurs) = tmpstr(j:j)
		  ccurs = ccurs + 1
		enddo
		tmpstr = primary(i)
		do j = 1,8
		  cbuf_bcst(ccurs) = tmpstr(j:j)
		  ccurs = ccurs + 1
		enddo
		do k = 1, ptapes-1
		  tmpstr = aux(i,k)
		  do j = 1,8
		    cbuf_bcst(ccurs) = tmpstr(j:j)
		    ccurs = ccurs + 1
		  enddo
		enddo
	      enddo

        if ( icurs .gt. ibuf_sz ) then
          write(6,*)'READRG: icurs .gt. ibuf_sz',icurs,ibuf_sz
          write(0,*)'READRG: icurs .gt. ibuf_sz',icurs,ibuf_sz
        endif
        if ( rcurs .gt. rbuf_sz ) then
          write(6,*)'READRG: rcurs .gt. rbuf_sz',rcurs,rbuf_sz
          write(0,*)'READRG: rcurs .gt. rbuf_sz',rcurs,rbuf_sz
        endif
	if ( ccurs .gt. cbuf_sz ) then
          write(6,*)'READRG: ccurs .gt. cbuf_sz',ccurs,cbuf_sz
          write(0,*)'READRG: ccurs .gt. cbuf_sz',ccurs,cbuf_sz
	endif
        if (     (rcurs .gt. rbuf_sz) .or. (icurs .gt. ibuf_sz)
     $      .or. (ccurs .gt. cbuf_sz)                          ) then
          call endrun
        endif

        call bcast0f(ibuf_bcst,ibuf_sz*I_SIZE,m_type,mapmesh(0,0))
        m_type = m_type + 1
        call bcast0f(rbuf_bcst,rbuf_sz*R_SIZE,m_type,mapmesh(0,0))
        m_type = m_type + 1
        call bcast0f(cbuf_bcst,cbuf_sz,m_type,mapmesh(0,0))
        m_type = m_type + 1

        do i = 1, 5
          strbuf(i) = nrfil(i:i)
        enddo
        call bcast0f(strbuf,5,m_type,mapmesh(0,0))
        m_type = m_type + 1


C
Cccccccccccccccccccc Code for other processors cccccccccccccccccccc
C
      else  ! not processor zero ---------------------------------

        call bcast0f(ibuf_bcst,ibuf_sz*I_SIZE,m_type,mapmesh(0,0))
        m_type = m_type + 1
        call bcast0f(rbuf_bcst,rbuf_sz*R_SIZE,m_type,mapmesh(0,0))
        m_type = m_type + 1
        call bcast0f(cbuf_bcst,cbuf_sz,m_type,mapmesh(0,0))
        m_type = m_type + 1

        call bcast0f(strbuf,5,m_type,mapmesh(0,0))
        m_type = m_type + 1
        do i = 1, 5
          nrfil(i:i) = strbuf(i)
        enddo

        icurs = 1
                                        rcurs = 1
c comtim, comtfc variables
        nstepr = ibuf_bcst(icurs) 
        icurs = icurs+1
                                        twodt = rbuf_bcst(rcurs) 
                                        rcurs = rcurs+1
                                        eps = rbuf_bcst(rcurs) 
                                        rcurs = rcurs+1
        lhbuf = ibuf_bcst(icurs) 
        icurs = icurs+1
c Read comhst values
        mtapest = ibuf_bcst(icurs) 
        icurs = icurs+1
        nlfiltt = ( ibuf_bcst(icurs)  .eq. 1 )
        icurs = icurs+1

        do j = 1, ptapes
          do i = 0, pichsum
            nfldsc(i,j) = ibuf_bcst(icurs)
            icurs = icurs+1
          enddo
          do i = 1, pflds
            nupnt(i,j) = ibuf_bcst(icurs)
            icurs = icurs+1
            npnt(i,j) = ibuf_bcst(icurs)
            icurs = icurs+1
          enddo
        enddo
        do i = 1, ptapes
          nhtfrqt(i) = ibuf_bcst(icurs) 
          icurs = icurs+1
          ndenst(i) = ibuf_bcst(icurs) 
          icurs = icurs+1
          nflds(i) = ibuf_bcst(icurs) 
          icurs = icurs+1
          nfils(i) = ibuf_bcst(icurs) 
          icurs = icurs+1
          hbufpt(i) = ibuf_bcst(icurs) 
          icurs = icurs+1
        enddo
        do i = 1, ptapes
          nrlen(i) = ibuf_bcst(icurs) 
          icurs = icurs+1
          nplen(i) = ibuf_bcst(icurs) 
          icurs = icurs+1
          mfiltt(i) = ibuf_bcst(icurs) 
          icurs = icurs+1
        enddo
        nbdate = ibuf_bcst(icurs) 
        icurs = icurs+1
        nbsec = ibuf_bcst(icurs) 
        icurs = icurs+1
        ndbase = ibuf_bcst(icurs) 
        icurs = icurs+1
        nsbase = ibuf_bcst(icurs) 
        icurs = icurs+1
        mpsig = ibuf_bcst(icurs) 
        icurs = icurs+1
        do i = 1, 2*plev+1
          siga(i) = rbuf_bcst(rcurs)
          rcurs = rcurs+1
          sigb(i) = rbuf_bcst(rcurs)
          rcurs = rcurs+1
        enddo
        do i = 1, plev
          tqf(i) = rbuf_bcst(rcurs)
          rcurs = rcurs+1
          dqf(i) = rbuf_bcst(rcurs)
          rcurs = rcurs+1
        enddo
        fixmas = rbuf_bcst(rcurs) 
        rcurs = rcurs+1
c
        do i = 1, pnrg1
          rg1lat(i) = ibuf_bcst(icurs) 
          icurs = icurs+1
          rg1siz(i) = ibuf_bcst(icurs) 
          icurs = icurs+1
        enddo
        rg1lat(pnrg1+1) = ibuf_bcst(icurs)
        icurs = icurs + 1
        rg1buf = ibuf_bcst(icurs) 
        icurs = icurs+1
        nnrg1 = ibuf_bcst(icurs) 
        icurs = icurs+1
c
        do i = 1, pnrg2
          rg2lat(i) = ibuf_bcst(icurs) 
          icurs = icurs+1
          rg2siz(i) = ibuf_bcst(icurs) 
          icurs = icurs+1
        enddo
        rg2lat(pnrg2+1) = ibuf_bcst(icurs)
        icurs = icurs + 1
        rg2buf = ibuf_bcst(icurs) 
        icurs = icurs+1
        nnrg2 = ibuf_bcst(icurs) 
        icurs = icurs+1
        mxszrg = ibuf_bcst(icurs) 
        icurs = icurs+1
c
        do j = 1, ptapes
          do i = 1, pnrg3+1
            rg3lat(i,j) = ibuf_bcst(icurs) 
            icurs = icurs+1
          enddo
          do i = 1, pnrg3
            rg3lat(i,j) = ibuf_bcst(icurs) 
            icurs = icurs+1
          enddo
          rg3buf(j) = ibuf_bcst(icurs) 
          icurs = icurs+1
          nnrg3(j) = ibuf_bcst(icurs) 
          icurs = icurs+1
          rgnht(j) = ( ibuf_bcst(icurs)  .eq. 1 )
          icurs = icurs+1
        enddo
        rg3num = ibuf_bcst(icurs) 
        icurs = icurs+1
        aeregen = ( ibuf_bcst(icurs) .eq. 1 )
        icurs = icurs+1
c
c character data
c
              ccurs = 1
	      do i = 1, ptapes
		do j = 1,80
		  tmpstr(j:j) = cbuf_bcst(ccurs)
		  ccurs = ccurs + 1
		enddo
		nfpath(i) = tmpstr(1:80)
		do j = 1,80
		  tmpstr(j:j) = cbuf_bcst(ccurs)
		  ccurs = ccurs + 1
		enddo
		ppath(i) = tmpstr(1:80)
		do j = 1,80
		  tmpstr(j:j) = cbuf_bcst(ccurs)
		  ccurs = ccurs + 1
		enddo
		cpath(i) = tmpstr(1:80)
		do j = 1,20
		  tmpstr(j:j) = cbuf_bcst(ccurs)
		  ccurs = ccurs + 1
		enddo
		nhfil(i) = tmpstr(1:20)
		tmpstr(1:1) = cbuf_bcst(ccurs)
		ccurs = ccurs + 1
		ninavg(i) = tmpstr(1:1)
	      enddo
              do j = 1,80
		ctitle(j:j) = cbuf_bcst(ccurs)
		ccurs = ccurs + 1
	      enddo
              do j = 1,8
		caseid(j:j) = cbuf_bcst(ccurs)
		ccurs = ccurs + 1
	      enddo
	      do i = 1, pflds
		do j = 1,8
		  tmpstr(j:j) = cbuf_bcst(ccurs)
		  ccurs = ccurs + 1
		enddo
		fieldn(1,i) = tmpstr(1:8)
		do j = 1,8
		  tmpstr(j:j) = cbuf_bcst(ccurs)
		  ccurs = ccurs + 1
		enddo
		fieldn(2,i) = tmpstr(1:8)
		do j = 1,8
		  tmpstr(j:j) = cbuf_bcst(ccurs)
		  ccurs = ccurs + 1
		enddo
	        exclude(i) = tmpstr(1:8)
		do j = 1,8
		  tmpstr(j:j) = cbuf_bcst(ccurs)
		  ccurs = ccurs + 1
		enddo
		primary(i) = tmpstr(1:8)
		do k = 1, ptapes-1
		  do j = 1,8
		    tmpstr(j:j) = cbuf_bcst(ccurs)
		    ccurs = ccurs + 1
		  enddo
		  aux(i,k) = tmpstr(1:8)
		enddo
	      enddo
      endif

cccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccc

#if 0
              do j = 1, mtapest
                do i = 0, pichsum
                  write(0,*)'nfldsc(',i,',',j,')  = ', nfldsc(i,j)
                enddo
                do i = 0, mtapest
                  write(0,*)'nupnt(',i,',',j,')  = ', nupnt(i,j)
                  write(0,*)' npnt(',i,',',j,')  = ', npnt(i,j)
                enddo
              enddo


	      do i = 1, mtapest
	        write(0,*)'hbufpt(',i,') = ',hbufpt(i)
	      enddo
	      write(0,*)'lhbuf         = ',lhbuf

              do icurs = 1, ibuf_sz
                write(0,*)'ibuf_bcst(',icurs,') = ',ibuf_bcst(icurs)
              enddo
              do rcurs = 1, rbuf_sz
                write(0,*)'rbuf_bcst(',rcurs,') = ',rbuf_bcst(rcurs)
              enddo

	      do i = 1, ptapes
		print*, i,'nfpath(i) ',nfpath(i)
		print*, i,'ppath(i) ',ppath(i)
		print*, i,'cpath(i) ',cpath(i)
		print*, i,'nhfil(i) ',nhfil(i)
		print*, i,'ninavg(i) ',ninavg(i)
	      enddo
	      print*,'ctitle ',ctitle
	      print*,'caseid ',caseid
	      do i = 1, pflds
		print*,i,'fieldn(1,i) ',fieldn(1,i)
		print*,i,'fieldn(2,i) ',fieldn(2,i)
		print*,i,'exclude(i) ',exclude(i)
		print*,i,'primary(i) ',primary(i)
		do k = 1, ptapes-1
		  print*,i,k,'aux(i,k)     ',aux(i,k)
		enddo
	      enddo
#endif

cccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccc
      


      return
   10 continue
      write (clabel,'(a35,i2)') 'READRG: I/O error in record number ',
     $                          irec
      call iostop(ierr,nrg,0,clabel)
   20 continue
      call iostop(ierr,nrg,0,'READRG: End of file')
C
C Formats
C
 9050 format(//,'MASTER REGENERATION DATASET READ',/,
     $         ' NSTEPR,TWODT,EPS= ',1x,i9,f8.1,e12.2)
 9060 format('HISTORY TAPES FOR RESTART:')
      end


