#include "params.h"
      subroutine stats(lat     ,irow    ,pint    ,pdel    ,pstar   ,
     $                 vort    ,div     ,t       ,q       )
C-----------------------------------------------------------------------
C
C Accumulation of diagnostic statistics for 1 latitude.
C
C---------------------------Code history--------------------------------
C
C Original version:  J. Rosinski
C Standardized:      J. Rosinski, June 1992
C Reviewed:          D. Williamson, J. Hack, August 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pspect.com"
C------------------------------Commons----------------------------------
#include "commap.com"
C-----------------------------------------------------------------------
#include "comsta.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer
     $     lat,                ! latitude index (S->N)
     $     irow                ! latitude pair index
      real pint(plond,plevp),  ! pressure at model interfaces
     $     pdel(plond,plev),   ! pdel(k) = pint(k+1) - pint(k)
     $     pstar(plond),       ! ps + psr (surface pressure)
     $     vort(plond,plev),   ! vorticity
     $     div(plond,plev),    ! divergence
     $     t(plond,plev),      ! temperature
     $     q(plond,plev)       ! moisture
C
C---------------------------Local workspace-----------------------------
C
      real prat                ! pdel(i,k)/pint(i,plevp)
      integer
     $     i,k                 ! longitude, level indices
#ifdef PP_PCCM
      real psurf_tmp(plon)
      real rmsz_tmp(plon)
      real rmsd_tmp(plon)
      real rmst_tmp(plon)
      real stq_tmp(plon)
      real repro_tmp(plon)
c external function
      external f_locsum
      real f_locsum
#endif
C
C-----------------------------------------------------------------------
C
C Compute statistics for current latitude line
C
#ifdef PP_PCCM
      do i = 1,plon
         rmsz_tmp(i)  = 0.0
         rmsd_tmp(i)  = 0.0
         rmst_tmp(i)  = 0.0
         stq_tmp(i)   = 0.0
         psurf_tmp(i) = pstar(i)
      enddo

      do k = 1, plev
         do i = 1, plon
            prat = pdel(i,k)/pint(i,plevp)
            rmsz_tmp(i) = rmsz_tmp(i) + vort(i,k)*vort(i,k)*prat
            rmsd_tmp(i) = rmsd_tmp(i) + div(i,k)*div(i,k)*prat
            rmst_tmp(i) = rmst_tmp(i) + t(i,k)**2*prat
            stq_tmp(i)  = stq_tmp(i)  + q(i,k)*pdel(i,k)
         enddo
      enddo

      psurf(lat)  = f_locsum( psurf_tmp(1),plon, repro_tmp )
      rmsz(lat)   = f_locsum( rmsz_tmp(1), plon, repro_tmp )
      rmsd(lat)   = f_locsum( rmsd_tmp(1), plon, repro_tmp )
      rmst(lat)   = f_locsum( rmst_tmp(1), plon, repro_tmp )
      stq(lat)    = f_locsum( stq_tmp(1),  plon, repro_tmp )
#else
      do i=1,plon
         psurf(lat) = psurf(lat) + pstar(i)
      end do
      do k=1,plev
         do i=1,plon
            prat = pdel(i,k)/pint(i,plevp)
            rmsz(lat) = rmsz(lat) + vort(i,k)*vort(i,k)*prat
            rmsd(lat) = rmsd(lat) + div(i,k)*div(i,k)*prat
            rmst(lat) = rmst(lat) + (t(i,k)**2)*prat
            stq(lat) = stq(lat) + q(i,k)*pdel(i,k)
         end do
      end do   
      psurf(lat) = w(irow)*psurf(lat)
      rmsz(lat) = w(irow)*rmsz(lat)
      rmsd(lat) = w(irow)*rmsd(lat)
      rmst(lat) = w(irow)*rmst(lat)
      stq (lat) = w(irow)*stq(lat)
#endif
C 
      return
      end
