#include "params.h"
      subroutine cldint(coslat  ,pmid    ,rpdeli  ,t       ,q       ,
     $                  omga    ,cldtop  ,cldbot  ,cnvprc  ,piln    ,
     $                  pmln    ,phis    ,tvm     ,zm      ,cld     ,
     $                  clc     ,clwp    ,emis    ,effcld  ,cltot   ,
     $                  cllow   ,clmed   ,clhgh   ,pdel    , cmfmc  ,
     $			oro     ,snowh   ,rel     ,rei     ,fice    ,
     $		        ps)
C-----------------------------------------------------------------------
C
C Interface routine for cloud fraction evaluation
C
C---------------------------Code history--------------------------------
C
C Original version:  B. Briegleb
C Standardized:      J. Rosinski, June 1992
C Reviewed:          J. Hack, J. Kiehl, August 1992
C Reviewed:          J. Kiehl, April 1996
C
C note: nearly identical to CCM3 version -RLJ Dec, 1996
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
C------------------------------Commons----------------------------------
#include "comcon.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      real coslat,              ! cosine of latitude
     $     pmid(plond,plev),    ! midpoint pressures
     $     rpdeli(plond,plev),  ! 1./(pmid(k+1)-pmid(k))
     $     t(plond,plev),       ! temperature
     $     q(plond,plev),       ! specific humidity
     $     omga(plond,plev)     ! vertical pressure velocity
      real cldtop(plond),       ! top level of convective activity
     $     cldbot(plond),       ! bottom level of convective activity
     $     cnvprc(plond),       ! convective precipitation rate (m/s)
     $     piln(plond,plevp),   ! log of interface pressures
     $     pmln(plond,plev),    ! log of midpoint pressures
     $     phis(plond),         ! surface geopotential height
     $     tvm(plond,plev),     ! virtual temperature 
     $     zm(plond,plev),      ! height of midpoints (above surface) 
C added for CCM3 cldfrc
     $     cmfmc(plond,plev),   ! convective mass flux--m sub c
     $     pdel(plond,plev)     ! pressure depth of layer
      real snowh(plond)         ! snow depth (liquid water equivalent)
      real oro(plond)           ! Land/ocean/seaice flag
      real ps(plond)            ! surface pressure

C
C Output arguments
C
      real cld(plond,plevp),    ! cloud fraction
     $     clc(plond),          ! column convective cloud fraction
     $     clwp(plond,plev),    ! cloud liquid water path
     $     emis(plond,plev),    ! cloud emissivity
     $     effcld(plond,plevp), ! effective cloud=cld*emis
     $     cltot(plond),        ! diagnosed tot random overlap cld cover
     $     cllow(plond),        ! diagnosed low random overlap cld cover
     $     clmed(plond),        ! diagnosed med random overlap cld cover
     $     clhgh(plond)         ! diagnosed hgh random overlap cld cover

c added CCM3 output
      real rel(plond,plev)      ! effective drop radius (microns)
      real rei(plond,plev)      ! ice effective drop size (microns)
      real fice(plond,plev)     ! fractional ice content within cloud

C
C---------------------------Local workspace-----------------------------
C
      integer i,k               ! longitude,level indices
      real zi(plond,plevp),     ! interface geometric height
     $     rgrav                ! inverse gravitational acceleration

c needed for CCM3 routines but not needed outside of cldint
      real tpw(plond)           ! total precipitable water
      real hl(plond)            ! Liquid water scale height

C
C------------------------------Externals--------------------------------
C
      external 
     $     cldfrc,              ! compute cloud fraction
     $     zint,                ! compute height interface surfaces
     $     cldclw,              ! compute cloud liquid water path
     $     cldems,              ! compute cloud emissivity
     $     cldsav               ! cloud fraction diagnostics
C
C-----------------------------------------------------------------------
C
C Compute cloud amount
C
c call the new CCM3 version
      call cldfrc(pmid    ,rpdeli  ,t       ,q       ,omga    ,
     $            cldtop  ,cldbot  ,cld     ,clc     ,pdel    ,
     $            cmfmc   ,oro     ,snowh   )

c     call cldfrc(coslat  ,pmid    ,rpdeli  ,t       ,q       ,
c    $            omga    ,cldtop  ,cldbot  ,cnvprc  ,cld     ,
c    $            clc     )
C
C Interface heights required for cldclw
C
c  CCM2 and CCM3 versions are identical
      call zint(piln    ,pmln    ,rair    ,gravit  ,tvm     ,
     $          zm      ,zi      )
C
C Add surface height to relative height
C
c not done in CCM3
c     rgrav = 1.0/gravit
c     do k=1,plevp
c        do i=1,plon
c           zi(i,k) = zi(i,k) + phis(i)*rgrav
c        end do
c     end do
C
C Cloud liquid water path
C
C CCM3 addition:
C Begin by diagnosing total preciptable water in column (in mm)
      do i=1,plon
       tpw(i) = 0.0
      end do
      rgrav = 1.0/gravit
      do k=1,plev
        do i=1,plon
	  tpw(i) = tpw(i) + pdel(i,k)*q(i,k)*rgrav
	end do
      end do


c     call cldclw(zi,coslat,clwp)
c Call CCM3 version
      call cldclw(zi      ,clwp    ,tpw     ,hl      )

c new CCM3 routine
C
C Cloud particle size and fraction of ice
C
      call cldefr(oro     ,t       ,rel     ,rei     ,fice    ,
     $            ps      ,pmid    )

C
C Cloud emissivity
C
c     call cldems(clwp,emis)
c call new CCM3 version
      call cldems(clwp    ,fice    ,rei     ,emis    )
C
C Cloud cover diagnostics (required to get random overlap correct)
C
      call cldsav(cld     ,pmid    ,cltot   ,cllow   ,clmed   ,
     $            clhgh   )
C
C Effective cloud cover
C
      do k=1,plev
         do i=1,plon
            effcld(i,k) = cld(i,k)*emis(i,k)
         end do
      end do
C
C Cloud cover at surface interface always zero (for safety-s sake)
C
      do i=1,plon
         effcld(i,plevp) = 0.
         cld(i,plevp)    = 0.
      end do
C
      return
      end
