#include "params.h"
      subroutine cldsav(cld     ,pmid    ,cldtot  ,cldlow  ,cldmed  ,
     $                  cldhgh  )
C-----------------------------------------------------------------------
C
C Compute total & three levels of cloud fraction assuming random            
C overlap. These diagnostics must be done on the fly to correctly deal  
C with random overlap. Pressure ranges for the three cloud levels are
C specified.
C
C---------------------------Code history--------------------------------
C
C Original version:  B. Briegleb
C Standardized:      J. Rosinski, June 1992
C Reviewed:          J. Hack, J. Kiehl, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
      real plowmax,            ! max prs for low cloud cover range
     $     plowmin,            ! min prs for low cloud cover range
     $     pmedmax,            ! max prs for mid cloud cover range
     $     pmedmin,            ! min prs for mid cloud cover range
     $     phghmax,            ! max prs for hgh cloud cover range
     $     phghmin             ! min prs for hgh cloud cover range
C
      parameter (plowmax = 120000.,plowmin = 70000.,
     $           pmedmax =  70000.,pmedmin = 40000.,
     $           phghmax =  40000.,phghmin =  5000.)
C
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      real cld(plond,plevp),   ! cloud fraction
     $     pmid(plond,plev)    ! level pressures
C
C Output arguments
C
      real cldtot(plond),      ! total random overlap cloud cover
     $     cldlow(plond),      ! low random overlap cloud cover
     $     cldmed(plond),      ! middle random overlap cloud cover
     $     cldhgh(plond)       ! high random overlap cloud cover
C
C---------------------------Local workspace-----------------------------
C
      integer i,k              ! longitude,level indices
      real clrsky(plond)       ! random overlap clear sky fraction
C
C-----------------------------------------------------------------------
C
C Total cloud
C
      do i=1,plon
         clrsky(i) = 1.0
      end do
      do k=plev,1,-1
         do i=1,plon
            clrsky(i) = clrsky(i)*(1. - cld(i,k))
         end do
      end do
      do i=1,plon
         cldtot(i) = 1. - clrsky(i)
      end do
C
C High cloud (400mb -> 50mb)
C
      do i=1,plon
         clrsky(i) = 1.0
      end do
      do k=plev,1,-1
         do i=1,plon
            if (phghmin.le.pmid(i,k) .and. pmid(i,k).lt.phghmax) then
               clrsky(i) = clrsky(i)*(1. - cld(i,k))
            end if
         end do
      end do
      do i=1,plon
         cldhgh(i) = 1. - clrsky(i)
      end do
C
C Medium cloud (700mb -> 400mb)
C
      do i=1,plon
         clrsky(i) = 1.0
      end do
      do k=plev,1,-1
         do i=1,plon
            if (pmedmin.le.pmid(i,k) .and. pmid(i,k).lt.pmedmax) then
               clrsky(i) = clrsky(i)*(1. - cld(i,k))
            end if
         end do
      end do
      do i=1,plon
         cldmed(i) = 1. - clrsky(i)
      end do
C
C Low cloud (sfc -> 700mb)
C
      do i=1,plon
         clrsky(i) = 1.0
      end do
      do k=plev,1,-1
         do i=1,plon
            if (plowmin.le.pmid(i,k) .and. pmid(i,k).lt.plowmax) then
               clrsky(i) = clrsky(i)*(1. - cld(i,k))
            end if
         end do
      end do
      do i=1,plon
         cldlow(i) = 1. - clrsky(i)
      end do
C
      return
      end
