#include "params.h"
      subroutine vdintr(hbuf    ,lhbuf   ,
     $                  pmidm1  ,pintm1  ,psomc   ,thm     ,
     $                  zm      ,tsm1    ,rpdel   ,rpdeli  ,
     $                  um1     ,vm1     ,tm1     ,
     $                  taux    ,tauy    ,hflx    ,cflx    ,ztodt   ,
     $                  lat     ,qm1     ,fu      ,fv      ,t2      ,
     $                  dtv     ,duv     ,dvv     ,up1     ,vp1     ,
     $                  pblh    ,tpert   ,qpert   )
C-----------------------------------------------------------------------
C
C interface routine for vertical diffusion and pbl scheme
C
C calling sequence:
C
C    vdinti        initializes vertical diffustion constants
C    pbinti        initializes pbl constants
C     .
C     .
C    vdintr        interface for vertical diffusion and pbl scheme
C      vdiff       performs vert diff and pbl
C        pbldif    boundary layer scheme
C        mvdiff    diffuse momentum
C        qvdiff    diffuse constituents
C
C---------------------------Code history--------------------------------
C
C Original version:  J. Rosinski
C Standardized:      J. Rosinski, June 1992
C Reviewed:          P. Rasch, B. Boville, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
C------------------------------Commons----------------------------------
#include "comtrcnm.com"
C-----------------------------------------------------------------------
#include "comvd.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      real hbuf(*),               ! history tape buffer,for calls to outfld
     $     pmidm1(plond,plev),    ! midpoint pressures
     $     pintm1(plond,plevp),   ! interface pressures
     $     psomc(plond,plev),     ! (psm1/pmidm1)**cappa
     $     thm(plond,plev),       ! potential temperature midpoints
     $     zm(plond,plev),        ! midpoint geopotential height above sfc
     $     tsm1(plond),           ! surface temperature
     $     rpdel(plond,plev),     ! 1./pdel (thickness between interfaces)
     $     rpdeli(plond,plev),    ! 1./pdeli (thickness between midpoints)
     $     um1(plond,plev),       ! u-wind input
     $     vm1(plond,plev),       ! v-wind input
     $     tm1(plond,plev),       ! temperature input
     $     taux(plond),           ! x surface stress (n)
     $     tauy(plond),           ! y surface stress (n)
     $     hflx(plond),           ! surface sensible heat flux (w/m2)
     $     cflx(plond,pcnst),     ! surface constituent flux (kg/m2/s)
     $     ztodt                  ! 2 delta-t
      integer lat                 ! latitude index
      integer lhbuf               ! history tape buffer length
C
C Input/output arguments
C
      real qm1(plond,plev,pcnst), ! initial/final constituent field
     $     fu(plond,plev),        ! cumulative u tendency
     $     fv(plond,plev),        ! cumulative v tendency
     $     t2(plond,plev)         ! cumulative T tendency
C
C Output arguments
C
      real dtv(plond,plev),       ! temperature tendency (heating)
     $     duv(plond,plev),       ! u-wind tendency
     $     dvv(plond,plev),       ! v-wind tendency
     $     up1(plond,plev),       ! u-wind after vertical diffusion
     $     vp1(plond,plev),       ! v-wind after vertical diffusion
     $     pblh(plond),           ! planetary boundary layer height
     $     tpert(plond),          ! convective temperature excess
     $     qpert(plond,pcnst)     ! convective humidity and constituent
C                                 !  excess
C
C---------------------------Local storage-------------------------------
C
      integer
     $     i,k,m                  ! longitude,level,constituent indices
      real dqv(plond,plev,pcnst), ! constituent diffusion tendency
     $     ustar(plond),          ! surface friction velocity
     $     kvh(plond,plevp),      ! diffusion coefficient for heat
     $     kvm(plond,plevp),      ! diffusion coefficient for momentum
     $     denom,                 ! denominator of expression
     $     qp1(plond,plev,pcnst), ! constituents after vdiff
     $     thp(plond,plev),       ! potential temperature after vdiff
     $     cgs(plond,plevp),      ! counter-gradient star (cg/flux)
     $     rztodt                 ! 1./ztodt
C
C------------------------------Externals--------------------------------
C
      external
     $     outfld,                ! history file handler
     $     vdiff                  ! vertical diffusion code
C
C-----------------------------------------------------------------------
C
C Call vertical diffusion code. No setup work is required.
C
      call vdiff(lat     ,um1     ,vm1     ,tm1     ,qm1     ,
     $           tsm1    			    ,pmidm1  ,
     $           pintm1  ,rpdel   ,rpdeli  ,ztodt   ,thm     ,
     $           zm      ,taux    ,tauy    ,hflx    ,cflx    ,
     $           up1     ,vp1     ,thp     ,qp1     ,pblh    ,
     $           ustar   ,kvh     ,kvm     ,tpert   ,qpert   ,
     $           cgs     )
C
C Convert the diffused fields back to diffusion tendencies.
C Add the diffusion tendencies to the cummulative physics tendencies,
C except for constituents. The diffused values of the constituents
C replace the input values.
C
      rztodt = 1./ztodt
      do k=ntopfl,plev
         do i=1,plon
            duv(i,k) = (up1(i,k) - um1(i,k))*rztodt
            dvv(i,k) = (vp1(i,k) - vm1(i,k))*rztodt
            fu(i,k)  = fu(i,k) + duv(i,k)
            fv(i,k)  = fv(i,k) + dvv(i,k)
            denom    = cpair*(1. + cpvir*qm1(i,k,1))
            dtv(i,k) = -(um1(i,k)*duv(i,k) + vm1(i,k)*dvv(i,k))/denom +
     $                 (thp(i,k)/psomc(i,k) - tm1(i,k))*rztodt
            t2(i,k)  = t2(i,k) + dtv(i,k)
         end do
         do m=1,pcnst
            do i=1,plon
               dqv(i,k,m) = (qp1(i,k,m) - qm1(i,k,m))*rztodt
               qm1(i,k,m) = qp1(i,k,m)
            end do
         end do
      end do
C
C Save the pbl output variables on the history file. The other
C vertical diffusion variables are saved later, after Rayleigh 
C friction tendencies have been added.
C
      call outfld('PBLH    ',pblh ,plond,lat,hbuf,lhbuf)
      call outfld('TPERT   ',tpert,plond,lat,hbuf,lhbuf)
      call outfld('QPERT   ',qpert,plond,lat,hbuf,lhbuf)
      call outfld('USTAR   ',ustar,plond,lat,hbuf,lhbuf)
C
C If you want to uncomment the following outfld calls you must modify
C the argument list to vdiff
C
C      call outfld ('T10     ',t10,plond,nrow,hbuf,lhbuf)
C      call outfld ('Q10     ',q10,plond,nrow,hbuf,lhbuf)
C      call outfld ('CGH     ',cgh(1,2)  ,plond,nrow,hbuf,lhbuf)
C      call outfld ('CGQ     ',cgq(1,2,1),plond,nrow,hbuf,lhbuf)
C      call outfld('KVH     ',kvh(1,2),plond,lat,hbuf,lhbuf)
C      call outfld('KVM     ',kvm(1,2),plond,lat,hbuf,lhbuf)
      call outfld ('CGS     ',cgs(1,2)  ,plond,lat,hbuf,lhbuf)
      do m=1,pcnst
         call outfld(vdiffnam(m),dqv(1,1,m),plond,lat,hbuf,lhbuf)
      end do
C
      return
      end
