#include "params.h"
      subroutine radinp(dodiavg ,clat    ,pmid    ,pint    ,h2ommr  ,
     $                  cld     ,plos    ,pmidrd  ,pintrd  ,plco2   ,
     $                  plh2o   ,tclrsf  ,eccf    ,coszrs  ,o3mmr   ,
     $			o3vmr)
C-----------------------------------------------------------------------
C
C Sets latitude and time dependent arrays for input to solar
C and longwave radiation.
C
C Computes solar input (earth-sun distance factor and cosine solar
C zenith angle), converts model pressures to cgs, computes
C path length arrays needed for the longwave radiation,
C and computes ozone mixing ratio, needed for the solar
C radiation.
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          J. Kiehl, B. Briegleb, August 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Parameters-------------------------------
#include "prgrid.com"
c-----------------------------------------------------------------------
#ifdef PP_PCCM
#include "pspect.com"
#include "comparll.com"
#endif
C------------------------------Commons----------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#include "crdcon.com"
C-----------------------------------------------------------------------
#include "comvmr.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      logical dodiavg          ! true => do diurnal averaging
      real clat,               ! Current latitude (radians)
     $     pmid(plond,plev),   ! Pressure at model mid-levels (pascals)
     $     pint(plond,plevp),  ! Pressure at model interfaces (pascals)
     $     h2ommr(plond,plev), ! H2o mass mixing ratio
     $     cld(plond,plevp),   ! Fractional cloud cover
     $     plos(plond,plevp)   ! O3 path length (cm)

c added for CCM3 version
      real o3vmr(plond,plev)   ! ozone volume mixing ratio

C
C Output arguments
C
      real pmidrd(plond,plev), ! Pressure at mid-levels (dynes/cm*2)
     $     pintrd(plond,plevp),! Pressure at interfaces (dynes/cm*2)
     $     plco2(plond,plevp), ! Vert. pth lngth of co2 (prs-weighted)
     $     plh2o(plond,plevp), ! Vert. pth lngth h2o vap.(prs-weighted)
     $     tclrsf(plond,plevp) ! Product of clr-sky fractions from top
C                              ! of atmosphere to level.
      real eccf,               ! Earth-sun distance factor
     $     coszrs(plond),      ! Cosine solar zenith angle
     $     o3mmr(plond,plev)   ! Ozone mass mixing ratio
C
C---------------------------Local variables-----------------------------
C
      integer i,    ! Longitude loop index
     $        k     ! Vertical loop index
C
      real phi,     ! Greenwich calendar day + local time + long offset
     $     theta,   ! Earth orbit seasonal angle in radians
     $     delta,   ! Solar declination angle  in radians
     $     sinc,    ! Sine   of latitude
     $     cosc,    ! Cosine of latitude
     $     sind,    ! Sine   of declination
     $     cosd,    ! Cosine of declination
     $     v0 ,     ! Volume of a gas at stp (cm**3/mol)
     $     p0 ,     ! Standard pressure (dynes/cm**2)
     $     amd,     ! Effective molecular weight of dry air (g/mol)
     $     amo,     ! Molecular weight of ozone (g/mol)
     $     amco2,   ! Molecular weight of co2   (g/mol)
     $     cpwpl,   ! Const in co2 mixing ratio to path length conversn
     $     vmmr     ! Ozone volume mixing ratio
      real frac,    ! Daylight fraction
     $     arg,     ! ?
     $     tsun,    ! temporary term in diurnal averaging
     $     coszrsu, ! uniform cosine zenith solar angle 

C    additional variables for new eccentricity calculation.
     $     anper,   ! #degrees betw vernal equinox and perihelion
     $     ecc,     ! eccentricity
     $     tiltd,   ! tilt angle(obliquity)
     $     dayp,    ! #days from jan1(modern) to date of perihelion
     $     tiltr,omega,coef1,coef2,coef3,xnu,denom,xcel

      save     v0,   p0   ,amd   ,amo  ,amco2
C
      data v0    /  22413.6   /
      data p0    /  1.01325e6 /
      data amd   /  28.9644   /
      data amo   /  48.0000   /
      data amco2 /  44.0000   /
C
C-----------------------------------------------------------------------
C
C Compute solar distance factor and cosine solar zenith angle using
C day value where a round day (such as 213.0) refers to 0z at
C Greenwich longitude.
C
C   
C DON'T USE formulas from Paltridge, G.W. and C.M.R. Platt 1976: Radiative
C Processes in Meterology and Climatology, Elsevier Scientific
C Publishing Company, New York  p. 57, p. 62,63.
C
C INSTEAD:
C Use general earth orbital formulas(e.g Smart,1977).
C Formulas adapted by R. Gallimore (gallimor@fig.aos.wisc.edu). 
C Key input parameters are anper=#degrees between vernal 
C equinox and perihelion measured counterclockwise to earth's 
C orbit, ecc=eccentricity,
C tiltd= tilt angle(oblquity) of earth to plane of the eccliptic,
C dayp=number  of days from jan1(modern) to date of perihelion
C
#undef CALDEF
#ifdef MODSOL
C MODERN VALUES BELOW
      data anper/77.9610/
      data ecc/0.016724/
      data tiltd/23.4463/
      data dayp/2.00/
# define CALDEF
#endif

#ifdef SOL3KY
C 3000YRBP VALUES BELOW
      data anper/128.9253/
      data ecc/0.017824/
      data tiltd/23.8205/
      data dayp/310.767/
# define CALDEF
#endif
 
#ifdef SOL6KY
C 6000YRBP VALUES BELOW
      data anper/179.1304/
      data ecc/0.018682/
      data tiltd/24.1054/
      data dayp/258.993/
# define CALDEF
#endif
 
#ifdef SOL8KY
C 8000YRBP VALUES BELOW
      data anper/212.2577/
      data ecc/0.019110/
      data tiltd/24.2092/
      data dayp/224.649/
# define CALDEF
#endif

#ifdef SOL11KY
C 11000YRBP VALUES BELOW
      data anper/261.5920/
      data ecc/0.019529/
      data tiltd/24.2012/
      data dayp/174.280/
# define CALDEF
#endif
 
#ifdef SOL21KY
C 21000YRBP VALUES BELOW
      data anper/65.5750/
      data ecc/0.018994/
      data tiltd/22.9490/
      data dayp/14.4850/
# define CALDEF
#endif
 
#ifdef SOL126KY
C 126000YRBP VALUES BELOW
      data anper/248.7659/
      data ecc/0.039710/
      data tiltd/23.9281/
      data dayp/185.037/
# define CALDEF
#endif

#ifndef CALDEF
^^^^^^^^^^^^^^^^^ Solar Calendar not defined.  See params.h ^^^^^^^^^^^^^^^^^^^^
#endif
C
C Compute tiltr,omega, and xnu for solar declination calculation
c and eccentricity calc
      tiltr=2.*pie*tiltd/360.
      omega=360.-anper
      coef1=2.*ecc-1./4.*ecc**3
      coef2=5./4.*ecc**2
      coef3=13./12.*ecc**3
      denom=(1.-ecc**2)**2
      theta=2.*(pie/dayspy)*(calday-dayp)
      xnu=theta+coef1*sin(theta)+coef2*sin(2.*theta)+coef3*sin(3.*theta)

C Compute eccentricity factor (sun-earth distance factor)
C
      eccf=(1.+ecc*cos(xnu))**2/denom

C Solar declination in radians:
C
      xcel=xnu+2.*pie*omega/360.
      delta=asin(sin(tiltr)*sin(xcel))

CCCCCCCCCC Remaining algorithm is unaltered from Paltridge and Platt-based code
C
C Compute local cosine solar zenith angle,
C
      sinc = sin(clat)
      sind = sin(delta)
      cosc = cos(clat)
      cosd = cos(delta)
C
C If using diurnal averaging, then compute the average local cosine solar 
C zenith angle using formulas from paltridge and platt 1976  p. 57, p. 62,63.
C
      if (dodiavg) then
         arg = -(sinc/cosc)*(sind/cosd)
         if (arg .lt. -1.) then
            frac = 1.0
         else if (arg .gt. 1.) then
            frac = 0.0
         else
            frac = (1./pie)*acos(arg)
         endif
         tsun = pie*frac
C
C Modify earth eccentricity factor (eccf) by day fraction for later use in
C RADCSW.
C
         eccf = eccf*frac
         if (tsun .gt. 0.) then
            coszrsu =  sinc*sind + (cosc*cosd*sin(tsun))/tsun
         else
            coszrsu = 0.0
         endif
         do i=1,plon
            coszrs(i) = coszrsu
         end do
      else                       ! No diurnal averaging
C
C Calday is the calender day for Greenwich, including fraction
C of day; the fraction of the day represents a local time at
C Greenwich; to adjust this to produce a true instantaneous time
C For other longitudes, we must correct for the local time change:
C local time based on the longitude and day of year
C then compute the local cosine solar zenith angle
C
         do i=1,plon
#ifdef PP_PCCM
            phi = calday + (float(i-1+(m_pin_ln*plon))/float(p_glon))
#else
            phi       = calday + (float(i-1)/float(plon))
#endif
            coszrs(i) = sinc*sind - cosc*cosd*cos(2.*pie*phi)
         end do
      end if
C
C Convert pressure from pascals to dynes/cm2
C
      do k=1,plev
         do i=1,plon
            pmidrd(i,k) = pmid(i,k)*10.0
            pintrd(i,k) = pint(i,k)*10.0
         end do
      end do
      do i=1,plon
         pintrd(i,plevp) = pint(i,plevp)*10.0
      end do
C
C Compute path quantities used in the longwave radiation:
C
      vmmr  = amco2/amd
      cpwpl = vmmr*0.5/(gravit*p0)
      do i=1,plon
         plh2o(i,1)  = rgsslp*h2ommr(i,1)*pintrd(i,1)*pintrd(i,1)
         plco2(i,1)  = co2vmr*cpwpl*pintrd(i,1)*pintrd(i,1)
         tclrsf(i,1) = 1.
      end do
      do k=1,plev
         do i=1,plon
            plh2o(i,k+1)  = plh2o(i,k) + rgsslp*
     $             (pintrd(i,k+1)**2 - pintrd(i,k)**2)*h2ommr(i,k)
            plco2(i,k+1)  = co2vmr*cpwpl*pintrd(i,k+1)**2
            tclrsf(i,k+1) = tclrsf(i,k)*(1.-cld(i,k+1))
         end do
      end do
C
C Convert ozone volume mixing ratio to mass mixing ratio:
C
      vmmr  = amd / amo
      do k=1,plev
       do i=1,plon
	 o3mmr(i,k) = vmmr*o3vmr(i,k)
       end do
      end do

C
      return
      end
