#include "params.h"
      subroutine rfintr(qm1     ,u       ,v       ,ztodt   ,fu      ,
     $                  fv      ,dtv     ,duv     ,dvv     ,t2      ,
     $                  up1     ,vp1     )
C-----------------------------------------------------------------------
C
C Interface routine for Rayleigh friction. Computes the implied frictional
C heating from rayleigh friction momentum tendency and sets the model 
C tendency arrays.
C 
C Using the standard parameters set in "rfinti", the rayleigh friction
C are zero at all levels of the standard 18 level model. The coeffs
C are nonzero only above about 2 mb in the middle atmosphere version 
C of the model.
C
C---------------------------Code history--------------------------------
C
C Original version:  B. Boville
C Standardized:      J. Rosinski, June 1992
C Reviewed:          P. Rasch, B. Boville, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
C------------------------------Commons----------------------------------
#include "comrf.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      real qm1(plond,plev),    ! input moisture
     $     u(plond,plev),      ! unfiltered u-wind
     $     v(plond,plev),      ! unfiltered v-wind
     $     ztodt               ! 2 delta-t
C
C Input/Output arguments
C
      real fu(plond,plev),     ! cumulative dudt
     $     fv(plond,plev),     ! cumulative dvdt
     $     dtv(plond,plev),    ! heating rate due to vdiff & rfric
     $     duv(plond,plev),    ! u-wind tendency due to vdiff & rfric
     $     dvv(plond,plev),    ! v-wind tendency due to vdiff & rfric
     $     t2(plond,plev),     ! cumulative heating rate
     $     up1(plond,plev),    ! u-wind after vdiff & rfric
     $     vp1(plond,plev)     ! v-wind after vdiff & rfric
C
C---------------------------Local workspace-----------------------------
C
      real dudt(plond,plev),   ! time rate of change of u-wind
     $     dvdt(plond,plev),   ! time rate of change of v-wind
     $     delduv,             ! change in duv
     $     deldvv,             ! change in dvv
     $     denom               ! denominator of expression
      integer i,k              ! longitude,level indices
C
C-----------------------------------------------------------------------
C
      if (ntoprl.le.nbotrl) then
C
C Compute momemtum tendency from raleigh friction
C
         call rfric(up1,vp1,ztodt,dudt,dvdt)
C
C Set model tendency arrays for momentum and temperature.  Also add
C Rayleigh friction tendencies to the diagnostic tendencies from the 
C vertical diffusion.
C
         do k=ntoprl,nbotrl
            do i=1,plon
               fu(i,k)  = fu(i,k)  + dudt(i,k)
               fv(i,k)  = fv(i,k)  + dvdt(i,k)
               duv(i,k) = duv(i,k) + dudt(i,k)
               dvv(i,k) = dvv(i,k) + dvdt(i,k)
               denom  = cpair*(1. + cpvir*qm1(i,k))
               delduv = -u(i,k)*dudt(i,k)/denom
               deldvv = -v(i,k)*dvdt(i,k)/denom
               t2(i,k)  = t2(i,k)  + delduv + deldvv
               dtv(i,k) = dtv(i,k) + delduv + deldvv
            end do
         end do
      end if
C
      return
      end
