#include "params.h"
      subroutine rfric(up1     ,vp1     ,ztodt   ,dudt    ,dvdt    )
C-----------------------------------------------------------------------
C
C Compute u and v tendencies due to Rayleigh friction.
C
C---------------------------Code history--------------------------------
C
C Original version:  B. Boville
C Standardized:      J. Rosinski, June 1992
C Reviewed:          P. Rasch, B. Boville, August 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
C------------------------------Commons----------------------------------
#include "comrf.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      real up1(plond,plev),  ! u after modification by VDIFF
     $     vp1(plond,plev),  ! v after modification by VDIFF
     $     ztodt             ! 2*delta-t
C
C Output arguments
C
      real dudt(plond,plev), ! u time rate of change
     $     dvdt(plond,plev)  ! v time rate of change
C
C---------------------------Local workspace-----------------------------
C
      real up2,              ! u-wind after rayleigh friction
     $     vp2,              ! v-wind after rayleigh friction
     $     tmp,              ! temporary workspace
     $     rztodt            ! 1./ztodt
      integer i,k            ! longitude, level indices
C
C-----------------------------------------------------------------------
C
C First use an Euler backward method to compute the new values.
C Then compute tendencies from the new values
C
      rztodt = 1./ztodt
      do k=ntoprl,nbotrl
         tmp = 1./(1.0 + ztodt*rlghco(k))
         do i=1,plon
            up2 = up1(i,k)*tmp
            vp2 = vp1(i,k)*tmp
            dudt(i,k) = (up2 - up1(i,k))*rztodt
            dvdt(i,k) = (vp2 - vp1(i,k))*rztodt
         end do
      end do
C
      return
      end
