#include "params.h"
      subroutine grmult(doslt   ,rcoslat ,d       ,q       ,t       ,
     $                  u       ,v       ,z       ,tm1    ,phis    ,
     $                  dpsl    ,dpsm    ,omga    ,pdel    ,pbot    ,
     $                  pstar   ,pstarmln,pmln    ,piln    ,rpmid   ,
     $                  rpdel   ,fu      ,fv      ,t2      ,ut      ,
     $                  vt      ,uq      ,vq      ,drhs    ,pmid    ,
     $                  etadot  ,etamid  ,engy    ,ddpn    ,vpdsn   ,
     $                  dqdt    ,dpslon  ,dpslat  )
C-----------------------------------------------------------------------
C
C Non-linear dynamics calculations in grid point space
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          B. Boville, D. Williamson, J. Hack, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pspect.com"
C------------------------------Commons----------------------------------
#include "comcon.com"
C-----------------------------------------------------------------------
#include "comhyb.com"
C-----------------------------------------------------------------------
#include "commap.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      logical doslt            ! semi-lagrangian or spectral transport
      real rcoslat,            ! 1./cosine(latitude)
     $     d(plond,plev),      ! divergence
     $     q(plond,plev),      ! specific humidity
     $     t(plond,plev),      ! temperature
     $     u(plond,plev),      ! zonal wind * cos(lat)
     $     v(plond,plev),      ! meridional wind * cos(lat)
     $     z(plond,plev),      ! vorticity
     $     phis(plond),        ! surface geopotential
     $     dpsl(plond),        ! longitudinal component of grad ln(ps)
     $     dpsm(plond),        ! latitudinal component of grad ln(ps)
     $     omga(plond,plev),   ! vertical pressure velocity
     $     pdel(plond,plev),   ! layer thicknesses (pressure)
     $     pbot(plond),        ! bottom interface pressure
     $     pstar(plond),       ! ps + psr
     $     pstarmln(plond),    ! log pstarm
     $     pmln(plond,plev),   ! alog (mid-level pressure)
     $     piln(plond,plevp),  ! alog (interface pressure)
     $     rpmid(plond,plev),  ! 1./pmid
     $     rpdel(plond,plev),  ! 1./pdel
     $     tm1(plond,plev)    ! perturbation temperature at previous time step
C
C Input/Output arguments
C
      real fu(plond,plev),     ! nonlinear term - u momentum eqn
     $     fv(plond,plev),     ! nonlinear term - v momentum eqn
     $     t2(plond,plev),     ! nonlinear term - temperature
     $     ut(plond,plev),     ! (u*T) - heat flux - zonal 
     $     vt(plond,plev),     ! (u*T) - heat flux - meridional
     $     uq(plond,plev),     ! (u*q) - water flux - zonal
     $     vq(plond,plev),     ! (v*q) - water flux - meridional
     $     drhs(plond,plev),   ! RHS of divergence eqn (del^2 term)
     $     pmid(plond,plev),   ! pressure at full levels
     $     etadot(plon,plevp), ! vertical velocity in eta coordinates
     $     etamid(plev),       ! midpoint values of eta (a+b)
     $     engy(plond,plev)    ! kinetic energy
C
C Output arguments
C
      real ddpn(plond),        ! complete sum of d*delta p
     $     vpdsn(plond),       ! complete sum V dot grad(ln(ps)) delta b
     $     dqdt(plond,plev)    ! moisture tendency (spectral transport only)
      real dpslat(plond,plev),
     $     dpslon(plond,plev)
C
C---------------------------Local workspace-----------------------------
C
      real tv(plond,plev),     ! virtual temperature
     $     ddpk(plond),        ! partial sum of d*delta p
     $     vkdp,               ! V dot grad(ln(ps))
     $     vpdsk(plond),       ! partial sum  V dot grad(ln(ps)) delta b
     $     qk0(plond),         ! q at phony level 0
     $     tk0(plond),         ! t at phony level 0
     $     uk0(plond),         ! u at phony level 0
     $     vk0(plond),         ! v at phone level 0
     $     rtv(plond,plev),    ! rair*tv
     $     pstarln(plond),     ! log pstar
     $     pterm(plond,plev),  ! intermediate term for hydrostatic eqn
     $     tterm(plond,plev),  ! intermediate term for hydrostatic eqn
     $     tp(plond,plev),     ! perturbation temperature
     $     hreft0,             ! hydrostatic matrix element * ref temp
     $     tmp,                ! temporary workspace
     $     tmpk,
     $     tmpkp1
      real edotdpde(plond,plevp),            ! etadot*dp/deta
     $     udel(plond,0:plev-1),
     $     vdel(plond,0:plev-1),
     $     tdel(plond,0:plev-1)
      integer
     $     i,k,kk              ! longitude, level indices
C
C Initialize arrays which represent vertical sums (ddpk, ddpn, vpdsk,
C vpdsn).  Set upper boundary condition arrays (k=0: qk0, tk0, uk0, vk0).
C Compute log(pstar).
C
      do i=1,plon
         ddpk(i)  = 0.0
         ddpn(i)  = 0.0
         vpdsk(i) = 0.0
         vpdsn(i) = 0.0
         qk0(i)  = 0.0
         tk0(i)  = 0.0
         uk0(i)  = 0.0
         vk0(i)  = 0.0
         pstarln(i)  = alog(pstar(i))
      end do
C
C Virtual temperature
C
      call virtem(t,q,zvir,tv)
C
C sum(plev)(d(k)*dp(k))
C
      do k=1,plev
         do i=1,plon
            ddpn(i) = ddpn(i) + d(i,k)*pdel(i,k)
            rtv(i,k) = rair*tv(i,k)
         end do
      end do
C
C sum(plev)(v(k)*grad(lnps)*db(k))
C
      do k=nprlev,plev
         do i=1,plon
            vkdp = rcoslat*(u(i,k)*dpsl(i) + v(i,k)*dpsm(i))*pbot(i)
            vpdsn(i) = vpdsn(i) + vkdp*hybd(k)
         end do
      end do
C
C Compute etadot (dp/deta) (k+1/2).  Note: sum(k)(d(j)*dp(j)) required in
C pressure region. sum(k)(d(j)*dp(j)) and sum(k)(v(j)*grad(pstar)*db(j)) 
C required in hybrid region
C
      do i=1,plon
         edotdpde(i,1) = 0.
         edotdpde(i,plevp) = 0.
      end do
      do k=1,nprlev-1
         do i=1,plon
            ddpk(i) = ddpk(i) + d(i,k)*pdel(i,k)
            edotdpde(i,k+1) = -ddpk(i)
         end do
      end do
      do k=nprlev,plev-1
         do i=1,plon
            ddpk(i) = ddpk(i) + d(i,k)*pdel(i,k)
            vkdp = rcoslat*(u(i,k)*dpsl(i) + v(i,k)*dpsm(i))*pbot(i)
            vpdsk(i) = vpdsk(i) + vkdp*hybd(k)
            edotdpde(i,k+1) = -ddpk(i) - vpdsk(i) +
     $                        hybi(k+1)*(ddpn(i)+vpdsn(i))
         end do
      end do
C
C Nonlinear advection terms.  u*t, v*t, kinetic energy first
C
      do k=1,plev
         do i=1,plon
            ut(i,k) = u(i,k)*t(i,k)
            vt(i,k) = v(i,k)*t(i,k)
            engy(i,k) = 0.5*(u(i,k)**2 + v(i,k)**2)
         end do
      end do
C
C Compute workspace arrays for delta-u, delta-v, delta-t (k)
C
      do i=1,plon
         udel(i,0) = u(i,1) - uk0(i)
         vdel(i,0) = v(i,1) - vk0(i)
         tdel(i,0) = t(i,1) - tk0(i)
      end do
      do k=1,plev-1
         do i=1,plon
            udel(i,k) = u(i,k+1) - u(i,k)
            vdel(i,k) = v(i,k+1) - v(i,k)
            tdel(i,k) = t(i,k+1) - t(i,k)
         end do
      end do
C
C Horizontal advection: u*z, v*z, energy conversion term (omega/p),
C vertical advection for interface above.  Pure pressure region first.
C
      do k=1,nprlev-1
         do i=1,plon
            dpslat(i,k) = 0.
            dpslon(i,k) = 0.
            tmpk   = 0.5*rpdel(i,k)*edotdpde(i,k  )
            tmpkp1 = 0.5*rpdel(i,k)*edotdpde(i,k+1)
            fu(i,k) = fu(i,k) + v(i,k)*z(i,k) - udel(i,k-1)*tmpk -
     $                                          udel(i,k  )*tmpkp1
            fv(i,k) = fv(i,k) - u(i,k)*z(i,k) - vdel(i,k-1)*tmpk -
     $                                          vdel(i,k  )*tmpkp1
            t2(i,k) = t2(i,k) + d(i,k)*t(i,k) - tdel(i,k-1)*tmpk +
     $                cappa*tv(i,k)/(1. + cpvir*q(i,k))*
     $                omga(i,k)*rpmid(i,k) - tdel(i,k)*tmpkp1
         end do
      end do
C
C Hybrid region above bottom level: Computations are the same as in pure
C pressure region, except that pressure gradient terms are added to 
C momentum tendencies.
C
      do k=nprlev,plev-1
         do i=1,plon
            tmpk   = 0.5*rpdel(i,k)*edotdpde(i,k  )
            tmpkp1 = 0.5*rpdel(i,k)*edotdpde(i,k+1)
            tmp = rtv(i,k)*hybm(k)*rpmid(i,k)*pbot(i)
            dpslon(i,k) = rcoslat*tmp*dpsl(i)
            dpslat(i,k) = rcoslat*tmp*dpsm(i)
            fu(i,k) = fu(i,k) + v(i,k)*z(i,k) - udel(i,k-1)*tmpk -
     $                                          udel(i,k  )*tmpkp1 -
     $                dpslon(i,k)
            fv(i,k) = fv(i,k) - u(i,k)*z(i,k) - vdel(i,k-1)*tmpk -
     $                                          vdel(i,k  )*tmpkp1 -
     $                dpslat(i,k)
            t2(i,k) = t2(i,k) + d(i,k)*t(i,k) - tdel(i,k-1)*tmpk +
     $                cappa*tv(i,k)/(1. + cpvir*q(i,k))*
     $                omga(i,k)*rpmid(i,k) - tdel(i,k)*tmpkp1
         end do
      end do
C
C Bottom level
C
      do i=1,plon
         tmpk = 0.5*rpdel(i,plev)*edotdpde(i,plev  )
         tmp  = rtv(i,plev)*hybm(plev)*rpmid(i,plev)*pbot(i)
         dpslon(i,plev) = rcoslat*tmp*dpsl(i)
         dpslat(i,plev) = rcoslat*tmp*dpsm(i)
         fu(i,plev) = fu(i,plev) + v(i,plev)*z(i,plev) -
     $                udel(i,plev-1)*tmpk - dpslon(i,plev)
         fv(i,plev) = fv(i,plev) - u(i,plev)*z(i,plev) -
     $                vdel(i,plev-1)*tmpk - dpslat(i,plev)
         t2(i,plev) = t2(i,plev) + d(i,plev)*t(i,plev) -
     $                tdel(i,plev-1)*tmpk +
     $                cappa*tv(i,plev)/(1. + cpvir*q(i,plev))*
     $                omga(i,plev)*rpmid(i,plev)
      end do
C
C Convert eta-dot(dp/deta) to eta-dot (top and bottom = 0.)
C
      do i=1,plon
         etadot(i,1) = 0.
         etadot(i,plevp) = 0.
      end do
      do k=2,plev
         tmp = etamid(k) - etamid(k-1)
         do i=1,plon
            etadot(i,k) = edotdpde(i,k)*tmp/(pmid(i,k) - pmid(i,k-1))
         end do
      end do
C
C-----------------------------------------------------------------------
C
C Divergence and hydrostatic equations
C Store some temporary terms
C
      do k=2,plev-1
         do i=1,plon
            pterm(i,k) = rtv(i,k)*rpmid(i,k)*pdel(i,k)
            tterm(i,k) = 0.5*tm1(i,k) - t(i,k)
         end do
      end do
      do i=1,plon
         tterm(i,1) = 0.5*tm1(i,1) - t(i,1)
         tterm(i,plev) = 0.5*tm1(i,plev) - t(i,plev)
      end do
C
C Del squared part of RHS of divergence equation.
C Kinetic energy and diagonal term of hydrostatic equation.
C Total temperature as opposed to  perturbation temperature is acceptable
C since del-square operator will operate on this term.

C
      do k=1,plev-1
         do i=1,plon
            drhs(i,k) = phis(i) + engy(i,k) + rtv(i,k)*0.5*
     $                  rpmid(i,k)*pdel(i,k) +
     $                  href(k,k)*tterm(i,k) +
     $                  bps(k)*(0.5*pstarmln(i) - pstarln(i))
         end do
      end do
      do i=1,plon
         drhs(i,plev) = phis(i) + engy(i,plev) + rtv(i,plev)*0.5*
     $                  rpmid(i,plev)*pdel(i,plev) +
     $                  href(plev,plev)*tterm(i,plev) +
     $                  bps(plev)*(0.5*pstarmln(i) - pstarln(i))
      end do
C
C Bottom level term of hydrostatic equation
C
      do k=1,plev-1
         do i=1,plon
            drhs(i,k) = drhs(i,k) + rtv(i,plev)*
     $             rpmid(i,plev)*pdel(i,plev) +
     $             href(plev,k)*tterm(i,plev)
         end do
      end do
C
C Interior terms of hydrostatic equation
C
      do k=1,plev-2
         do kk=k+1,plev-1
            do i=1,plon
               drhs(i,k) = drhs(i,k) + pterm(i,kk) +
     $                     href(kk,k)*tterm(i,kk)
            end do
         end do
      end do
C
C If spectral transport, compute appropriate nonlinear advection terms and
C moisture tendency.
C
      if (.not.doslt) then
         do k=1,plev
            do i=1,plon
               uq(i,k) = u(i,k)*q(i,k)
               vq(i,k) = v(i,k)*q(i,k)
            end do
         end do
C
C Top level
C
         do i=1,plon
            tmpk   = edotdpde(i,1)*0.5*rpdel(i,1)
            tmpkp1 = edotdpde(i,2)*0.5*rpdel(i,1)
            dqdt(i,1) = d(i,1)*q(i,1) - (q(i,1) - qk0(i))*tmpk -
     $                                  (q(i,2) - q(i,1))*tmpkp1
         end do
C
C Intermediate levels
C
         do k=2,plev-1
            do i=1,plon
               tmpk   = edotdpde(i,k  )*0.5*rpdel(i,k)
               tmpkp1 = edotdpde(i,k+1)*0.5*rpdel(i,k)
               dqdt(i,k) = d(i,k)*q(i,k) - (q(i,k  ) - q(i,k-1))*tmpk -
     $                                     (q(i,k+1) - q(i,k  ))*tmpkp1
            end do
         end do
C
C Bottom level
C
         do i=1,plon
            tmpk = edotdpde(i,plev)*0.5*rpdel(i,plev)
            dqdt(i,plev) = d(i,plev)*q(i,plev) -
     $                     (q(i,plev) - q(i,plev-1))*tmpk
         end do
      end if
C    
      return
      end
