#include "params.h"
      subroutine physics1(irow   ,lat     ,tdt     ,ps      ,psm1    ,
     $                  u3      ,u3m1    ,v3      ,v3m1    ,t3      ,
     $                  t3m1    ,q3      ,q3m1    ,etadot  ,psslt   ,
     $                  etamid  ,ztodt   ,vmax    ,vmaxt   ,vcour   ,
     $                  detam   ,grlps1  ,grt1    ,grq1    ,grz1    ,
     $                  grd1    ,grfu1   ,grfv1   ,grut1   ,gruq1   ,
     $                  grvt1   ,grvq1   ,grrh1   ,grlps2  ,grt2    ,
     $                  grq2    ,grz2    ,grd2    ,grfu2   ,grfv2   ,
     $                  grut2   ,gruq2   ,grvt2   ,grvq2   ,grrh2   ,
     $                  hbuf    ,lhbuf   ,psm2    ,bpstr   ,um2     ,
     $                  vm2     ,tm2     ,qm2     ,qminus  ,vort    ,
     $                  vortm1  ,vorttl  ,div     ,divm1   ,divtl   ,
     $                  dpsl    ,dpsm    ,ts      ,ws      ,sn      ,
     $                  oro     ,rnfp1   ,phis    ,pblht   ,qpert   ,
     $                  tpert   ,duh     ,dvh     ,dth     ,qrs     ,
     $                  qrl     ,flwds   ,fsns    ,plos    ,plol    ,
#ifndef BLOCK_FFT
     $                  tssub   )
#else
     $                  tssub   ,fft_buffer, ip )
#endif
C-----------------------------------------------------------------------
C
C Control adjustment physics, time filter, history tape write-up,
C tendency physics, non-linear dynamical terms, FFT and combine terms
C in preparation for Fourier -> spectral quadrature.
C
C The naming convention is as follows:
C  - prefix gr contains grid point values before FFT and Fourier
C     coefficients after
C  - t, q, d, z and ps refer to temperature, specific humidity,
C     divergence, vorticity and surface pressure
C  - "1" suffix to an array => symmetric component current latitude pair
C  - "2" suffix to an array => antisymmetric component.
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          D. Williamson, J. Hack, August 1992
C
C physics split for coupling: C. Schafer, R. Jacob, April 1995
C additions to work with CCM3:  R. Jacob, December 1996
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
C------------------------------Commons----------------------------------
#include "comcon.com"
C-----------------------------------------------------------------------
#include "comctl.com"
C-----------------------------------------------------------------------
#include "comfft.com"
C-----------------------------------------------------------------------
#include "comhed.com"
C-----------------------------------------------------------------------
#include "comhst.com"
C-----------------------------------------------------------------------
#include "comhyb.com"
C-----------------------------------------------------------------------
#include "comlun.com"
C-----------------------------------------------------------------------
#include "commap.com"
C-----------------------------------------------------------------------
#include "commss.com"
C-----------------------------------------------------------------------
#include "comqfl.com"
C-----------------------------------------------------------------------
#include "comslt.com"
C-----------------------------------------------------------------------
#include "comspe.com"
C-----------------------------------------------------------------------
#include "comtfc.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#ifdef COUPLED
#include "pccmcoup.com"
#endif
C-----------------------------------------------------------------------
#include "comtrcnm.com"
#ifdef INTERVAL_TIMER
#include "timers.com"
#endif
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer
     $     irow,                     ! latitude pair index
     $     lat,                      ! latitude index for S->N storage
     $     lhbuf                     ! length history tape buffer, hbuf
      real tdt,                      ! two delta t (unless nstep<2)
     $     ps(plond),                ! surface pressure (time n)
     $     psm1(plond),              ! surface pressure (time n-1)
     $     u3(plond,plev),           ! u-wind (time n)
     $     u3m1(plond,plev),         ! u-wind (time n-1)
     $     v3(plond,plev),           ! v-wind (time n)
     $     v3m1(plond,plev),         ! v-wind (time n-1)
     $     t3(plond,plev),           ! temperature (time n)
     $     t3m1(plond,plev),         ! temperature (time n-1)
     $     q3(plond,plev,pcnst),     ! constituent conc(time n  :h2o first)
     $     q3m1(plond,plev,pcnst),   ! constituent conc(time n-1:h2o first)
     $     etadot(plon,plevp),       ! vertical motion (3-d used by slt)
     $     psslt(plond),             ! surface pressure (2-d used by slt)
     $     etamid(plev),             ! midpoint values of eta (a+b)
     $     ztodt,                    ! 2*timestep unless nstep = 0
     $     vmax(plev),               ! maximum wind speed squared (m^2/s^2)
     $     vmaxt(plev),              ! maximum truncated wind speed (m^2/s^2)
     $     vcour(plev),              ! maximum Courant number in vert.
     $     detam(plev)               ! intervals between vert.levels
#ifdef PP_PCCM
c parallel fft needs entire width
      real grlps1(plon),          ! sym. undiff. term in lnps eqn.
     $     grlps2(plon),          ! antisym undiff. term in lnps eqn.
     $     grt1(plon,plev),       ! sym. undiff. term in t eqn.
     $     grt2(plon,plev),       ! antisym. undiff. term in t eqn.
     $     grq1(plon,plev),       ! sym. undiff. term in q eqn.
     $     grq2(plon,plev),       ! antisym. undiff. term in q eqn.
     $     grz1(plon,plev),       ! sym. undiff. term in z eqn.
     $     grz2(plon,plev),       ! antisym. undiff. term in z eqn.
     $     grd1(plon,plev),       ! sym. undiff. term in d eqn.
     $     grd2(plon,plev),       ! antisym. undiff. term in d eqn.
     $     grfu1(plon,plev),      ! sym. nonlinear terms in u eqn.
     $     grfu2(plon,plev),      ! antisym. nonlinear terms in u eqn.
     $     grfv1(plon,plev),      ! sym. nonlinear terms in v eqn.
     $     grfv2(plon,plev),      ! antisym. nonlinear terms in v eqn.
     $     grut1(plon,plev),      ! sym. lambda deriv. term in t eqn.
     $     grut2(plon,plev),      ! antisym. lambda deriv. term in t eqn.
     $     gruq1(plon,plev),      ! sym. lambda deriv. term in q eqn.
     $     gruq2(plon,plev),      ! antisym. lambda deriv. term in q eqn.
     $     grvt1(plon,plev),      ! sym. mu derivative term in t eqn.
     $     grvt2(plon,plev),      ! antisym. mu deriv. term in t eqn.
     $     grvq1(plon,plev),      ! sym. mu derivative term in q eqn.
     $     grvq2(plon,plev),      ! antisym. mu deriv. term in q eqn.
     $     grrh1(plon,plev),      ! sym. del**2 term in d eqn.
     $     grrh2(plon,plev)       ! antisym. del**2 term in d eqn.
#else
      real grlps1(2*pmmax),          ! sym. undiff. term in lnps eqn.
     $     grlps2(2*pmmax),          ! antisym undiff. term in lnps eqn.
     $     grt1(2*pmmax,plev),       ! sym. undiff. term in t eqn.
     $     grt2(2*pmmax,plev),       ! antisym. undiff. term in t eqn.
     $     grq1(2*pmmax,plev),       ! sym. undiff. term in q eqn.
     $     grq2(2*pmmax,plev),       ! antisym. undiff. term in q eqn.
     $     grz1(2*pmmax,plev),       ! sym. undiff. term in z eqn.
     $     grz2(2*pmmax,plev),       ! antisym. undiff. term in z eqn.
     $     grd1(2*pmmax,plev),       ! sym. undiff. term in d eqn.
     $     grd2(2*pmmax,plev),       ! antisym. undiff. term in d eqn.
     $     grfu1(2*pmmax,plev),      ! sym. nonlinear terms in u eqn.
     $     grfu2(2*pmmax,plev),      ! antisym. nonlinear terms in u eqn.
     $     grfv1(2*pmmax,plev),      ! sym. nonlinear terms in v eqn.
     $     grfv2(2*pmmax,plev),      ! antisym. nonlinear terms in v eqn.
     $     grut1(2*pmmax,plev),      ! sym. lambda deriv. term in t eqn.
     $     grut2(2*pmmax,plev),      ! antisym. lambda deriv. term in t eqn.
     $     gruq1(2*pmmax,plev),      ! sym. lambda deriv. term in q eqn.
     $     gruq2(2*pmmax,plev),      ! antisym. lambda deriv. term in q eqn.
     $     grvt1(2*pmmax,plev),      ! sym. mu derivative term in t eqn.
     $     grvt2(2*pmmax,plev),      ! antisym. mu deriv. term in t eqn.
     $     grvq1(2*pmmax,plev),      ! sym. mu derivative term in q eqn.
     $     grvq2(2*pmmax,plev),      ! antisym. mu deriv. term in q eqn.
     $     grrh1(2*pmmax,plev),      ! sym. del**2 term in d eqn.
     $     grrh2(2*pmmax,plev)       ! antisym. del**2 term in d eqn.
#endif
      real hbuf(lhbuf)               ! history tape buffer
      real psm2(plond),              ! surface pressure (time n-2)
     $     bpstr(plond),             ! storage next to vortm1 for ps transfm
     $     um2(plond,plev),          ! u-wind (time n-2)
     $     vm2(plond,plev),          ! v-wind (time n-2)
     $     tm2(plond,plev),          ! temperature (time n-2)
     $     qm2(plond,plev,pcnst),    ! constituent-h2o first (time n-2)
     $     qminus(plond,plev,pcnst), ! saves q being advected in slt code
     $     vort(plond,plev),         ! vorticity (time n)
     $     vortm1(plond,plev),       ! vorticity (time n-1)
     $     vorttl(plond,plev),       ! vorticity: half time-filtered
     $     div(plond,plev),          ! divergence (time n)
     $     divm1(plond,plev),        ! divergence (time n-1)
     $     divtl(plond,plev)         ! divergence: half time-filtered
      real dpsl(plond),              ! longitude derivative of lnps (time n)
     $     dpsm(plond),              ! latitude  derivative of lnps (time n)
     $     ts(plond),                ! surface temperature
     $     ws(plond),                ! surface wetness
     $     sn(plond),                ! equivalent liquid water snow depth
     $     oro(plond),               ! land/ocean/sea ice flag
     $     rnfp1(plond),             ! runoff
     $     phis(plond),              ! surface geopotential
     $     pblht(plond),             ! PBL height
     $     qpert(plond,pcnst)        ! moisture/constituent perturb. (PBL)
      real tpert(plond),             ! temperature perturbation (PBL)
     $     duh(plond,plev),          ! horizontal diffusion of u-momentum
     $     dvh(plond,plev),          ! horizontal diffusion of v-momentum
     $     dth(plond,plev),          ! horizontal temperature diffusion
     $     qrs(plond,plev),          ! shortwave radiative heating rate 
     $     qrl(plond,plev),          ! longwave  radiative heating rate 
     $     flwds(plond),             ! surface longwave down flux
     $     fsns(plond),              ! surface absorbed solar flux
     $     plos(plond,plevp),        ! O3 path length (cm)
     $     plol(plond,plevp),        ! O3 pressure-weighted pathlength(cm)
     $     tssub(plond,plevmx)       ! sub-surface soil temperatures
#ifdef BLOCK_FFT
      real fft_buffer(plond,*)
      integer ip
#endif
C
C---------------------------Local workspace-----------------------------
C
      real cnt(plond,plat),               ! top of convective layer
     $     cnb(plond,plat),               ! bottom of convective layer
     $     qc(plond,plev,plat),           ! q tendency due to total rainout
     $     vqfcst(plond,plev,pcnst), ! q tendency due to vert. adv. (slt)
     $     pmid(plond,plev,plat),         ! pressure at model levels (time n)
     $     pmidm1(plond,plev,plat),       ! pressure at model levels (time n-1)
     $     rpmid(plond,plev,plat),        ! 1./pmid
     $     pmln(plond,plev,plat),         ! alog(pmid)
     $     pint(plond,plevp,plat),        ! pressure at model interfaces (n  )
     $     pintm1(plond,plevp,plat),      ! pressure at model interfaces (n-1)
     $     pdel(plond,plev,plat),         ! pdel(k)   = pint  (k+1)-pint  (k)
     $     pdelm1(plond,plev,plat),       ! pdelm1(k) = pintm1(k+1)-pintm1(k)
     $     rpdel(plond,plev,plat),        ! 1./pdel
     $     piln(plond,plevp,plat)         ! alog(pint)
      real est(plond,plevp),         ! sat vap pressure
     $     qsat(plond,plevp),        ! saturation specific humidity
     $     psl(plond)                ! sea level pressure
      real pstar(plond,plat),             ! ps + psr (time n)
     $     pstarm(plond),            ! ps + psr (time n-1)
     $     pstarmln(plond,plat),          ! alog(pstarm)
     $     t2(plond,plev,plat),           ! nonlinear term - temperature
     $     engy(plond,plev)          ! kinetic energy
      real xnlin(plndlv*7)           ! non-linear terms (equivalence 
C                                    !  region for following arrays to 
C                                    !  optimize fft performance)
#ifndef BLOCK_FFT
      real xnlin_sv(plndlv*7,plat)  ! save these quantities for all lat (JM)
      common /xxxx/ xnlin_sv
#endif
      real fu(plond,plev),           ! nonlinear term - u momentum eqn.
     $     fv(plond,plev),           ! nonlinear term - v momentum eqn.
     $     ut(plond,plev),           ! (u*T) - heat flux - zonal
     $     vt(plond,plev),           ! (v*T) - heat flux - meridional
     $     drhs(plond,plev),         ! RHS of divergence eqn. (del^2 term)
     $     uq(plond,plev),           ! (u*q) - water flux - zonal
     $     vq(plond,plev)            ! (v*q) - water flux - meridional
#if ( defined SUN )
C
C Sun compiler does not allow "equivalence" for stack variables; make
C them static.
C
      static xnlin,fu,fv,ut,vt,drhs,uq,vq
#endif
C
C The following equivalences are for optimal fft performance
C
      equivalence (fu,xnlin(1)),
     $            (fv,xnlin(1+1*plndlv)),
     $            (ut,xnlin(1+2*plndlv)),
     $            (vt,xnlin(1+3*plndlv)),
     $            (drhs,xnlin(1+4*plndlv)),
     $            (uq,xnlin(1+5*plndlv)),
     $            (vq,xnlin(1+6*plndlv))
C
      real dqfx3(plond,plev,pcnst),  ! q tendency due to mass adjustment
     $     dtcond(plond,plev),       ! dT/dt due to convective adjustment
     $     dqcond(plond,plev,pcnst), ! dq/dt due to convective adjustment
     $     cmfdt(plond,plev,plat),        ! convective temperature tendency
     $     cmfdq(plond,plev,plat),        ! convective moisture tendency
     $     cmfdqr(plond,plev,plat),       ! dq/dt due to convective rainout
     $     cmfmc(plond,plev,plat),        ! convective mass flux--m sub c
     $     cmfsl(plond,plev,plat),        ! liquid water static energy flux
     $     cmflq(plond,plev,plat)         ! total water flux
      real ftem(plond,plev)          ! temporary workspace
      real omga(plond,plev,plat),         ! vertical velocity
     $     precl(plond,plat),             ! large scale stable precip rate
     $     precc(plond,plat),             ! convective precip rate
     $     precsl(plond,plat),            ! snow component large scale precip
     $     precsc(plond,plat)             ! snow component convective precip
C      real utend(plond,plev),        ! du/dt
C     $     vtend(plond,plev),        ! dv/dt
C     $     ttend(plond,plev),        ! dT/dt
C     $     qtend(plond,plev,pcnst),  ! dq/dt
C     $     pstend(plond),            ! d(ps)/dt
      real wind,                     ! u**2 + v**2 (m/s)
     $     lvcour,                   ! local vertical courant number
     $     om2eps,                   ! 1. - 2.*eps for time filter
     $     dtdz                      ! dt/detam(k)
      integer
     $     ioro(plond),              ! nint(oro(i))
     $     i,k,kk,m,                 ! longitude,level,constituent indices
     $     ilen,                     ! length to copy to avoid overrun
     $     inc,                      ! increment for fft991
     $     isign,                    ! flag indicates transform direction
     $     ntr,                      ! number of transforms to perform
     $     itape,
     $     kfld
      real ta(plond,plev,pcnst),     ! total advection of constituents
     $     ddivdt(plond,plev),       ! temporary workspace
     $     ddpn(plond),              ! complete sum of d*delta p
     $     vpdsn(plond),             ! complete sum V dot grad(ln(ps)) delta b
     $     dqdt(plond,plev)          ! moisture tendency (spectral transport)
      real dpslat(plond,plev),       ! Pressure gradient term 
     $     dpslon(plond,plev)        ! Pressure gradient term 
      real coslat,                   ! cosine(latitude)
     $     rcoslat,                  ! 1./cosine(latitude)
     $     rhypi,                    ! 1./hypi(plevp)
     $     utfac,                    ! Only Byron knows
     $     vtfac                     ! Only Byron knows
      real numomgup(plond,plev),     ! number of times wind directed +z
     $     omegaup(plond,plev),      ! omega during omega < 0.
     $     tvm(plond,plev),          ! virtual temperature 
     $     zm(plond,plev)            ! height of midpoints (above surface) 
C
C------------------------------Externals--------------------------------
C
      external
     $     resetr,  ! reset real vector
     $     copyac,  ! copy data between vectors
     $     outfld,  ! store field in history tape buffer
     $     omcalc,  ! calc. vertical pressure velocity
     $     aphys,   ! apply adjustment physics
     $     plevs0,  ! define interface/midpoint pressures
     $     writup,  ! history tape write
     $     tphys,   ! drives model tendency physics (not adjustment physics)
     $     grmult,  ! nonlinear dynamics in grid space
     $     dyn,     ! assemble terms for quadrature
     $     zmid,    ! compute level heights (above surface)
     $     virtem,  ! virtual temperature calculation
     $     stats,   ! diagnostic statistics
     $     qneg3,
     $     fft991
C
C-----------------------------------------------------------------------
C
C Common block needed for carry of variables from physics1 to physics2-CMS
C
      common/physics_hold/ cnt,cnb,qc,pmid,pmidm1,rpmid,pmln,pint,
     $                     pintm1,
     $                     pdel,pdelm1,rpdel,piln,pstar,pstarmln,t2,
     $                     cmfdt,cmfdq,cmfdqr,cmfmc,cmfsl,cmflq,omga,
     $                     precl,precc,precsl,precsc

C-------------------------------------------------------------------------
C
#ifdef INTERVAL_TIMER
      call interval_timer( 1, TIMER_Physics, 0 )
#endif

C
C Begin executable code: zero history tape buffer for first time step
C

      if (nstep.eq.nrstrt) then
        call resetr(hbuf,lhbuf,0.)
      endif

      coslat = cos(clat(lat))
      rcoslat = 1./coslat
C
C Set slt advection diagnostics to zero if spectral run
C
      if (.not.doslt) then
         call resetr(dqfx3,plndlv*pcnst,0.)
         call resetr(vqfcst,plndlv*pcnst,0.)
         call resetr(ta,plndlv*pcnst,0.)
         call resetr(hqfcst(1,1,1,lat),plndlv*pcnst,0.)
      end if
C
C Set average dry mass to specified constant preserving horizontal
C gradients of ln(ps). Proportionality factor was calculated in STEPON
C for nstep=0 or SCAN2 otherwise from integrals calculated in INIDAT
C and SCAN2 respectively.
C Set p*.
C
      do i=1,plon
         ps(i) = (ps(i) + psr)*fixmas - psr
         pstar(i,lat) = ps(i) + psr
      end do
C
C Set current time pressure arrays for model levels etc.
C
      call plevs0(pstar(1,lat),pint(1,1,lat),pmid(1,1,lat),
     $            pdel(1,1,lat))
      do k=1,plev
         do i=1,plon
            pmln(i,k,lat) = alog(pmid(i,k,lat))
            rpmid(i,k,lat) = 1./pmid(i,k,lat)
            rpdel(i,k,lat) = 1./pdel(i,k,lat)
            piln(i,k,lat) = alog(pint(i,k,lat))
         end do
      end do
      do i=1,plon
         piln(i,plevp,lat) = alog(pint(i,plevp,lat))
      end do
C
      if (nstep.eq.0) then
C
C Copy past time pressure arrays for model levels for first time step
C
         call copyac(ps  ,psm1  ,plon  )
         call copyac(pint(1,1,lat),pintm1(1,1,lat),plndlv)
         call copyac(pmid(1,1,lat),pmidm1(1,1,lat),plndlv)
         call copyac(pdel(1,1,lat),pdelm1(1,1,lat),plndlv)
      else
         if (doslt) then
C
C Compute time tendency due to vertical advection of q(slt).
C Compute time tendency due to mass adjustment of q(slt).
C Update q(slt) with the mass adjustment
C Compute time tendency due to total advection of q(slt)
C (qminus contains q at (n-2), i.e. q at n-1 from the previous time
C step after the sources from tphys have been added)
C
            do m=1,pcnst
               do k=1,plev
                  do i=1,plon
                     vqfcst(i,k,m) = (q3(i,k,m) - qminus(i,k,m))/tdt -
     $                               hqfcst(i,k,m,lat)
                     dqfx3(i,k,m) = alpha(m)*q3(i,k,m)*
     $                   (sqrt(abs(q3(i,k,m) - qminus(i,k,m))))**3/tdt
                     q3(i,k,m) = q3(i,k,m) + tdt*dqfx3(i,k,m)
                     ta(i,k,m) = hqfcst(i,k,m,lat) +
     $                           vqfcst(i,k,m) + dqfx3(i,k,m)
                  end do
               end do
            end do
C
C Check for and correct invalid constituents
C
            call qneg3('LINEMS  ',lat,q3(1,1,1))
         end if
C
C Send slt tendencies to the history tape
C
         do m=1,pcnst
C            call outfld(hadvnam(m),hqfcst(1,1,m,lat),plond,lat,hbuf,lhbuf)
C            call outfld(vadvnam(m),vqfcst(1,1,m),plond,lat,hbuf,lhbuf)
C            call outfld(fixcnam(m),dqfx3(1,1,m) ,plond,lat,hbuf,lhbuf)
            call outfld(tottnam(m),ta(1,1,m) ,plond,lat,hbuf,lhbuf)
         end do
      end if
C
C Calculate vertical motion field
C
      call omcalc(rcoslat ,div     ,u3      ,v3      ,dpsl    ,
     $            dpsm,pmid(1,1,lat) ,pdel(1,1,lat),pmln(1,1,lat),
     $            piln(1,1,lat), rpmid(1,1,lat),
     $            rpdel(1,1,lat),pint(1,plevp,lat),omga(1,1,lat))
C
C Convert land/ocean/sea ice flag to integer for convection, physics
C
      do i=1,plon
         ioro(i) = nint(oro(i))
      end do
C
C Do adjustment physics (dry convective adjustment, convection,
C and stable precip)
C
#ifndef DISABLE_APHYS
      call aphys(nstep   ,doslt   ,lat     ,tdt     ,phis    ,
     $           pblht   ,tpert   ,pmid(1,1,lat) ,pint(1,1,lat),
     $           pdel(1,1,lat),
     $           pmln(1,1,lat) ,piln(1,1,lat),t3  ,q3   ,ts   ,
     $           qpert   ,ws      ,precl(1,lat)   ,precc(1,lat),
     $           precsl(1,lat),
     $           precsc(1,lat)  ,rnfp1   ,dtcond  ,dqcond  ,cnt(1,lat),
     $           cnb(1,lat)     ,qc(1,1,lat) ,cmfdt(1,1,lat),
     $           cmfdq(1,1,lat),   cmfdqr(1,1,lat)  ,
     $           cmfmc(1,1,lat)   ,cmfsl(1,1,lat)   ,cmflq(1,1,lat)   )
#else
       write(0,*)'WARNING DISABLE_APHYS'
#endif
C
C Copy adjusted time n values to time n-1 first time step only
C
      if (nstep.eq.0) then
         ilen = plndlv - nxpt
         call copyac(t3,t3m1,ilen)
         do m=1,pcnst
            call copyac(q3(1,1,m),q3m1(1,1,m),ilen)
         end do
      end if
C
C Time filter (second half of filter for vorticity and divergence only)
C
      if (nstep.ge.3) then
         om2eps = 1. - 2.*eps
         do k=1,plev
            do i=1,plon
               u3m1(i,k) = om2eps*u3m1(i,k) + eps*um2(i,k) + eps*u3(i,k)
               v3m1(i,k) = om2eps*v3m1(i,k) + eps*vm2(i,k) + eps*v3(i,k)
               t3m1(i,k) = om2eps*t3m1(i,k) + eps*tm2(i,k) + eps*t3(i,k)
               q3m1(i,k,1) = om2eps*q3m1(i,k,1) + eps*qm2(i,k,1) +
     $                       eps*q3(i,k,1)
               vortm1(i,k) = vorttl(i,k) + eps*vort(i,k)
               divm1(i,k) = divtl(i,k) + eps*div(i,k)
            end do
            do m=2,pcnst
               do i=1,plon
                  q3m1(i,k,m) = om2eps*q3m1(i,k,m) + eps*qm2(i,k,m) +
     $                          eps*q3(i,k,m)
               end do
            end do
         end do
         do i=1,plon
            psm1(i) = om2eps*psm1(i) + eps*psm2(i) + eps*ps(i)
         end do
      else
C
C Copy vorticity, divergence when not time filtered
C
         call copyac(vorttl,vortm1,plndlv)
         call copyac(divtl,divm1,plndlv)
      end if
C
C Set past time pressure arrays for model levels etc.
C
      do i=1,plon
         pstarm(i) = psm1(i) + psr
         pstarmln(i,lat) = alog(pstarm(i))
      end do
      call plevs0(pstarm,pintm1(1,1,lat),pmidm1(1,1,lat),
     $            pdelm1(1,1,lat))
      call copyac(pstarm,psslt,plon)
#ifdef INTERVAL_TIMER
      call interval_timer( 0, TIMER_Physics, 0 )
#endif
C
C Compute time tendencies:comment out since currently not on h-t
C
C      do k=1,plev
C         do i=1,plon
C            ttend(i,k) = (t3(i,k)-tm2(i,k))/tdt
C            utend(i,k) = (u3(i,k)-um2(i,k))/tdt
C            vtend(i,k) = (v3(i,k)-vm2(i,k))/tdt
C         end do  
C      end do  
C      do m=1,pcnst
C         do k=1,plev
C            do i=1,plon
C               qtend(i,k,m) = (q3(i,k,m) - qm2(i,k,m))/tdt
C            end do
C         end do
C      end do
C      do i=1,plon
C         pstend(i) = (ps(i) - psm2(i))/tdt
C      end do
C
C Send tendencies to history tape
C
      call outfld('DTCOND  ',dtcond       ,plond,lat,hbuf,lhbuf)
      do m=1,pcnst
         call outfld(dcconnam(m),dqcond(1,1,m),plond,lat,hbuf,lhbuf)
      end do
      call outfld('DTH     ',dth,plond,lat,hbuf,lhbuf)
C      call outfld('DUH     ',duh,plond,lat,hbuf,lhbuf)
C      call outfld('DVH     ',dvh,plond,lat,hbuf,lhbuf)
C      call outfld('NDQH    ',b(ndqh+1),plond,lat,hbuf,lhbuf)
C
C Send prognostic variables to history tape
C
      call outfld('PS      ',pstarm,plond,lat,hbuf,lhbuf)
      call outfld('T       ',t3m1  ,plond,lat,hbuf,lhbuf)
      call outfld('U       ',u3m1  ,plond,lat,hbuf,lhbuf)
      call outfld('V       ',v3m1  ,plond,lat,hbuf,lhbuf)
C      call outfld('ETADOT  ',etadot(1,2),plon,lat,hbuf,lhbuf)
      call outfld('Q       ',q3m1(1,1,1),plond,lat,hbuf,lhbuf)
      do m=2,pcnst
         call outfld(tracnam(m),q3m1(1,1,m),plond,lat,hbuf,lhbuf)
      end do
C
C Calculate sea level pressure
C
      if (ninavg(1).eq.'Q') then
         call cpslhy(phis    ,pstarm    ,t3m1  ,psl   )
         call outfld('PSL     ',psl    ,plond   ,lat     ,hbuf ,lhbuf)
C     
C Outfld mass of q for monthly averages
C
         do k=1,plev
            do i=1,plon
               ftem(i,k) = q3m1(i,k,1) * pdel(i,k,lat) * rga
            end do
         end do
         call outfld('MQ      ',ftem,plond,lat,hbuf,lhbuf)
C
C Calculate relative humidity for mnthly avg
C
         call aqsat(t3m1    ,pmidm1(1,1,lat),est ,qsat ,plond   ,
     $              plon    ,plev    ,1       ,plev    )
         do k=1,plev
            do i=1,plon
               ftem(i,k) = q3m1(i,k,1)/qsat(i,k)*100.
            end do
         end do
         call outfld('RELHUM  ',ftem,plond,lat,hbuf,lhbuf)
C
C Compute major meridional energy transports vT vq vv+uu 
C
         do k=1,plev
            do i=1,plon
               ftem(i,k) = v3m1(i,k)*t3m1(i,k)
            end do
         end do
C
         call outfld('VT      ',ftem,plond,lat,hbuf,lhbuf)
         do k=1,plev
            do i=1,plon
               ftem(i,k) = v3m1(i,k)*q3m1(i,k,1)
            end do
         end do
         call outfld('VQ      ',ftem,plond,lat,hbuf,lhbuf)
C        
         do k=1,plev
            do i=1,plon
               ftem(i,k) = v3m1(i,k)**2 + u3m1(i,k)**2
            end do
         end do
         call outfld('VVPUU   ',ftem  ,plond,lat,hbuf,lhbuf)
      end if    ! ninavg(1).eq.'Q'
C
C      do m=1,pcnst
C         call outfld(tendnam(m),qtend(1,1,m),plond,lat,hbuf,lhbuf)
C      end do
C      call outfld('UTEND   ',utend,plond,lat,hbuf,lhbuf)
C      call outfld('VTEND   ',vtend,plond,lat,hbuf,lhbuf)
C      call outfld('TTEND   ',ttend,plond,lat,hbuf,lhbuf)
C      call outfld('LPSTEN  ',pstend,plond,lat,hbuf,lhbuf)
C
C Clear hbuf and nacs of the above outfld calls on the first timestep of an
C initial or restart run for those parts of the history buffer which have no
C corresponding history restart file.
C
C Note for parallel code.  History regeneration files are not supported.
C A restart can only occur in between averaging periods.  Therefore, we
C reset the entire history buffer for a latitude on the first time step,
C without regard for what is happening in that part of the buffer.
C
      if (nstep.eq.nrstrt) then
        call resetr(hbuf,lhbuf,0.)
        kfld = 1
        do itape=1,mtapes
          if ( rgnht(itape) ) then
            print*,   'WARNING -- hist regen files not supported'
            print*,   '    The accumulated averages in tape ',itape
            print*,   '    are likely to be wrong in this restart run.'
            print*,   '    Try restarting in between averaging periods.'
            write(0,*)'WARNING -- hist regen files not supported'
            write(0,*)'    The accumulated averages in tape ',itape
            write(0,*)'    are likely to be wrong in this restart run.'
            write(0,*)'    Try restarting in between averaging periods.'
          endif
          do i=kfld,kfld+nflds(itape)-1
            nacs(i,lat) = 0
          end do
          kfld = kfld + nflds(itape)
        end do
      end if
C
C Write history tape unless the hbuf is incore.  For the
C incore case, the write occurs in scan1 after the physics call.
C
C
C Bug change was made here-CMS
#  ifdef INTERVAL_TIMER
      call interval_timer( 1, TIMER_History_out, 0 )
#  endif
      call writup(nstep,lat,hbuf,lhbuf)
#  ifdef INTERVAL_TIMER
      call interval_timer( 0, TIMER_History_out, 0 )
#  endif

      if (.not.nlend) then
         if (nstep.ge.2) then
C
C Time filter (first half for vorticity and divergence only)
C
            do k=1,plev
               do i=1,plon
                 vorttl(i,k) = (1. - 2.*eps)*vort(i,k) + eps*vortm1(i,k)
                 divtl(i,k) = (1. - 2.*eps)*div(i,k) + eps*divm1(i,k)
               end do
            end do
         else
C
C Copy time filter fields for first steps when not filtered
C
            call copyac(vort,vorttl,plndlv)
            call copyac(div,divtl,plndlv)
         end if
#ifdef INTERVAL_TIMER
         call interval_timer( 1, TIMER_Physics, 0 )
#endif
C
C Copy n-1 fields to n-2 time level
C
         do k=1,plev
            do i=1,plon
               um2(i,k) = u3m1(i,k)
               vm2(i,k) = v3m1(i,k)
               tm2(i,k) = t3m1(i,k)
               qm2(i,k,1) = q3m1(i,k,1)
            end do
            do m=2,pcnst
               do i=1,plon
                  qm2(i,k,m) = q3m1(i,k,m)
               end do
            end do
         end do
         do i=1,plon
            psm2(i) = psm1(i)
         end do
C
C Do tendency physics: clouds, radiation, surface temp and fluxes,
C pbl and vertical diffusion,rayleigh friction and gravity wave drag
C
         call resetr(t2(1,1,lat),plndlv,0.)
         call resetr(fu,plndlv,0.)
         call resetr(fv,plndlv,0.)
#ifndef DISABLE_TPHYS
         call tphys1(lat     ,clat(lat),coslat ,hbuf    ,lhbuf   ,
     $                                                  ztodt   ,
     $              oro     ,ioro  ,phis,pmidm1(1,1,lat),
     $              pintm1(1,1,lat),
     $              pdelm1(1,1,lat) ,u3  ,v3   ,q3  ,omga(1,1,lat),
     $              t2(1,1,lat)      ,fu      ,fv      ,u3m1 ,v3m1 ,
     $              t3m1    ,q3m1    ,psm1    ,pblht   ,qpert   ,
     $              tpert   ,ts      ,sn      ,cnt(1,lat) ,cnb(1,lat),
     $              qrs     ,qrl     ,flwds   ,fsns    ,plol    ,
     $              plos    ,tssub   ,ws      ,precc(1,lat)   ,
     $		    cmfmc(1,1,lat))
#else
       write(0,*)'WARNING DISABLE_TPHYS'
#endif

#ifdef BLOCK_FFT
#  ifdef INTERVAL_TIMER
      call interval_timer( 1, TIMER_FwdFFT, 0 )
#  endif
         call copyac( xnlin, fft_buffer( 1, ip ), plndlv*5 )
         ip = ip + 5*plev
         call copyac( t3m1,  fft_buffer( 1, ip ), plndlv )
         ip = ip + plev
         call copyac( psm1,  fft_buffer( 1, ip ), plond )
         ip = ip + 1
         call copyac( vortm1,  fft_buffer( 1, ip ), plndlv )
         ip = ip + plev
         call copyac( divm1,   fft_buffer( 1, ip ), plndlv )
         ip = ip + plev
#  ifdef INTERVAL_TIMER
      call interval_timer( 0, TIMER_FwdFFT, 0 )
#  endif
#else
         call copyac(xnlin,xnlin_sv(1,lat),plndlv*7)
#endif

#ifdef COUPLED
      do i=1,plon
        to_coupler(i,lat,11)=precc(i,lat) + precl(i,lat)
        to_coupler(i,lat,12)=precc(i,lat)
      enddo
#endif
C endif needed because of split-CMS
      endif


      return
      end





      subroutine physics2(irow   ,lat     ,tdt     ,ps      ,psm1    ,
     $                  u3      ,u3m1    ,v3      ,v3m1    ,t3      ,
     $                  t3m1    ,q3      ,q3m1    ,etadot  ,psslt   ,
     $                  etamid  ,ztodt   ,vmax    ,vmaxt   ,vcour   ,
     $                  detam   ,grlps1  ,grt1    ,grq1    ,grz1    ,
     $                  grd1    ,grfu1   ,grfv1   ,grut1   ,gruq1   ,
     $                  grvt1   ,grvq1   ,grrh1   ,grlps2  ,grt2    ,
     $                  grq2    ,grz2    ,grd2    ,grfu2   ,grfv2   ,
     $                  grut2   ,gruq2   ,grvt2   ,grvq2   ,grrh2   ,
     $                  hbuf    ,lhbuf   ,psm2    ,bpstr   ,um2     ,
     $                  vm2     ,tm2     ,qm2     ,qminus  ,vort    ,
     $                  vortm1  ,vorttl  ,div     ,divm1   ,divtl   ,
     $                  dpsl    ,dpsm    ,ts      ,ws      ,sn      ,
     $                  oro     ,rnfp1   ,phis    ,pblht   ,qpert   ,
     $                  tpert   ,duh     ,dvh     ,dth     ,qrs     ,
     $                  qrl     ,flwds   ,fsns    ,plos    ,plol    ,
#ifndef BLOCK_FFT
     $                  tssub   )
#else
     $                  tssub   ,fft_buffer, ip )
#endif
C-----------------------------------------------------------------------
C
C Control adjustment physics, time filter, history tape write-up,
C tendency physics, non-linear dynamical terms, FFT and combine terms
C in preparation for Fourier -> spectral quadrature.
C
C The naming convention is as follows:
C  - prefix gr contains grid point values before FFT and Fourier
C     coefficients after
C  - t, q, d, z and ps refer to temperature, specific humidity,
C     divergence, vorticity and surface pressure
C  - "1" suffix to an array => symmetric component current latitude pair
C  - "2" suffix to an array => antisymmetric component.
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          D. Williamson, J. Hack, August 1992
C
C physics split for coupling: C. Schafer, R. Jacob, April 1995
C additions to work with CCM3:  R. Jacob, December 1996
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
C------------------------------Commons----------------------------------
#include "comcon.com"
C-----------------------------------------------------------------------
#include "comctl.com"
C-----------------------------------------------------------------------
#include "comfft.com"
C-----------------------------------------------------------------------
#include "comhed.com"
C-----------------------------------------------------------------------
#include "comhst.com"
C-----------------------------------------------------------------------
#include "comhyb.com"
C-----------------------------------------------------------------------
#include "comlun.com"
C-----------------------------------------------------------------------
#include "commap.com"
C-----------------------------------------------------------------------
#include "commss.com"
C-----------------------------------------------------------------------
#include "comqfl.com"
C-----------------------------------------------------------------------
#include "comslt.com"
C-----------------------------------------------------------------------
#include "comspe.com"
C-----------------------------------------------------------------------
#include "comtfc.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#include "comtrcnm.com"
#ifdef INTERVAL_TIMER
#include "timers.com"
#endif
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer
     $     irow,                     ! latitude pair index
     $     lat,                      ! latitude index for S->N storage
     $     lhbuf                     ! length history tape buffer, hbuf
      real tdt,                      ! two delta t (unless nstep<2)
     $     ps(plond),                ! surface pressure (time n)
     $     psm1(plond),              ! surface pressure (time n-1)
     $     u3(plond,plev),           ! u-wind (time n)
     $     u3m1(plond,plev),         ! u-wind (time n-1)
     $     v3(plond,plev),           ! v-wind (time n)
     $     v3m1(plond,plev),         ! v-wind (time n-1)
     $     t3(plond,plev),           ! temperature (time n)
     $     t3m1(plond,plev),         ! temperature (time n-1)
     $     q3(plond,plev,pcnst),     ! constituent conc(time n  :h2o first)
     $     q3m1(plond,plev,pcnst),   ! constituent conc(time n-1:h2o first)
     $     etadot(plon,plevp),       ! vertical motion (3-d used by slt)
     $     psslt(plond),             ! surface pressure (2-d used by slt)
     $     etamid(plev),             ! midpoint values of eta (a+b)
     $     ztodt,                    ! 2*timestep unless nstep = 0
     $     vmax(plev),               ! maximum wind speed squared (m^2/s^2)
     $     vmaxt(plev),              ! maximum truncated wind speed (m^2/s^2)
     $     vcour(plev),              ! maximum Courant number in vert.
     $     detam(plev)               ! intervals between vert.levels
#ifdef PP_PCCM
c parallel fft needs entire width
      real grlps1(plon),          ! sym. undiff. term in lnps eqn.
     $     grlps2(plon),          ! antisym undiff. term in lnps eqn.
     $     grt1(plon,plev),       ! sym. undiff. term in t eqn.
     $     grt2(plon,plev),       ! antisym. undiff. term in t eqn.
     $     grq1(plon,plev),       ! sym. undiff. term in q eqn.
     $     grq2(plon,plev),       ! antisym. undiff. term in q eqn.
     $     grz1(plon,plev),       ! sym. undiff. term in z eqn.
     $     grz2(plon,plev),       ! antisym. undiff. term in z eqn.
     $     grd1(plon,plev),       ! sym. undiff. term in d eqn.
     $     grd2(plon,plev),       ! antisym. undiff. term in d eqn.
     $     grfu1(plon,plev),      ! sym. nonlinear terms in u eqn.
     $     grfu2(plon,plev),      ! antisym. nonlinear terms in u eqn.
     $     grfv1(plon,plev),      ! sym. nonlinear terms in v eqn.
     $     grfv2(plon,plev),      ! antisym. nonlinear terms in v eqn.
     $     grut1(plon,plev),      ! sym. lambda deriv. term in t eqn.
     $     grut2(plon,plev),      ! antisym. lambda deriv. term in t eqn.
     $     gruq1(plon,plev),      ! sym. lambda deriv. term in q eqn.
     $     gruq2(plon,plev),      ! antisym. lambda deriv. term in q eqn.
     $     grvt1(plon,plev),      ! sym. mu derivative term in t eqn.
     $     grvt2(plon,plev),      ! antisym. mu deriv. term in t eqn.
     $     grvq1(plon,plev),      ! sym. mu derivative term in q eqn.
     $     grvq2(plon,plev),      ! antisym. mu deriv. term in q eqn.
     $     grrh1(plon,plev),      ! sym. del**2 term in d eqn.
     $     grrh2(plon,plev)       ! antisym. del**2 term in d eqn.
#else
      real grlps1(2*pmmax),          ! sym. undiff. term in lnps eqn.
     $     grlps2(2*pmmax),          ! antisym undiff. term in lnps eqn.
     $     grt1(2*pmmax,plev),       ! sym. undiff. term in t eqn.
     $     grt2(2*pmmax,plev),       ! antisym. undiff. term in t eqn.
     $     grq1(2*pmmax,plev),       ! sym. undiff. term in q eqn.
     $     grq2(2*pmmax,plev),       ! antisym. undiff. term in q eqn.
     $     grz1(2*pmmax,plev),       ! sym. undiff. term in z eqn.
     $     grz2(2*pmmax,plev),       ! antisym. undiff. term in z eqn.
     $     grd1(2*pmmax,plev),       ! sym. undiff. term in d eqn.
     $     grd2(2*pmmax,plev),       ! antisym. undiff. term in d eqn.
     $     grfu1(2*pmmax,plev),      ! sym. nonlinear terms in u eqn.
     $     grfu2(2*pmmax,plev),      ! antisym. nonlinear terms in u eqn.
     $     grfv1(2*pmmax,plev),      ! sym. nonlinear terms in v eqn.
     $     grfv2(2*pmmax,plev),      ! antisym. nonlinear terms in v eqn.
     $     grut1(2*pmmax,plev),      ! sym. lambda deriv. term in t eqn.
     $     grut2(2*pmmax,plev),      ! antisym. lambda deriv. term in t eqn.
     $     gruq1(2*pmmax,plev),      ! sym. lambda deriv. term in q eqn.
     $     gruq2(2*pmmax,plev),      ! antisym. lambda deriv. term in q eqn.
     $     grvt1(2*pmmax,plev),      ! sym. mu derivative term in t eqn.
     $     grvt2(2*pmmax,plev),      ! antisym. mu deriv. term in t eqn.
     $     grvq1(2*pmmax,plev),      ! sym. mu derivative term in q eqn.
     $     grvq2(2*pmmax,plev),      ! antisym. mu deriv. term in q eqn.
     $     grrh1(2*pmmax,plev),      ! sym. del**2 term in d eqn.
     $     grrh2(2*pmmax,plev)       ! antisym. del**2 term in d eqn.
#endif
      real hbuf(lhbuf)               ! history tape buffer
      real psm2(plond),              ! surface pressure (time n-2)
     $     bpstr(plond),             ! storage next to vortm1 for ps transfm
     $     um2(plond,plev),          ! u-wind (time n-2)
     $     vm2(plond,plev),          ! v-wind (time n-2)
     $     tm2(plond,plev),          ! temperature (time n-2)
     $     qm2(plond,plev,pcnst),    ! constituent-h2o first (time n-2)
     $     qminus(plond,plev,pcnst), ! saves q being advected in slt code
     $     vort(plond,plev),         ! vorticity (time n)
     $     vortm1(plond,plev),       ! vorticity (time n-1)
     $     vorttl(plond,plev),       ! vorticity: half time-filtered
     $     div(plond,plev),          ! divergence (time n)
     $     divm1(plond,plev),        ! divergence (time n-1)
     $     divtl(plond,plev)         ! divergence: half time-filtered
      real dpsl(plond),              ! longitude derivative of lnps (time n)
     $     dpsm(plond),              ! latitude  derivative of lnps (time n)
     $     ts(plond),                ! surface temperature
     $     ws(plond),                ! surface wetness
     $     sn(plond),                ! equivalent liquid water snow depth
     $     oro(plond),               ! land/ocean/sea ice flag
     $     rnfp1(plond),             ! runoff
     $     phis(plond),              ! surface geopotential
     $     pblht(plond),             ! PBL height
     $     qpert(plond,pcnst)        ! moisture/constituent perturb. (PBL)
      real tpert(plond),             ! temperature perturbation (PBL)
     $     duh(plond,plev),          ! horizontal diffusion of u-momentum
     $     dvh(plond,plev),          ! horizontal diffusion of v-momentum
     $     dth(plond,plev),          ! horizontal temperature diffusion
     $     qrs(plond,plev),          ! shortwave radiative heating rate 
     $     qrl(plond,plev),          ! longwave  radiative heating rate 
     $     flwds(plond),             ! surface longwave down flux
     $     fsns(plond),              ! surface absorbed solar flux
     $     plos(plond,plevp),        ! O3 path length (cm)
     $     plol(plond,plevp),        ! O3 pressure-weighted pathlength(cm)
     $     tssub(plond,plevmx)       ! sub-surface soil temperatures
#ifdef BLOCK_FFT
      real fft_buffer(plond,*)
      integer ip
#endif
C
C---------------------------Local workspace-----------------------------
C
      real cnt(plond,plat),               ! top of convective layer
     $     cnb(plond,plat),               ! bottom of convective layer
     $     qc(plond,plev,plat),           ! q tendency due to total rainout
     $     vqfcst(plond,plev,pcnst), ! q tendency due to vert. adv. (slt)
     $     pmid(plond,plev,plat),         ! pressure at model levels (time n)
     $     pmidm1(plond,plev,plat),       ! pressure at model levels (time n-1)
     $     rpmid(plond,plev,plat),        ! 1./pmid
     $     pmln(plond,plev,plat),         ! alog(pmid)
     $     pint(plond,plevp,plat),        ! pressure at model interfaces (n  )
     $     pintm1(plond,plevp,plat),      ! pressure at model interfaces (n-1)
     $     pdel(plond,plev,plat),         ! pdel(k)   = pint  (k+1)-pint  (k)
     $     pdelm1(plond,plev,plat),       ! pdelm1(k) = pintm1(k+1)-pintm1(k)
     $     rpdel(plond,plev,plat),        ! 1./pdel
     $     piln(plond,plevp,plat)         ! alog(pint)
      real est(plond,plevp),         ! sat vap pressure
     $     qsat(plond,plevp),        ! saturation specific humidity
     $     psl(plond)                ! sea level pressure
      real pstar(plond,plat),             ! ps + psr (time n)
     $     pstarm(plond),            ! ps + psr (time n-1)
     $     pstarmln(plond,plat),          ! alog(pstarm)
     $     t2(plond,plev,plat),           ! nonlinear term - temperature
     $     engy(plond,plev)          ! kinetic energy
      real xnlin(plndlv*7)           ! non-linear terms (equivalence 
C                                    !  region for following arrays to 
C                                    !  optimize fft performance)
#ifndef BLOCK_FFT
      real xnlin_sv(plndlv*7,plat)  ! save these quantities for all lat (JM)
      common /xxxx/ xnlin_sv
#endif
      real fu(plond,plev),           ! nonlinear term - u momentum eqn.
     $     fv(plond,plev),           ! nonlinear term - v momentum eqn.
     $     ut(plond,plev),           ! (u*T) - heat flux - zonal
     $     vt(plond,plev),           ! (v*T) - heat flux - meridional
     $     drhs(plond,plev),         ! RHS of divergence eqn. (del^2 term)
     $     uq(plond,plev),           ! (u*q) - water flux - zonal
     $     vq(plond,plev)            ! (v*q) - water flux - meridional
#if ( defined SUN )
C
C Sun compiler does not allow "equivalence" for stack variables; make
C them static.
C
      static xnlin,fu,fv,ut,vt,drhs,uq,vq
#endif
C
C The following equivalences are for optimal fft performance
C
      equivalence (fu,xnlin(1)),
     $            (fv,xnlin(1+1*plndlv)),
     $            (ut,xnlin(1+2*plndlv)),
     $            (vt,xnlin(1+3*plndlv)),
     $            (drhs,xnlin(1+4*plndlv)),
     $            (uq,xnlin(1+5*plndlv)),
     $            (vq,xnlin(1+6*plndlv))
C
      real dqfx3(plond,plev,pcnst),  ! q tendency due to mass adjustment
     $     dtcond(plond,plev),       ! dT/dt due to convective adjustment
     $     dqcond(plond,plev,pcnst), ! dq/dt due to convective adjustment
     $     cmfdt(plond,plev,plat),        ! convective temperature tendency
     $     cmfdq(plond,plev,plat),        ! convective moisture tendency
     $     cmfdqr(plond,plev,plat),       ! dq/dt due to convective rainout
     $     cmfmc(plond,plev,plat),        ! convective mass flux--m sub c
     $     cmfsl(plond,plev,plat),        ! liquid water static energy flux
     $     cmflq(plond,plev,plat)         ! total water flux
      real ftem(plond,plev)          ! temporary workspace
      real omga(plond,plev,plat),         ! vertical velocity
     $     precl(plond,plat),             ! large scale stable precip rate
     $     precc(plond,plat),             ! convective precip rate
     $     precsl(plond,plat),            ! snow component large scale precip
     $     precsc(plond,plat)             ! snow component convective precip
C      real utend(plond,plev),        ! du/dt
C     $     vtend(plond,plev),        ! dv/dt
C     $     ttend(plond,plev),        ! dT/dt
C     $     qtend(plond,plev,pcnst),  ! dq/dt
C     $     pstend(plond),            ! d(ps)/dt
      real wind,                     ! u**2 + v**2 (m/s)
     $     lvcour,                   ! local vertical courant number
     $     om2eps,                   ! 1. - 2.*eps for time filter
     $     dtdz                      ! dt/detam(k)
      integer
     $     ioro(plond),              ! nint(oro(i))
     $     i,k,kk,m,                 ! longitude,level,constituent indices
     $     ilen,                     ! length to copy to avoid overrun
     $     inc,                      ! increment for fft991
     $     isign,                    ! flag indicates transform direction
     $     ntr,                      ! number of transforms to perform
     $     itape,
     $     kfld
      real ta(plond,plev,pcnst),     ! total advection of constituents
     $     ddivdt(plond,plev),       ! temporary workspace
     $     ddpn(plond),              ! complete sum of d*delta p
     $     vpdsn(plond),             ! complete sum V dot grad(ln(ps)) delta b
     $     dqdt(plond,plev)          ! moisture tendency (spectral transport)
      real dpslat(plond,plev),       ! Pressure gradient term 
     $     dpslon(plond,plev)        ! Pressure gradient term 
      real coslat,                   ! cosine(latitude)
     $     rcoslat,                  ! 1./cosine(latitude)
     $     rhypi,                    ! 1./hypi(plevp)
     $     utfac,                    ! Only Byron knows
     $     vtfac                     ! Only Byron knows
      real numomgup(plond,plev),     ! number of times wind directed +z
     $     omegaup(plond,plev),      ! omega during omega < 0.
     $     tvm(plond,plev),          ! virtual temperature 
     $     zm(plond,plev)            ! height of midpoints (above surface) 
C
C------------------------------Externals--------------------------------
C
      external
     $     resetr,  ! reset real vector
     $     copyac,  ! copy data between vectors
     $     outfld,  ! store field in history tape buffer
     $     omcalc,  ! calc. vertical pressure velocity
     $     aphys,   ! apply adjustment physics
     $     plevs0,  ! define interface/midpoint pressures
     $     writup,  ! history tape write
     $     tphys,   ! drives model tendency physics (not adjustment physics)
     $     grmult,  ! nonlinear dynamics in grid space
     $     dyn,     ! assemble terms for quadrature
     $     zmid,    ! compute level heights (above surface)
     $     virtem,  ! virtual temperature calculation
     $     stats,   ! diagnostic statistics
     $     qneg3,
     $     fft991
C
C-----------------------------------------------------------------------
C
C Common block needed for carry of variables from physics1 to physcics2-CMS
C
      common/physics_hold/ cnt,cnb,qc,pmid,pmidm1,rpmid,pmln,pint,
     $                     pintm1,
     $                     pdel,pdelm1,rpdel,piln,pstar,pstarmln,t2,
     $                     cmfdt,cmfdq,cmfdqr,cmfmc,cmfsl,cmflq,omga,
     $                     precl,precc,precsl,precsc  
C-------------------------------------------------------------------------

      if (.not.nlend) then
C Inserted by Chad, needed for split of physics-CMS
C Add BLOCK_FFT ifdefs to match other uses of this block.
#ifdef BLOCK_FFT
#  ifdef INTERVAL_TIMER
      call interval_timer( 1, TIMER_FwdFFT, 0 )
#  endif
      call copyac( fft_buffer( 1, ip ), xnlin, plndlv*5 )
      ip = ip + 5*plev
      call copyac(  fft_buffer( 1, ip ), t3m1, plndlv )
      ip = ip + plev
      call copyac(  fft_buffer( 1, ip ), psm1, plond )
      ip = ip + 1
      call copyac(  fft_buffer( 1, ip ), vortm1, plndlv )
      ip = ip + plev
      call copyac(   fft_buffer( 1, ip ), divm1, plndlv )
      ip = ip + plev
      ip = ip - (5*plev + plev + 1 + plev + plev )
#  ifdef INTERVAL_TIMER
      call interval_timer( 0, TIMER_FwdFFT, 0 )
#  endif
#else
         call copyac(xnlin_sv(1,lat),xnlin,plndlv*7)
#endif


      coslat = cos(clat(lat))
      rcoslat = 1./coslat

C
C Convert land/ocean/sea ice flag to integer for convection, physics
C
      do i=1,plon
         ioro(i) = nint(oro(i))
      end do

C End of piece inserted by Chad

#ifndef DISABLE_TPHYS
         call tphys2(lat     ,clat(lat),coslat ,hbuf    ,lhbuf   ,
     $                                                  ztodt   ,
     $              oro,ioro ,phis ,pmidm1(1,1,lat) ,pintm1(1,1,lat) ,
     $              pdelm1(1,1,lat)  ,u3  ,v3  ,q3  ,omga(1,1,lat),
     $              t2(1,1,lat)      ,fu   ,fv      ,u3m1 ,v3m1 ,
     $              t3m1    ,q3m1    ,psm1    ,pblht   ,qpert   ,
     $              tpert   ,ts      ,sn  ,cnt(1,lat) ,cnb(1,lat) ,
     $              qrs     ,qrl     ,flwds ,fsns    ,plol    ,
     $              plos    ,tssub   ,ws    ,precc(1,lat) )
#else
       write(0,*)'WARNING DISABLE_TPHYS'
#endif
C
C Compute maximum wind speed this latitude (used in Courant number estimate)
C
         if (ptrm .lt. ptrn) then
            utfac = float(ptrm)/float(ptrn)
            vtfac = 1.
         else if (ptrn .lt. ptrm) then
            utfac = 1.
            vtfac = float(ptrn)/float(ptrm) 
         else if (ptrn .eq. ptrm) then
            utfac = 1.
            vtfac = 1.
         end if
         do k=1,plev
            vmax(k) = 0.
            vmaxt(k) = 0.
            do i=1,plon
               wind = u3m1(i,k)**2 + v3m1(i,k)**2
               vmax(k) = amax1(wind,vmax(k))
               wind = (u3m1(i,k)*utfac)**2 + (v3m1(i,k)*vtfac)**2
               vmaxt(k) = amax1(wind,vmaxt(k))
            end do
         end do
C
C Convert t to t-prime (n-1).
C
c        do k=1,plev
c           do i=1,plon
c              t3m1(i,k) = t3m1(i,k) - t0(k)
c           end do
c        end do
C
C Calculate non-linear terms in tendencies
C
         call grmult(doslt   ,rcoslat ,div     ,q3      ,t3      ,
     $               u3      ,v3      ,vort    ,t3m1    ,phis    ,
     $               dpsl    ,dpsm ,omga(1,1,lat) ,pdel(1,1,lat),
     $               pint(1,plevp,lat),
     $               pstar(1,lat),   pstarmln(1,lat),  pmln(1,1,lat),
     $               piln(1,1,lat),   rpmid(1,1,lat),
     $               rpdel(1,1,lat)   ,fu     ,fv  ,t2(1,1,lat) ,ut  ,
     $               vt      ,uq      ,vq      ,drhs    ,pmid(1,1,lat),
     $               etadot  ,etamid  ,engy    ,ddpn    ,vpdsn   ,
     $               dqdt    ,dpslon  ,dpslat  )
#ifdef INTERVAL_TIMER
      call interval_timer( 0, TIMER_Physics, 0 )
#endif
C
C Add tendencies to previous timestep values of surface pressure,
C temperature, and (if spectral transport) moisture.  Store *log* surface
C pressure in bpstr array for transform to spectral space.
C
         do i=1,plon
            psm1(i) = pstarmln(i,lat) -
     $               ztodt*(vpdsn(i)+ddpn(i))/pstar(i,lat)
         end do
         rhypi = 1./hypi(plevp)
         do k=1,plev
            do i=1,plon
               ddivdt(i,k) = ztodt*(0.5*divm1(i,k) - div(i,k))
               psm1(i) = psm1(i) - ddivdt(i,k)*hypd(k)*rhypi
               t3m1(i,k) = t3m1(i,k) + ztodt*t2(i,k,lat)
            end do
         end do
         do k=1,plev
            do kk=1,plev
               do i=1,plon
                  t3m1(i,k) = t3m1(i,k) - ddivdt(i,kk)*tau(kk,k)
               end do
            end do
         end do
         if (.not.doslt) then
            do k=1,plev
               do i=1,plon
                  q3m1(i,k,1) = q3m1(i,k,1) + ztodt*dqdt(i,k)
               end do
            end do
         end if
C
C Outfld Pressure Gradient terms
C
         if (ninavg(1).eq.'Q') then
            call outfld('DPSLAT  ',dpslat ,plond,lat,hbuf,lhbuf)
            call outfld('DPSLON  ',dpslon ,plond,lat,hbuf,lhbuf)
C
C Outfld geopotential height(meters above sea level) and vz for monthly
C averages      
C
            call virtem(t3      ,q3      ,zvir    ,tvm     )
            call zmid(piln(1,1,lat),pmln(1,1,lat),rair,gravit,tvm ,
     $                zm   )
            do k=1,plev
               do i=1,plon
                  ftem(i,k) = zm(i,k) + phis(i)*rga
               end do
            end do
            call outfld('Z2      ',ftem,plond,lat,hbuf,lhbuf)
C           
            do k=1,plev
               do i=1,plon
                  ftem(i,k) = (phis(i)+zm(i,k)*gravit)*v3(i,k)
               end do
            end do
            call outfld('VZ      ',ftem  ,plond,lat,hbuf,lhbuf)
         end if
C
C Compute maximum vertical Courant number this latitude.  The variable
C "dtime" (in common block "comtim") is used rather than "0.5*ztodt"
C since "dtime" remains constant from the very start of the model run.
C
         do k=2,plev
            dtdz = dtime/detam(k-1)
            vcour(k) = 0.
            do i=1,plon
               lvcour = abs(etadot(i,k))*dtdz
               vcour(k) = amax1(lvcour,vcour(k))
            end do
         end do
C
C Copy updated "q3m1" field into "qminus" array space for use next
C time step at beginning of LINEMS to diagnose slt advection. This
C is the q field that is advected by the slt proceedure.
C
         call copyac(q3m1,qminus,plndlv*pcnst)
C
C Apply cos(lat) to momentum terms before fft
C
         do k=1,plev
            do i=1,plon
               fu(i,k) = coslat*fu(i,k)
               fv(i,k) = coslat*fv(i,k)
               ut(i,k) = coslat*ut(i,k)
               vt(i,k) = coslat*vt(i,k)
            end do
         end do
         if (.not.doslt) then
            do k=1,plev
               do i=1,plon
                  uq(i,k) = coslat*uq(i,k)
                  vq(i,k) = coslat*vq(i,k)
               end do
            end do
         end if
C
C FFT of undifferentiated terms
C !!!! NOTE CONTIGUITY ASSUMPTIONS !!!!
C First: copy ps to location contiguous to vorticity for transform
C
         call copyac(psm1,bpstr,plon)
#ifdef BLOCK_FFT
#  ifdef INTERVAL_TIMER
      call interval_timer( 1, TIMER_FwdFFT, 0 )
#  endif
         call copyac( xnlin, fft_buffer( 1, ip ), plndlv*5 )
         ip = ip + 5*plev
         call copyac( t3m1,  fft_buffer( 1, ip ), plndlv )
         ip = ip + plev
         call copyac( psm1,  fft_buffer( 1, ip ), plond )
         ip = ip + 1
         call copyac( vortm1,  fft_buffer( 1, ip ), plndlv )
         ip = ip + plev
         call copyac( divm1,   fft_buffer( 1, ip ), plndlv )
         ip = ip + plev
#  ifdef INTERVAL_TIMER
      call interval_timer( 0, TIMER_FwdFFT, 0 )
#  endif
#else
         call copyac(xnlin,xnlin_sv(1,lat),plndlv*7)
#endif
C
C Accumulate statistics for diagnostic print
C
         call stats(lat     ,irow ,pint(1,1,lat) ,pdel(1,1,lat),
     $              pstar(1,lat),
     $              vort    ,div     ,t3      ,q3      )
      end if
C
C Dump most history tape fields to buffer
C (Tendencies calculated above and dumped to ht on next iter)
C
      call outfld('PHIS    ',phis    ,plond,lat,hbuf,lhbuf)
      call outfld('ORO     ',oro     ,plond,lat,hbuf,lhbuf)
      call outfld('WET     ',ws      ,plond,lat,hbuf,lhbuf)
      call outfld('SNOWH   ',sn      ,plond,lat,hbuf,lhbuf)
      call outfld('PRECL   ',precl(1,lat)   ,plond,lat,hbuf,lhbuf)
      call outfld('PRECC   ',precc(1,lat)   ,plond,lat,hbuf,lhbuf)
C
C      call outfld('PRECSL  ',precsl(1,lat)  ,plond,lat,hbuf,lhbuf)
C      call outfld('PRECSC  ',precsc(1,lat)  ,plond,lat,hbuf,lhbuf)
C      call outfld('RUNOFF  ',rnfp1   ,plond,lat,hbuf,lhbuf)
C
      call outfld('OMEGA   ',omga(1,1,lat)  ,plond,lat,hbuf,lhbuf)
C
C Outfield avg omega when omega up
C      
      if (ninavg(1).eq.'Q') then
         do k=1,plev
            do i=1,plon
               numomgup(i,k) = 0.
               omegaup(i,k) = 0.
               if (omga(i,k,lat).lt.0.) then
                  numomgup(i,k) = 1.0
                  omegaup(i,k) = omga(i,k,lat)
               end if
            end do
         end do
         call outfld('NUMOMGUP',numomgup,plond,lat,hbuf,lhbuf)
         call outfld('OMEGAUP ',omegaup ,plond,lat,hbuf,lhbuf)
      end if
      call outfld('DQP     ',qc(1,1,lat)   ,plond,lat,hbuf,lhbuf)
      call outfld('CMFDT   ',cmfdt(1,1,lat)   ,plond,lat,hbuf,lhbuf)
      call outfld('CMFDQ   ',cmfdq(1,1,lat)   ,plond,lat,hbuf,lhbuf)
C
C     call outfld('CMFDQR  ',cmfdqr(1,1,lat)  ,plond,lat,hbuf,lhbuf)
C
      call outfld('CMFMC   ',cmfmc(1,1,lat)   ,plond,lat,hbuf,lhbuf)
      call outfld('CMFSL   ',cmfsl(1,1,lat)   ,plond,lat,hbuf,lhbuf)
      call outfld('CMFLQ   ',cmflq(1,1,lat)   ,plond,lat,hbuf,lhbuf)
      call outfld('QRS     ',qrs     ,plond,lat,hbuf,lhbuf)
      call outfld('QRL     ',qrl     ,plond,lat,hbuf,lhbuf)
C
      return
      end
