#include "params.h"
      subroutine caldyi(nstep   ,dtime   ,mdbase  ,msbase  ,mbdate  ,
     $                  mbsec   ,mdcur   ,mscur   ,mcdate  ,mcsec   ,
     $                  calday  )
C-----------------------------------------------------------------------
C
C Compute current date and day information for history tape header.
C Compute current julian day (including fraction) for sun angle 
C computations and time interpolation of boundary dataset information.
C One year is defined as YEAR_LENGTH days.
C
C Computational notes: 
C
C 86400 is the number of seconds in 1 day.
C
C Dividing an integer by 10**n has the effect of right-shifting the           
C decimal digits n positions (ex: 861231/100 = 008612).
C
C mod(integer,10**n) has the effect of extracting the rightmost     
C n decimal digits of the integer (ex: mod(861231,10000) = 1231).
C
C---------------------------Code history--------------------------------
C
C Original version:  J. Rosinski
C Standardized:      J. Rosinski, June 1992
C Reviewed:          J. Kiehl, B. Boville, August 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer nstep        ! current time step
      real dtime           ! length of time step (seconds)
      integer mdbase,      ! base day of run
     $                speedup,
     $        msbase,      ! base seconds of base day
     $        mbdate,      ! base date (yymmdd format) of run
     $        mbsec        ! base seconds of base date
C
C Output arguments
C
      integer mdcur,       ! current day
     $              oldmdcur,
     $        mscur,       ! current seconds of current day
     $              oldmscur,
     $        mcdate,      ! current date (yymmdd format)
     $        mcsec,        ! current seconds of current date
     $              oldmcsec
      real calday          ! current julian day including fraction
C
C---------------------------Local workspace-----------------------------
C
      integer*8 nstep8     ! current time step (0, 1, ...)
      integer*8 nsecs      ! run time in seconds

      integer oldnsecs,
     $        ndays,       ! ndays + nyears = number of day changes since start
     $               oldndays,
     $        nyears,      ! ndays + nyears = number of day changes since start
     $        mcyear,      ! current year of current date (0-99)
     $        mcmnth,      ! current month of current date (1-12)
     $        mcday,       ! current day of current date (1-31)
     $        jday,        ! Julian day (1-YEAR_LENGTH)
     $        mbmnth,      ! base month (1-12) validity check
     $        mbday        ! base day of base date (1-31) validity check
      integer ndm(12),     ! number of days per month
     $        jdcon(12)    ! convert month index to julian day
      integer*8 nspdy
      external endrun        ! error abort call
      save ndm,jdcon
      parameter (nspdy=86400)
#if (YEAR_LENGTH == 360) 
      data ndm/30,30,30,30,30,30,30,30,30,30,30,30/
      data jdcon/0,30,60,90,120,150,180,210,240,270,300,330/
#else
      data ndm/31,28,31,30,31,30,31,31,30,31,30,31/
      data jdcon/0,31,59,90,120,151,181,212,243,273,304,334/
#endif

C
c declarations inserted by implnone


C
C-----------------------------------------------------------------------
C
      nstep8=nstep
C Check validity of input data
C
      mbmnth = mod(mbdate,10000)/100
      mbday = mod(mbdate,100)
      if (mbmnth.lt.1 .or. mbmnth.gt.12) then
         write(6,*)' CALDYI: Invalid base month input:',mbmnth
         call endrun
      end if
      if (mbday.lt.1 .or. mbday.gt.ndm(mbmnth)) then
         write(6,*)' CALDYI: Invalid base day of base date input:',mbday
         call endrun
      end if
      if (msbase.lt.0 .or. msbase.ge.nspdy) then
         write(6,*)' CALDYI: Invalid base seconds(msbase):',msbase
         call endrun
      end if
      if (mbsec.lt.0 .or. mbsec.ge.nspdy) then
         write(6,*)' CALDYI: Invalid base seconds(mbsec):',mbsec
         call endrun
      end if
C
       nsecs = nstep8*nint(dtime)
C
C First: current day, seconds
C
      mdcur = mdbase + (nsecs+msbase)/nspdy
      mscur = mod((nsecs+msbase),nspdy)

C Next: current date, seconds.
C Uncommenting the next line will have the effect of modifying  
C nsecs to include base day and base seconds.  This is the way             
C the current date was computed in CCM1.
C
C      nsecs = nsecs + mdbase*86400 + msbase

      ndays = (nsecs+mbsec)/nspdy
      nyears = ndays/YEAR_LENGTH
      ndays = mod(ndays,YEAR_LENGTH)
C
C Current seconds of current date
C
      mcsec = mod(nsecs+mbsec,nspdy)

C
C Initialize current year, month, day.
C
      mcyear = mbdate/10000 + nyears
      mcmnth = mod(mbdate,10000)/100
      mcday = mod(mbdate,100) + ndays
C
C Now loop through months, converting yy, mm, and ddd to yymmdd.
C ex: 791235 becomes 800104. 190001370 becomes 19010105.
C
   10 if (mcday.gt.ndm(mcmnth)) then
         mcday = mcday - ndm(mcmnth)
         mcmnth = mcmnth + 1
         if (mcmnth.eq.13) then            ! add a year
            mcyear = mcyear + 1
            mcmnth = 1
         end if
         go to 10
      end if
      mcdate = mcyear*10000 + mcmnth*100 + mcday
C
C Convert current month, day, seconds to Julian day + fraction
C
      jday = jdcon(mcmnth) + mcday
      calday = float(jday) + float(mcsec)/(1.*nspdy)

c     calday = float(jday) + float(mcsec)/86400.
C
      return
      end
