#include "params.h"
      subroutine cubxdr(f,dx,idim,ibeg,len,fxl,fxr)
c
      implicit none
      integer idim,ibeg,len
      real f(idim),dx,fxl(idim),fxr(idim)
c
c...Compute Lagrangian cubic derivative estimates for data on a equally
c   spaced grid.  Suppose grid interval i is centered in a 4 point
c   stencil consisting of grid points i-1, i, i+1, and i+2.  Then the
c   derivative at the left edge of the interval (i.e., grid point i)
c   is stored in fxl(i), and the derivative at the right edge of the
c   interval (i.e., grid point i+1) is stored in fxr(i).  Note that
c   fxl(i) is not necessarily equal to fxr(i-1) even though both of
c   these values are estimates of the derivative at grid point i.
c
c...On entry.
c
c  f       Values on equally spaced grid for which derivatives are compu
c  dx      Value of grid spacing.
c  idim    Length of f, fxl, and fxr.
c  ibeg    First interval of grid for which derivatives are computed.
c  len     Number of grid intervals for which derivatives are computed.
c          (There are idim - 1 intervals between the idim gridpoints
c          represented in f, fxl, and fxr.)
c
c...On return.
c
c  fxl     fxl(i) is the derivative at the left edge of interval i.
c  fxr     fxr(i) is the derivative at the right edge of interval i.
c
      integer i,iend
      real rdx6
c
      iend = ibeg + len - 1
      rdx6 = 1./(6.*dx)
c
      do 10  i=ibeg,iend
         fxl(i) = ( -2.*f(i-1) - 3.*f(i) + 6.*f(i+1) - f(i+2) )*rdx6
         fxr(i) = (  f(i-1) - 6.*f(i) + 3.*f(i+1) + 2.*f(i+2) )*rdx6
 10   continue
c
      return
      end
