#include "params.h"
      subroutine cubzdr(idim,kdim,f,lbasdz,dfz1,dfz2)
c
      implicit none
#include "pmgrid.com"
#include "parslt.com"
c
      integer idim,kdim
      real f(idim,kdim), lbasdz(4,2,kdim),
     *     dfz1(idim,kdim), dfz2(idim,kdim)
c
c...Vertical derivative estimates for a vertical slice using Lagrangian
c   cubic formulas.  Derivatives are set to zero at the top and bottom.
c   At the "inner nodes" of the top and bottom intervals, a "one sided"
c   estimate is used.
c
c...On entry.
c
c  idim    Horizontal dimension of arrays. (idim = plon)
c  kdim    Vertical dimension of arrays.
c  f       Vertical slice of data for which derivative estimates are mad
c  lbasdz  Lagrangian cubic basis functions for evaluating the derivativ
c          on the unequally spaced vertical grid.
c
c...On return.
c
c  dfz1    dfz1 contains derivative estimates at the "top" edges of the
c          intervals in the f array.
c  dfz2    dfz2 contains derivative estimates at the "bottom" edges of t
c          intervals in the f array.
c
c...Parameters required.
c
c  plon    Number of longitudes in global grid.
c
c
      integer i,k
c
      do 30  k=2,kdim-2
         do 10  i=1,idim
c
c           ...Lagrangian derivative estimates (cubic) for the two
c              center nodes in a four node stencil.
c
            dfz1(i,k) = lbasdz(1,1,k)*f(i,k-1) +
     *                  lbasdz(2,1,k)*f(i,k)   +
     *                  lbasdz(3,1,k)*f(i,k+1) +
     *                  lbasdz(4,1,k)*f(i,k+2)
            dfz2(i,k) = lbasdz(1,2,k)*f(i,k-1) +
     *                  lbasdz(2,2,k)*f(i,k)   +
     *                  lbasdz(3,2,k)*f(i,k+1) +
     *                  lbasdz(4,2,k)*f(i,k+2)
c
 10      continue
 30   continue
c
c     ...Constrain derivatives to zero at top and bottom of vertical
c        grid.
c     ...At the interior nodes of the intervals at the top and bottom
c        of the vertical grid, use the derivative estimate at that
c        same node for the adjacent interval.  (This is a "one-sided"
c        estimate for that node.)
c
      do 40  i=1,idim
         dfz1(i,1) = 0.0
         dfz2(i,1) = dfz1(i,2)
 40   continue
c
      do 50  i=1,idim
         dfz1(i,kdim-1) = dfz2(i,kdim-2)
         dfz2(i,kdim-1) = 0.0
 50   continue
c
      return
      end
