#include "params.h"
      subroutine herzin(kdim,nf,f,fst,fsb,sig,dsig,sigdp,kdp,fdp)
c
      implicit none
#include "pmgrid.com"
#include "parslt.com"
c
      integer ngls
      parameter ( ngls = plon*plev )
c
      integer kdim,nf,kdp(plon,plev)
      real f(plon,kdim,nf), fst(plon,kdim,nf), fsb(plon,kdim,nf),
     *     sig(kdim), dsig(kdim), sigdp(plon,plev), fdp(plon,plev,nf)
c
c...Interpolate field on vertical slice to departure point using Hermite
c   cubic interpolation.
c
c...On entry.
c
c  kdim    Vertical dimension of vertical slice arrays.
c  nf      Number of fields being interpolated.
c  f       Vertical slice of data to be interpolated.
c  fst     z-derivatives at the top edge of each interval contained in f
c  fsb     z-derivatives at the bottom edge of each interval contained i
c  sig     Sigma values corresponding to the vertical grid in f.
c  dsig    Increment in sigma value for each interval in vertical grid.
c  sigdp   Sigma value at the trajectory midpoint or endpoint for each
c          gridpoint in a vertical slice from the global grid.
c  kdp     Vertical index for each gridpoint.  This index points into a
c          vertical slice array whose vertical grid is given by sig.
c          E.g.,   sig(kdp(i,j)) .le. sigdp(i,j) .lt. sig(kdp(i,j)+1) .
c
c...On return.
c
c  fdp     Value of field at the trajectory midpoints or endpoints.
c
c...Parameters required.
c
c  plev    Number of levels in global grid.
c  plon    Number of longitudes in global grid.
c
c
      integer i,k,n
      real dzk(plon,plev),zt(plon,plev),zb(plon,plev),
     *     ht(plon,plev),hb(plon,plev),dht(plon,plev),dhb(plon,plev)
c
#ifdef CRAY 
      do 10   i = 1,ngls
         dzk(i,1) = dsig(kdp(i,1))
         zt (i,1) = ( sig(kdp(i,1)+1) - sigdp(i,1) )/dzk(i,1)
         zb (i,1) = 1. - zt(i,1)
         ht (i,1) = ( 3.0 - 2.0*zt(i,1) )*zt(i,1)**2
         hb (i,1) = ( 3.0 - 2.0*zb(i,1) )*zb(i,1)**2
         dht(i,1) = -dzk(i,1)*( zt(i,1) - 1. )*zt(i,1)**2
         dhb(i,1) =  dzk(i,1)*( zb(i,1) - 1. )*zb(i,1)**2
   10 continue
#else
      do 10   k = 1,plev
       do 10  i = 1,plon
         dzk(i,k) = dsig(kdp(i,k))
         zt (i,k) = ( sig(kdp(i,k)+1) - sigdp(i,k) )/dzk(i,k)
         zb (i,k) = 1. - zt(i,k)
         ht (i,k) = ( 3.0 - 2.0*zt(i,k) )*zt(i,k)**2
         hb (i,k) = ( 3.0 - 2.0*zb(i,k) )*zb(i,k)**2
         dht(i,k) = -dzk(i,k)*( zt(i,k) - 1. )*zt(i,k)**2
         dhb(i,k) =  dzk(i,k)*( zb(i,k) - 1. )*zb(i,k)**2
   10 continue
#endif
c
c     ...Loop over fields.
c
      do 100      n = 1,nf
         do 90    k = 1,plev
            do 80 i = 1,plon
               fdp(i,k,n) =
     $            f(i,kdp(i,k)  ,n)*ht(i,k) + fst(i,kdp(i,k),n)*dht(i,k)
     *          + f(i,kdp(i,k)+1,n)*hb(i,k) + fsb(i,kdp(i,k),n)*dhb(i,k)
   80       continue
   90    continue
c
  100 continue
      return
      end
