#include "params.h"
      subroutine lagyin(nf,fint,wdy,ydp,jdp,fdp)
c
      implicit none
#include "pmgrid.com"
#include "parslt.com"
      integer jxdm,ngls
#ifdef PP_PCCM
      parameter( jxdm= 2*jintmx + p_lato2 + 2*nxpt,
     *           ngls = plon*plev )
#else
      parameter( jxdm= 2*jintmx + plat + 2*nxpt,
     *           ngls = plon*plev )
#endif
c

      integer jgc_foo
      common /debug0804/ jgc_foo

      integer nf,jdp(ngls)
      real fint(ngls,npdy,nf),wdy(4,2,jxdm),ydp(ngls),fdp(ngls,nf)
c
c...For each departure point in the latitude slice to be forecasted,
c   interpolate (using unequally spaced Lagrange cubic formulas) the
c   x interpolates to the y value of the departure point.
c
c...On entry.
c
c  nf      Number of fields being interpolated.
c  fint    (fint(i,k,j,n),j=npdy/2,npdy/2 + 1) contains the x interpolan
c          at the endpoints of the y-interval that contains the departur
c          point for grid point (i,k).  The last index of fint allows fo
c          interpolation of multiple fields.  fint is generated by a cal
c          to herxin.
c  wdy     Grid values and weights for Lagrange cubic interpolation on
c          the unequally spaced y-grid.
c  ydp     ydp(i,k) is the y-coordinate of the departure point that
c          corresponds to global grid point (i,k) in the latitude slice
c          being forecasted.
c  jdp     jdp(i,k) is the index of the y-interval that contains the
c          departure point corresponding to global grid point (i,k) in t
c          latitude slice being forecasted.
c          Note that
c                y(jdp(i,k)) .le. ydp(i,k) .lt. y(jdp(i,k)+1) .
c
c...On return.
c
c  fdp     Horizontally interpolated field values at the departure point
c          for the latitude slice being forecasted.
c
c...Parameters required.
c
c  plon    Number of longitudes in global grid.
c  plev    Number of levels in global grid.
c  jxdm    Number of latitudes in extended array.
c  npdy    The number of points in the stencil for computing the y
c          derivatives.
c  ngls    Number of points in latitude slice of global grid.
c
c
      integer i,k,n
      real  ymy1(ngls)
     $     ,ymy2(ngls)
     $     ,ymy3(ngls)
     $     ,ymy4(ngls)
     $     ,coef12(ngls)
     $     ,coef34(ngls)
     $     ,term1(ngls)
     $     ,term2(ngls)
     $     ,term3(ngls)
     $     ,term4(ngls)
c
      external abortf
c
      if( npdy .ne. 4) then
         write(*,*) '**** error in "lagyin":  npdy .ne. 4'
         call abortf
      endif
c
      do 10      i = 1,ngls
         ymy3  (i) = ydp   (i)       - wdy(3,1,jdp(i))
         ymy4  (i) = ydp   (i)       - wdy(4,1,jdp(i))
         coef12(i) = ymy3  (i)*ymy4(i)
         ymy2  (i) = ydp   (i)       - wdy(2,1,jdp(i))
         term1 (i) = coef12(i)*ymy2(i)*wdy(1,2,jdp(i))
         ymy1  (i) = ydp   (i)       - wdy(1,1,jdp(i))
         term2 (i) = coef12(i)*ymy1(i)*wdy(2,2,jdp(i))
         coef34(i) = ymy1  (i)*ymy2(i)
         term3 (i) = coef34(i)*ymy4(i)*wdy(3,2,jdp(i))
         term4 (i) = coef34(i)*ymy3(i)*wdy(4,2,jdp(i))
   10 continue

c
c     ...Loop over fields.
c
      do 100       n = 1,nf
         do 80     i = 1,ngls
            fdp(i,n) = fint(i,1,n)*term1(i)
     $               + fint(i,2,n)*term2(i)
     $               + fint(i,3,n)*term3(i)
     $               + fint(i,4,n)*term4(i)
   80    continue
  100 continue
c
      return
      end
