#include "params.h"
      subroutine trjgl(jgc,finc,phicen,lam,phib,lampr,phipr,lamp,phip)
c
      implicit none
c
c-----------------------------------------------------------------------
c
c PURPOSE:  Map trajectory mid/departure point coordinates to global
c           latitude/longitude coordinates and test limits
c           
c-------------------------- parameters ---------------------------------
#include "pmgrid.com"
c-----------------------------------------------------------------------
c-------------------------- passed variables ---------------------------
c
c  ...On input
c
      integer
     $      jgc                 ! latitude index in model grid
      real
     $      finc                ! number of time increments
     $     ,phicen              ! current latitude value in extended grid
     $     ,lam(plon)           ! Longitude values for the extended grid
     $     ,phib(platd)         ! Latitude  values for the extended grid
     $     ,lampr(plon,plev)    ! relative x-coordinate of departure pt.
     $     ,phipr(plon,plev)    ! relative y-coordinate of departure pt.
c
c  ...On output
c
      real
     $      lamp (plon,plev)    ! See below
     $     ,phip (plon,plev)    ! See below
c
c
c  lamp    Longitude coordinates of the trajectory mid-points of the
c          parcels that correspond to the global grid points contained
c          in the latitude slice being forecast.
c  phip    Latitude coordinates of the trajectory mid-points of the
c          parcels that correspond to the global grid points contained
c          in the latitude slice being forecast.
c
c-------------------------- local variables ----------------------------
      integer
     $      i                   ! index
     $     ,k                   ! index
     $     ,imax                ! index
     $     ,imin                ! index
     $     ,iimin               ! index
     $     ,kkmin               ! index
     $     ,iimax               ! index
     $     ,kkmax               ! index
     $     ,ismax               ! intrinsic
     $     ,ismin               ! intrinsic
      real
     $      pi
     $     ,twopi

c-----------------------------------------------------------------------
c
      pi     = 4.*atan(1.)
      twopi  = pi*2.
c
#ifdef PP_PCCM
      do 20          k = 1,plev
         do 10       i = 1,plon
            lamp(i,k)  = lam(i) + finc*lampr(i,k)
            phip(i,k)  = phicen + finc*phipr(i,k)
            if (p_nproc_ln.eq.1) then
              if(lamp(i,k) .ge. twopi) lamp(i,k) = lamp(i,k) - twopi
              if(lamp(i,k) .lt.   0.0) lamp(i,k) = lamp(i,k) + twopi
            endif
   10    continue
   20 continue
#else
      do 20          k = 1,plev
         do 10       i = 1,plon
            lamp(i,k)  = lam(i) + finc*lampr(i,k)
            phip(i,k)  = phicen + finc*phipr(i,k)
            if(lamp(i,k) .ge. twopi) lamp(i,k) = lamp(i,k) - twopi
            if(lamp(i,k) .lt.   0.0) lamp(i,k) = lamp(i,k) + twopi
   10    continue
   20 continue
#endif
c+
c     Test that the latitudinal extent of trajectory is NOT over the
c     poles
c-
      imax = ismax( plon*plev, phip, 1 )
      imin = ismin( plon*plev, phip, 1 )
      kkmax   = (imax-1)/plon + 1
      iimax   = imax - (kkmax-1)*plon
      kkmin   = (imin-1)/plon +1
      iimin   = imin - (kkmin-1)*plon
#ifdef PP_PCCM
      if    ( phip(iimax,kkmax) .ge. phib(j1+p_lato2) ) then
#else
      if    ( phip(iimax,kkmax) .ge. phib(j1+plat) ) then
#endif
         write(*,900) iimax,kkmax,jgc
         call abortf('departure point out of bounds in routine trjgl')
      elseif( phip(iimin,kkmin) .lt. phib(j1-1)    ) then
         write(*,901) iimin,kkmin,jgc
         call abortf('departure point out of bounds in routine trjgl')
      endif
      return
c
c****** formats
c
  900 format(//' *** Error in subroutine trjgl ***'/
     *   ' Parcel associated with longitude ',i5,', level ',i5,
     *   ' and latitude ',i5,' is outside the model domain.'/
     *   ' Probable cause:  excessive Northerly winds' )
  901 format(//' *** Error in subroutine trjgl ***'/
     *   ' Parcel associated with longitude ',i5,', level ',i5,
     *   ' and latitude ',i5,' is outside the model domain.'/
     *   ' Probable cause:  excessive Southerly winds' )
c
      end
