#include "params.h"
      subroutine vrtdep(pmap,dt,iterdp,jcen,wb,wst,wsb,sig,sigh,dsig,
     $                  dsigh,kdpmpf,kdpmph,sigmp,sigdp,kdp)
c
      implicit none
#include "pmgrid.com"
      integer plevp1
      parameter( plevp1= plev + 1 )
c
      integer pmap
      integer iterdp, jcen,kdpmpf(pmap),kdpmph(pmap),kdp(plon,plev)
      real dt, wb(plon,plevp1), wst(plon,plevp1), wsb(plon,plevp1),
     *     sig(plev), sigh(plevp1), dsig(plev), dsigh(plevp1),
     *     sigmp(plon,plev), sigdp(plon,plev)
c
c...Computation of vertical departure point and departure point index.
c
c...On entry.
c
c  pmap    Dimension of kdpmap arrays
c  dt      Time interval that parameterizes the parcel trajectory.
c  iterdp  Number of iterations used for departure point calculation.
c  wb      Vertical velocity component (sigma dot).
c  wst     z-derivatives at the top edge of each interval contained in w
c  wsb     z-derivatives at the bottom edge of each interval contained i
c  sig     Sigma values at the full-index levels.
c  sigh    Half-index sigma levels including sigh(1) = sigma(1/2) = 0.0
c          sigh(plev+1) = sigma(plev+1/2) = 1.0 .  Note that in general
c          sigh(k) .lt. sig(k)  where sig(k) is the sigma value at the
c          k_th full-index level.
c  dsig    Increment in full-index sigma levels.
c  dsigh   Increment in half-index sigma levels.
c  kdpmpf  Array of indicies of the model full levels which are mapped
c          into an artificial evenly spaced vertical grid.  Used to aid
c          in search for vertical position of departure point 
c  kdpmph  Array of indicies of the model half levels which are mapped
c          into an artificial evenly spaced vertical grid.  Used to aid
c          in search for vertical position of departure point 
c  sigmp   Sigma value at the trajectory midpoint for each gridpoint
c          in a vertical slice from the global grid.  On entry sigmp is
c          an initial guess.
c
c...On return.
c
c  sigmp   Iterated sigma value at the trajectory midpoint for each grid
c          in a vertical slice from the global grid.
c  sigdp   Sigma value at the trajectory endpoint for each gridpoint
c          in a vertical slice from the global grid.
c  kdp     Vertical index for each gridpoint.  This index points into a
c          vertical slice array whose vertical grid is given by sig.
c          E.g.,   sig(kdp(i,j)) .le. sigdp(i,j) .lt. sig(kdp(i,j)+1) .
c
c...Parameters required.
c
c  plev    Number of vertical levels in global grid.
c  plon    Number of longitudes in global grid.
c  plevp1  Number of half-index sigma-levels.
c
c
      integer i,iter,k,kx
      real wmp(plon,plev)
c
      external kdpfnd,herzin,vdplim
c
c     ...Loop over departure point iterates.
c
      do 30  iter=1,iterdp
c
c        ...Compute midpoint indicies in half-index sigma-level arrays
c           (use kdp as temporary storage).
c
         call kdpfnd(plevp,pmap,sigh,sigmp,kdpmph,kdp)
c
c        ...Interpolate sigma dot field to trajectory midpoints using
c           Hermite cubic interpolant.
c
         call herzin(plevp1,1,wb,wst,wsb,sigh,dsigh,sigmp,kdp,wmp)
c
c        ...Update estimate of trajectory midpoint.
c
         do 20  k=1,plev
            do 19  i=1,plon
               sigmp(i,k) = sig(k) - .5*dt*wmp(i,k)
   19       continue
   20    continue
c
c     ...Restrict vertical midpoints to be between the top and bottom
c        half-index sigma levels.
c
         call vdplim(plevp1,sigh,sigmp)
   30 continue
c
c     ...Compute trajectory endpoints.
c
      do 40  k=1,plev
         do 39  i=1,plon
            sigdp(i,k) = sig(k) - dt*wmp(i,k)
   39    continue
   40 continue
c
c     ...Restrict vertical departure points to be between the top and bo
c        full-index sigma levels.
c
      call vdplim(plev,sig,sigdp)
c
c     ...Vertical indices for trajectory endpoints that point into
c        full-index sigma level arrays.
c
      call kdpfnd(plev,pmap,sig,sigdp,kdpmpf,kdp)
c
      return
      end
