#include "params.h"
      subroutine vrtmap(kdim,pmap,sigln,dsigln,kdpmap)
c
c-----------------------------------------------------------------------
c PURPOSE:
c
c   Map indices of an artificial evenly spaced (in log) vertical grid
c   to the indices of the log of the model vertical grid.  The
c   resultant array of mapped indices will be used by "kdpfnd" to find
c   the vertical location of the departure point relative to the model
c   grid.
c
c-----------------------------------------------------------------------
      implicit none
c-------------------------- input arguments ----------------------------
      integer
     $      kdim                ! dimension of "sigln" and "dsigln"
     $     ,pmap                ! dimension of "kdpmap"
      real
     $      sigln(kdim)         ! model levels (log(eta))
     $     ,dsigln(kdim)        ! intervals between model levels (log)
c-------------------------- output arguments ---------------------------
      integer kdpmap(pmap)      ! array of mapped indices
c------------------------ local variables ------------------------------
      integer imin,k,kk
      real
     $      del                 ! artificial grid interval
     $     ,dp                  ! artificial departure point
     $     ,eps                 ! epsilon factor
c--------------------------- externals ---------------------------------
      integer  ismin
      external abortf,ismin
c-----------------------------------------------------------------------
c
      eps    = 1.e-05
      del    = ( sigln(kdim) - sigln(1) )/float(pmap)
      imin   = ismin( kdim-1,dsigln, 1 )
      if(del + eps .ge. dsigln(imin)) then
         write(*,*)'**** Error in vrtmap:  Not enough artificial grid',
     $ 'intervals.  User must increase parameter pmap'
#ifdef PP_PCCM
         write(*,*) 'eps = ',eps
         write(*,*) 'pmap = ',pmap
         write(*,*) 'sigln(1) = ',sigln(1)
         write(*,*) 'kdim = ',kdim
	 write(*,*) 'kdim-1', kdim-1
         write(*,*) 'sigln(kdim) = ',sigln(kdim)
	 write(*,*) 'del = ',del
	 write(*,*) 'del + eps = ',del+eps
	 write(*,*) 'imin = ',imin
	 write(*,*) 'dsigln(imin) = ',dsigln(imin)
#endif
         call abortf
      endif      
c
      kdpmap(1) = 1
      do 20 kk = 2,pmap
         dp   = sigln(1) + float(kk-1)*del
         do 10 k = 1,kdim-1
            if(dp .gt. sigln(k) + eps) then
               kdpmap(kk) = k
            endif
   10    continue
   20 continue
c
      return
      end
