#include "params.h"
C----------------------------------------------------------------------
C  FORWARD FULL-TRANSPOSE FFT
C                    FFT
C  (nllon_p+2,nvect) ---> (mxlfc_s,nvect)
C
C  Arguments:
C     fc_i    : Input and output array.
C     work    : Work array; size is 2 x size(fc_i).
C     nvect   : Number of FFTs in block
C
C Ian Foster, Argonne National Laboratory, January 1993.
c -----
c adapted to PCCM, John Michalakes, ANL, FEB 1993.
c -----
c rewritten to use PSTSWM software and to not transpose truncated
c Fourier coefficients, Pat Worley, ORNL, FEB 1995.
C----------------------------------------------------------------------
      subroutine fwd_fftx(fc_i,work1,work2,nvect)

#ifdef PP_FFTX
      implicit none
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
#include "comparll.com"
#include "comfft.com"
#include "comxpose.com"
#include "palgor.com"

      integer jump
#ifdef ESSL_FFT
      parameter ( jump = p_glon + 2 )
#else
      parameter ( jump = p_glon )
#endif

      integer nvect
      real fc_i(plond, nvect )
      real work1(plond, nvect )
      real work2(jump , * )
      integer i
      integer ntmp1, ntmp2, mxlvect, nlvect
      integer nlvects(0:p_nproc_ln-1)
c
      if ( .not. xpose_init ) call xpose_ini

      if (p_nproc_ln .gt. 1) then
C       Compute actual number of vectors; may be 0:
        ntmp1 = nvect/p_nproc_ln
        ntmp2 = mod(nvect,p_nproc_ln)
        if (ntmp2 .gt. 0) then
          mxlvect = ntmp1 + 1
        else
          mxlvect = ntmp1
        endif
        do i=0,ntmp2-1
          nlvects(i) = ntmp1+1
        enddo
        do i=ntmp2,p_nproc_ln-1
          nlvects(i) = ntmp1
        enddo
        nlvect = nlvects(m_pin_ln)
c
c       reorder and transpose fields
        call reorder1(fc_i, work1, plond, plon, nvect)
        call transpose(p_tfftopt, p_tfftbufs, p_tfftprot,
     &                 p_nproc_ln, maprow, m_pin_ln, m_type,
     &                 -1, 1, p_glon, nvect, 1, 1,
     &                 nllons, nlvects, jump, work1,
     &                 work2, work2(1,mxlvect+1))
        m_type = m_type+p_nproc
c
c       serial real Fourier transform
        if (nlvect .gt. 0) then
#ifdef ESSL_FFT
          call esslft(work2(1,mxlvect+1),work2,
     &                1, jump, p_glon, nlvect, -1)
#else
          call RFT_235(0, 0, 1, maprow(m_pin_ln), 0,
     &                 m_type, m_type, -1, p_glon,
     &                 nlvect, p_glon, mxtrigs, trig, 
     &                 work2, work2(1,mxlvect+1))
#endif
c         "unpack" the last (truncated) Fourier coefficient from the 
c         first complex value - assumes that the last wavenumber is
c         always truncated, and that wavenumber 0 is always the first
          do i=mxlvect+1,mxlvect+nlvect
            work2(2,i) = 0.0
          enddo
        endif
c
c       reorder and transpose fields
        call reorder2(work2(1,mxlvect+1),work2,jump/2,
     &                jump/2,nlvect,ordfwd_s) 
        call transpose(p_tfftopt, p_tfftbufs, p_tfftprot,
     &                 p_nproc_ln, maprow, m_pin_ln, m_type,
     &                 -6, 2, nvect, ptrm+1, 1, 1,
     &                 nlvects, nlfcs, plond, work2,
     &                 work2(1,mxlvect+1), fc_i)
        m_type = m_type+p_nproc
      else
c       serial real Fourier transform
        if (nvect .gt. 0) then
#ifdef ESSL_FFT
          call esslft( fc_i,work2,1,plond,plon,nvect,-1)
#else
          call RFT_235(0, 0, 1, maprow(m_pin_ln), 0,
     &                 m_type, m_type, -1, plon,
     &                 nvect, plond, mxtrigs, trig, 
     &                 work2, fc_i)
#endif
c         "unpack" the last (truncated) Fourier coefficient from the 
c         first complex value - assumes that the last wavenumber is
c         always truncated
          do i=1,nvect
            fc_i(2*m_pwave0_dex,i)  = 0.0
          enddo
        endif
      endif
c
#endif
      return
      end

#ifdef PP_FFTX
C ------------------------------------------------------------
C REORDER1(from,to,nlond,nlon,nvect)
C 
C FROM(nlond,nvect) ->
C	TO(nlon,nvect)
C ------------------------------------------------------------
      subroutine reorder1(from,to,nlond,nlon,nvect)
      implicit none
      integer nvect,nlon,nlond
      real from(nlond,nvect)
      real to(nlon,nvect)
      integer i,j
      
      do j = 1,nvect
         do i = 1,nlon
             to(i,j) = from(i,j)
         enddo
      enddo

      return
      end


C ------------------------------------------------------------
C REORDER2(from,to,nfcd,nfc,nlvect,order) 
C 
C FROM(nfcd,nlvect) ->
C	TO(nlvect,order(nfc))
C ------------------------------------------------------------
      subroutine reorder2(from,to,nfcd,nfc,nlvect,order)
      implicit none
      integer nfcd,nfc,nlvect,order(nfc)
      complex from(nfcd,nlvect)
      complex to(nlvect,nfc)
      integer i,j,k
      
      do j = 1,nlvect
        do i = 1,nfc
          to(j,order(i)) = from(i,j)
        enddo
      enddo
      
      return
      end
C ------------------------------------------------------------

#endif
