#include "params.h"
      subroutine spetru(ps      ,phis    ,u3      ,v3      ,t3      ,
     $                  q3      ,vort    ,div     ,pl      ,pm)
c-----------------------------------------------------------------------
c Spectrally truncate input fields which have already
c been transformed into spectral space.
c Some arrays are dimensioned (2,...), where (1,...) is the
c real part of the complex fourier coefficient, and (2,...) is th
c imaginary.  Any array dimensioned (plond,...) *cannot* be
c dimensioned (2,plond/2,...) because plond *may* be (and in fact
c currently is) odd.  In these cases reference to real and
c imaginary parts is by (2*m-1,...) and (2*m  ,...) respectively.
c-----------------------------------------------------------------------
c inserted by implnone
      implicit none
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
c-----------------------------------------------------------------------
#include "comcon.com"
c-----------------------------------------------------------------------
#include "commap.com"
c-----------------------------------------------------------------------
#include "comspe.com"
c-----------------------------------------------------------------------
#ifdef PP_PCCM
#include "comparll.com"
#endif
c-----------------------------------------------------------------------
c
c Input/Output arguments
c
#ifdef PP_PCCM
      real ps(plond,plat),        ! Fourier coeffs. for ln(ps)
     $     phis(plond,plat),      ! ditto for surf. geopotential
     $     u3(plond,plevd,platd,2),  ! ditto for u-wind
     $     v3(plond,plevd,platd,2),  ! ditto for v-wind
     $     t3(plond,plevd,platd,2),  ! ditto for temperature
     $     q3(plond,plevd,platd,2)   ! ditto for moisture
#else
      real ps(plond,plat),        ! Fourier coeffs. for ln(ps)
     $     phis(plond,plat),      ! ditto for surf. geopotential
     $     u3(plond,plevd,plat),  ! ditto for u-wind
     $     v3(plond,plevd,plat),  ! ditto for v-wind
     $     t3(plond,plevd,plat),  ! ditto for temperature
     $     q3(plond,plevd,plat)   ! ditto for moisture
#endif
c
c Output arguments
c
      real vort(plond,plev,plat), ! Spectrally truncated vorticity
     $     div(plond,plev,plat),  ! ditto for divergence
     $     pl(plond,plat),        ! ditto for d(ln(ps))/d(longitude)
     $     pm(plond,plat)         ! ditto for d(ln(ps))/d(latitude)
c
c Local workspace
c
#ifdef PP_PCCM
c some additional indexing variables
      integer lathp, lathm	! indexes into hemispheric grids
      integer ih		! 1 = southern hemi, 2 = northern
c      real phi(2,(psp*max((p_log2plat-1),1))/2),
      integer phisize
      parameter (phisize=max(p_log2plat-1,1))
      real phi(2,psp*phisize/2),
     $     alpn(pspt),
     $     dalpn(pspt)
#else
      real phi(2,psp/2),
     $     alpn(pspt),
     $     dalpn(pspt)
#endif
c declarations inserted by implnone
      real phialpi,phialpr,psalpi,psalpr,psdalpi,psdalpr,tmp1,tmp2,
     $ tmpi,tmpr,zcor,zcsj,zrcsj,zw,zwalp,zwdalp
c declarations inserted by implnone
      integer i,ii,ir,irow,k,latm,latp,m,mc,mck,mr,n
c
#ifdef PP_PCCM
      integer mm, ma, ip, iq, ij
      real tru_m ! used to hold true wave number
      integer itru_m
#endif
c 
#ifdef FILTPHIS
      integer nspec
      real filtlim,ft
#endif

#ifdef PP_ERROR
	    write(0,*) 'entering spetru', stack_ptr
#endif

c
c Zero spectral arrays
c
      call resetr(vz,pspl,0.)
      call resetr(d,pspl,0.)
      call resetr(t,pspl,0.)
      call resetr(q,pspl,0.)
      call resetr(alps,psp,0.)
      call resetr(phi,psp,0.)
c
      do 160 irow=1,plat/2
         latp = irow
         latm = plat - irow + 1
         zcsj = cs(irow)
         zrcsj = ra/zcsj
         zw = w(irow)*2.
#ifdef PP_PCCM
	 lathp = j1m + irow
	 lathm = j1m + plat/2 - irow + 1

#  ifdef PP_TRIANG_TRUNC
	 if ( irow - 1 .eq. 0 ) stack_ptr = -1
	 if ( mod( irow - 1, 2 ) .eq. 0 ) then
	     stack_ptr = stack_ptr + 1
             call resetr(vz( psp * plev * stack_ptr + 1),pspl,0.)
             call resetr( d( psp * plev * stack_ptr + 1),pspl,0.)
             call resetr( t( psp * plev * stack_ptr + 1),pspl,0.)
             call resetr( q( psp * plev * stack_ptr + 1),pspl,0.)
             call resetr(alps( psp * stack_ptr + 1 ),psp,0.)
             call resetr( phi( 1, pspt * stack_ptr + 1 ),psp,0.)
	 endif
#  else
         stack_ptr = 0
#  endif

         do 50 ii=m_fwavbeg,m_fwavend
          do 50 i = 2*m_fwavdex(ii)-1,2*m_fwavdex(ii)
#else
         do 50 i=1,2*pmmax
#endif

c
c Compute symmetric and antisymmetric components: ps first
c
            tmp1 = 0.5*(ps(i,latm) - ps(i,latp))
            tmp2 = 0.5*(ps(i,latm) + ps(i,latp))
            ps(i,latm) = tmp1
            ps(i,latp) = tmp2
c
c phis
c
            tmp1 = 0.5*(phis(i,latm) - phis(i,latp))
            tmp2 = 0.5*(phis(i,latm) + phis(i,latp))
            phis(i,latm) = tmp1
            phis(i,latp) = tmp2
   50    continue
         do 60 k=1,plev
#ifdef PP_PCCM
            do 55 ii=m_fwavbeg,m_fwavend
             do 55 i = 2*m_fwavdex(ii)-1,2*m_fwavdex(ii)
c
c U
c
               tmp1 = 0.5*(u3(i,k,lathm,2) - u3(i,k,lathp,1))
               tmp2 = 0.5*(u3(i,k,lathm,2) + u3(i,k,lathp,1))
               u3(i,k,lathm,2) = tmp1
               u3(i,k,lathp,1) = tmp2
c
c V
c
               tmp1 = 0.5*(v3(i,k,lathm,2) - v3(i,k,lathp,1))
               tmp2 = 0.5*(v3(i,k,lathm,2) + v3(i,k,lathp,1))
               v3(i,k,lathm,2) = tmp1
               v3(i,k,lathp,1) = tmp2
c
c T
c
               tmp1 = 0.5*(t3(i,k,lathm,2) - t3(i,k,lathp,1))
               tmp2 = 0.5*(t3(i,k,lathm,2) + t3(i,k,lathp,1))
               t3(i,k,lathm,2) = tmp1
               t3(i,k,lathp,1) = tmp2
c
c Q
c
               tmp1 = 0.5*(q3(i,k,lathm,2) - q3(i,k,lathp,1))
               tmp2 = 0.5*(q3(i,k,lathm,2) + q3(i,k,lathp,1))
               q3(i,k,lathm,2) = tmp1
               q3(i,k,lathp,1) = tmp2
   55       continue
#else
            do 55 i=1,2*pmmax
c
c U
c
               tmp1 = 0.5*(u3(i,k,latm) - u3(i,k,latp))
               tmp2 = 0.5*(u3(i,k,latm) + u3(i,k,latp))
               u3(i,k,latm) = tmp1
               u3(i,k,latp) = tmp2
c
c V
c
               tmp1 = 0.5*(v3(i,k,latm) - v3(i,k,latp))
               tmp2 = 0.5*(v3(i,k,latm) + v3(i,k,latp))
               v3(i,k,latm) = tmp1
               v3(i,k,latp) = tmp2
c
c T
c
               tmp1 = 0.5*(t3(i,k,latm) - t3(i,k,latp))
               tmp2 = 0.5*(t3(i,k,latm) + t3(i,k,latp))
               t3(i,k,latm) = tmp1
               t3(i,k,latp) = tmp2
c
c Q
c
               tmp1 = 0.5*(q3(i,k,latm) - q3(i,k,latp))
               tmp2 = 0.5*(q3(i,k,latm) + q3(i,k,latp))
               q3(i,k,latm) = tmp1
               q3(i,k,latp) = tmp2
   55       continue
#endif
   60    continue

c
c Compute vzmn,dmn and ln(p*)mn and also phi*mn,tmn and qmn
c
#ifdef PP_PCCM
         do mm=m_fwavbeg,m_fwavend
            m  = m_fwavdex(mm)
	    ma = m_offalp(mm)
            mr = ma + stack_ptr*pspt
            mc = 2*mr
            do n=1,m_lenalp(mm),2
               zwalp = zw*alp(ma+n,irow)
#else
         do m=1,pmmax
            mr = nstart(m)
            mc = 2*mr
            do n=1,nlen(m),2
               zwalp = zw*alp(mr+n,irow)
#endif
               phi(1,mr+n) = phi(1,mr+n) + zwalp*phis(2*m-1,latp)
               phi(2,mr+n) = phi(2,mr+n) + zwalp*phis(2*m  ,latp)
               ir = mc + 2*n - 1
               ii = ir + 1
               alps(ir) = alps(ir) + zwalp*ps(2*m-1,latp)
               alps(ii) = alps(ii) + zwalp*ps(2*m  ,latp)
            end do
#ifdef PP_PCCM
            do n=2,m_lenalp(mm),2
               zwalp = zw*alp(ma+n,irow)
#else
            do n=2,nlen(m),2
               zwalp = zw*alp(mr+n,irow)
#endif
               phi(1,mr+n) = phi(1,mr+n) + zwalp*phis(2*m-1,latm)
               phi(2,mr+n) = phi(2,mr+n) + zwalp*phis(2*m  ,latm)
               ir = mc + 2*n - 1
               ii = ir + 1
               alps(ir) = alps(ir) + zwalp*ps(2*m-1,latm)
               alps(ii) = alps(ii) + zwalp*ps(2*m  ,latm)
            end do
         end do
         do k=1,plev
#ifdef PP_PCCM
            do mm=m_fwavbeg,m_fwavend
               m  = m_fwavdex(mm)
	       ma = m_offalp(mm)
               mr = ma + stack_ptr*pspt*plev
               mc = 2*mr
c               mck = (k-1)*psp + mc
               mck = (k-1)*m_nspect2 + mc
               tru_m = float(m_mtrue(mm))
               do n=1,m_lenalp(mm),2
                  zwdalp = zw*dalp(ma+n,irow)
                  zwalp  = zw*alp (ma+n,irow)
                  ir = mck + 2*n - 1
                  ii = ir + 1
                  d(ir) = d(ir) -
     $                 (zwdalp*v3(2*m-1,k,lathm,2) +
     $             tru_m*zwalp*u3(2*m  ,k,lathp,1))*zrcsj
                  d(ii) = d(ii) -
     $                 (zwdalp*v3(2*m  ,k,lathm,2) -
     $             tru_m*zwalp*u3(2*m-1,k,lathp,1))*zrcsj
                  t(ir) = t(ir) + zwalp*t3(2*m-1,k,lathp,1)
                  t(ii) = t(ii) + zwalp*t3(2*m  ,k,lathp,1)
                  q(ir) = q(ir) + zwalp*q3(2*m-1,k,lathp,1)
                  q(ii) = q(ii) + zwalp*q3(2*m  ,k,lathp,1)
                  vz(ir) = vz(ir) +
     $                 (zwdalp*u3(2*m-1,k,lathm,2) -
     $             tru_m*zwalp*v3(2*m  ,k,lathp,1))*zrcsj
                  vz(ii) = vz(ii) +
     $                 (zwdalp*u3(2*m  ,k,lathm,2) +
     $             tru_m*zwalp*v3(2*m-1,k,lathp,1))*zrcsj
               end do
            end do
#else
            do m=1,pmmax
               mr = nstart(m)
               mc = 2*mr
               mck = (k-1)*psp + mc
               do n=1,nlen(m),2
                  zwdalp = zw*dalp(mr+n,irow)
                  zwalp  = zw*alp (mr+n,irow)
                  ir = mck + 2*n - 1
                  ii = ir + 1
                  d(ir) = d(ir) -
     $                 (zwdalp*v3(2*m-1,k,latm) +
     $             xm(m)*zwalp*u3(2*m  ,k,latp))*zrcsj
                  d(ii) = d(ii) -
     $                 (zwdalp*v3(2*m  ,k,latm) -
     $             xm(m)*zwalp*u3(2*m-1,k,latp))*zrcsj
                  t(ir) = t(ir) + zwalp*t3(2*m-1,k,latp)
                  t(ii) = t(ii) + zwalp*t3(2*m  ,k,latp)
                  q(ir) = q(ir) + zwalp*q3(2*m-1,k,latp)
                  q(ii) = q(ii) + zwalp*q3(2*m  ,k,latp)
                  vz(ir) = vz(ir) +
     $                 (zwdalp*u3(2*m-1,k,latm) -
     $             xm(m)*zwalp*v3(2*m  ,k,latp))*zrcsj
                  vz(ii) = vz(ii) +
     $                 (zwdalp*u3(2*m  ,k,latm) +
     $             xm(m)*zwalp*v3(2*m-1,k,latp))*zrcsj
               end do
            end do
#endif
#ifdef PP_PCCM
            do mm=m_fwavbeg,m_fwavend
               m  = m_fwavdex(mm)
	       ma = m_offalp(mm)
               mr = ma + stack_ptr*pspt*plev
               mc = 2*mr
               mck = (k-1)*m_nspect2 + mc
               tru_m = float( m_mtrue(mm) )
               do n=2,m_lenalp(mm),2
                  zwdalp = zw*dalp(ma+n,irow)
                  zwalp  = zw*alp (ma+n,irow)
                  ir = mck + 2*n - 1
                  ii = ir + 1
                  d(ir) = d(ir) -
     $                 (zwdalp*v3(2*m-1,k,lathp,1) +
     $             tru_m*zwalp*u3(2*m  ,k,lathm,2))*zrcsj
                  d(ii) = d(ii) -
     $                 (zwdalp*v3(2*m  ,k,lathp,1) -
     $             tru_m*zwalp*u3(2*m-1,k,lathm,2))*zrcsj
                  t(ir) = t(ir) + zwalp*t3(2*m-1,k,lathm,2)
                  t(ii) = t(ii) + zwalp*t3(2*m  ,k,lathm,2)
                  q(ir) = q(ir) + zwalp*q3(2*m-1,k,lathm,2)
                  q(ii) = q(ii) + zwalp*q3(2*m  ,k,lathm,2)
                  vz(ir) = vz(ir) +
     $                 (zwdalp*u3(2*m-1,k,lathp,1) -
     $             tru_m*zwalp*v3(2*m  ,k,lathm,2))*zrcsj
                  vz(ii) = vz(ii) +
     $                 (zwdalp*u3(2*m  ,k,lathp,1) +
     $             tru_m*zwalp*v3(2*m-1,k,lathm,2))*zrcsj
               end do
            end do
#else
            do m=1,pmmax
               mr = nstart(m)
               mc = 2*mr
               mck = (k-1)*psp + mc
               do n=2,nlen(m),2
                  zwdalp = zw*dalp(mr+n,irow)
                  zwalp  = zw*alp (mr+n,irow)
                  ir = mck + 2*n - 1
                  ii = ir + 1
                  d(ir) = d(ir) -
     $                 (zwdalp*v3(2*m-1,k,latp) +
     $             xm(m)*zwalp*u3(2*m  ,k,latm))*zrcsj
                  d(ii) = d(ii) -
     $                 (zwdalp*v3(2*m  ,k,latp) -
     $             xm(m)*zwalp*u3(2*m-1,k,latm))*zrcsj
                  t(ir) = t(ir) + zwalp*t3(2*m-1,k,latm)
                  t(ii) = t(ii) + zwalp*t3(2*m  ,k,latm)
                  q(ir) = q(ir) + zwalp*q3(2*m-1,k,latm)
                  q(ii) = q(ii) + zwalp*q3(2*m  ,k,latm)
                  vz(ir) = vz(ir) +
     $                 (zwdalp*u3(2*m-1,k,latp) -
     $             xm(m)*zwalp*v3(2*m  ,k,latm))*zrcsj
                  vz(ii) = vz(ii) +
     $                 (zwdalp*u3(2*m  ,k,latp) +
     $             xm(m)*zwalp*v3(2*m-1,k,latm))*zrcsj
               end do
            end do
#endif
         end do
#ifdef PP_PCCM

#ifdef PP_TRIANG_TRUNC
c sum up the reproducibility stack as necessary
         if ( mod( irow - 1, 2 ) .eq. 1 ) then
            ii = (irow - 1) / 2
            do while ( mod( ii, 2 ) .eq. 1 )
               ip = stack_ptr * pspt
               iq = ( stack_ptr - 1 ) * pspt
               if ( iq .lt. 0 ) then
                  call abortf(
     $             'internal error: spetru.f repro stack underflow')
               endif
	       do ij = 1, m_nspect
                  phi( 1, iq+ij ) = phi( 1, iq+ij ) + phi( 1, ip+ij )
                  phi( 2, iq+ij ) = phi( 2, iq+ij ) + phi( 2, ip+ij )
	       enddo

               do ij = 1, m_nspect2
                  alps( 2*iq+ij )   = alps( 2*iq+ij ) +
     $                                alps( 2*ip+ij )
               enddo

               ip = stack_ptr * psp * plev
               iq = ( stack_ptr - 1 ) * psp * plev
               do ij = 1, m_nspect2 * plev
                  t( iq+ij ) = t( iq+ij ) + t ( ip+ij )
                  q( iq+ij ) = q( iq+ij ) + q ( ip+ij )
                  d( iq+ij ) = d( iq+ij ) + d ( ip+ij )
                  vz( iq+ij ) = vz( iq+ij ) + vz ( ip+ij )
               enddo
               stack_ptr = stack_ptr - 1
               ii = ii / 2
            enddo
         endif
#endif
#endif
  160 continue

#ifdef PP_PCCM
# if ( PP_NPROC_LT > 1 )
      call colsum0(phi,r_tmp,2*m_nspect,m_type)
      m_type=m_type+2*p_nproc
      call colsum0(alps,r_tmp,2*m_nspect,m_type)
      m_type=m_type+2*p_nproc
      call colsum0(d,r_tmp,2*m_nspect*plev,m_type)
      m_type=m_type+2*p_nproc
      call colsum0(t,r_tmp,2*m_nspect*plev,m_type)
      m_type=m_type+2*p_nproc
      call colsum0(q,r_tmp,2*m_nspect*plev,m_type)
      m_type=m_type+2*p_nproc
      call colsum0(vz,r_tmp,2*m_nspect*plev,m_type)
      m_type=m_type+2*p_nproc
# endif
#endif

c
c Compute grid point values of:phi*,u,v,ln(p*),t,q,vz,d and
c grad(ln(p*)).  Update some constants for fft
c
      do 360 irow=1,plat/2
         latp = irow
         latm = plat - irow + 1
#ifdef PP_PCCM
         lathp = j1m + irow
         lathm = j1m + plat/2 - irow + 1

         if ( l_pwave0 ) then
           zcor = ez*alp(m_offalp(m_pwave0_dex)+2,irow)
         endif
#else
         zcor = ez*alp(2,irow)
#endif
c
c Zero fourier fields
c
         call resetr(phis(1,latm),plond,0.)
         call resetr(phis(1,latp),plond,0.)
c
         call resetr(ps(1,latm),plond,0.)
         call resetr(ps(1,latp),plond,0.)
c
         call resetr(pl(1,latm),plond,0.)
         call resetr(pl(1,latp),plond,0.)
c
         call resetr(pm(1,latm),plond,0.)
         call resetr(pm(1,latp),plond,0.)
#ifdef PP_PCCM
c
         call resetr(u3(1,1,lathm,2),plndlv,0.)
         call resetr(u3(1,1,lathp,1),plndlv,0.)
c
         call resetr(v3(1,1,lathm,2),plndlv,0.)
         call resetr(v3(1,1,lathp,1),plndlv,0.)
c
         call resetr(t3(1,1,lathm,2),plndlv,0.)
         call resetr(t3(1,1,lathp,1),plndlv,0.)
c
         call resetr(q3(1,1,lathm,2),plndlv,0.)
         call resetr(q3(1,1,lathp,1),plndlv,0.)
c
         call resetr(vort(1,1,latm),plndlv,0.)
         call resetr(vort(1,1,latp),plndlv,0.)
c
         call resetr(div(1,1,latm),plndlv,0.)
         call resetr(div(1,1,latp),plndlv,0.)
c
#else
c
         call resetr(u3(1,1,latm),plndlv,0.)
         call resetr(u3(1,1,latp),plndlv,0.)
c
         call resetr(v3(1,1,latm),plndlv,0.)
         call resetr(v3(1,1,latp),plndlv,0.)
c
         call resetr(t3(1,1,latm),plndlv,0.)
         call resetr(t3(1,1,latp),plndlv,0.)
c
         call resetr(q3(1,1,latm),plndlv,0.)
         call resetr(q3(1,1,latp),plndlv,0.)
c
         call resetr(vort(1,1,latm),plndlv,0.)
         call resetr(vort(1,1,latp),plndlv,0.)
c
         call resetr(div(1,1,latm),plndlv,0.)
         call resetr(div(1,1,latp),plndlv,0.)
c
#endif

#ifdef FILTPHIS
c filter the PHI field if its a "pure" geopotential
	 write(0,*) 'FILTERING phis'
	 filtlim=float(int(1.4*float(ptrn)))
#ifdef PP_PCCM
         do mm=m_fwavbeg,m_fwavend
            m  = m_fwavdex(mm)
            mr = m_offalp(mm)
            mc = 2*mr
            do n=1,m_lenalp(mm),2
#else
         do m=1,pmmax
            mr = nstart(m)
            mc = 2*mr
            do n=1,nlen(m)
#endif
             nspec=m-1+n
	     ft=1.-(float(nspec)/filtlim)**2
	     if (float(nspec) .ge. filtlim) ft = 0.
c	     write(0,*)m,n,nspec,ft
	     phi(1,mr+n) = phi(1,mr+n)*ft
	     phi(2,mr+n) = phi(2,mr+n)*ft
            enddo
          enddo
#endif

c Compute(phi*,u,v,ln(p*),t,q,vz,d,grad(ln(p*)))m
c
#ifdef PP_PCCM
         do mm=m_fwavbeg,m_fwavend
            m  = m_fwavdex(mm)
            mr = m_offalp(mm)
            mc = 2*mr
            do n=1,m_lenalp(mm),2
#else
         do m=1,pmmax
            mr = nstart(m)
            mc = 2*mr
            do n=1,nlen(m),2
#endif
               phialpr = phi(1,mr+n)*alp(mr+n,irow)
               phialpi = phi(2,mr+n)*alp(mr+n,irow)
c
               phis(2*m-1,latm) = phis(2*m-1,latm) + phialpr
               phis(2*m  ,latm) = phis(2*m  ,latm) + phialpi
c
               ir = mc + 2*n - 1
               ii = ir + 1
               psalpr = alps(ir)*alp(mr+n,irow)
               psalpi = alps(ii)*alp(mr+n,irow)
c
               ps(2*m-1,latm) = ps(2*m-1,latm) + psalpr
               ps(2*m  ,latm) = ps(2*m  ,latm) + psalpi
               pl(2*m-1,latm) = pl(2*m-1,latm) - psalpi*ra
               pl(2*m  ,latm) = pl(2*m  ,latm) + psalpr*ra
c
               psdalpr = alps(ir)*dalp(mr+n,irow)
               psdalpi = alps(ii)*dalp(mr+n,irow)
c
               pm(2*m-1,latp) = pm(2*m-1,latp) + psdalpr*ra
               pm(2*m  ,latp) = pm(2*m  ,latp) + psdalpi*ra
            end do
         end do

#ifdef PP_PCCM
         do mm=m_fwavbeg,m_fwavend
            m  = m_fwavdex(mm)
            mr = m_offalp(mm)
            mc = 2*mr
            do n=2,m_lenalp(mm),2
#else
         do m=1,pmmax
            mr = nstart(m)
            mc = 2*mr
            do n=2,nlen(m),2
#endif
               phialpr = phi(1,mr+n)*alp(mr+n,irow)
               phialpi = phi(2,mr+n)*alp(mr+n,irow)
c
               phis(2*m-1,latp) = phis(2*m-1,latp) + phialpr
               phis(2*m  ,latp) = phis(2*m  ,latp) + phialpi
c
               ir = mc + 2*n - 1
               ii = ir + 1
               psalpr = alps(ir)*alp(mr+n,irow)
               psalpi = alps(ii)*alp(mr+n,irow)
c
               ps(2*m-1,latp) = ps(2*m-1,latp) + psalpr
               ps(2*m  ,latp) = ps(2*m  ,latp) + psalpi
               pl(2*m-1,latp) = pl(2*m-1,latp) - psalpi*ra
               pl(2*m  ,latp) = pl(2*m  ,latp) + psalpr*ra
c
               psdalpr = alps(ir)*dalp(mr+n,irow)
               psdalpi = alps(ii)*dalp(mr+n,irow)
c
               pm(2*m-1,latm) = pm(2*m-1,latm) + psdalpr*ra
               pm(2*m  ,latm) = pm(2*m  ,latm) + psdalpi*ra
            end do
         end do
#ifdef PP_PCCM
         do mm=m_fwavbeg,m_fwavend
            m  = m_fwavdex(mm)
            mr = m_offalp(mm)
            itru_m = m_mtrue(mm)
	    tru_m = float( itru_m )
            do n=1,m_lenalp(mm)
c
c These statements will likely not be bfb since xm(m)*ra is now a scalar
c
               alpn (mr+n) =  alp(mr+n,irow)*rsq(n+itru_m)*tru_m*ra
               dalpn(mr+n) = dalp(mr+n,irow)*rsq(n+itru_m)      *ra
            end do
         end do
#else
         do m=1,pmmax
            mr = nstart(m)
            do n=1,nlen(m)
c
c These statements will likely not be bfb since xm(m)*ra is now a scalar
c
               alpn (mr+n) =  alp(mr+n,irow)*rsq(n+m-1)*xm(m)*ra
               dalpn(mr+n) = dalp(mr+n,irow)*rsq(n+m-1)      *ra
            end do
         end do
#endif
CDIR$ IVDEP
#ifdef PP_PCCM
         do 230 mm=m_fwavbeg,m_fwavend
            m     = m_fwavdex(mm)
            tru_m = float( m_mtrue(mm) )
            pl(2*m-1,latm) = tru_m*pl(2*m-1,latm)
            pl(2*m  ,latm) = tru_m*pl(2*m  ,latm)
            pl(2*m-1,latp) = tru_m*pl(2*m-1,latp)
            pl(2*m  ,latp) = tru_m*pl(2*m  ,latp)
  230    continue
#else
         do 230 m=1,pmmax
            pl(2*m-1,latm) = xm(m)*pl(2*m-1,latm)
            pl(2*m  ,latm) = xm(m)*pl(2*m  ,latm)
            pl(2*m-1,latp) = xm(m)*pl(2*m-1,latp)
            pl(2*m  ,latp) = xm(m)*pl(2*m  ,latp)
  230    continue
#endif
         do 270 k=1,plev
#ifdef PP_PCCM
            do mm=m_fwavbeg,m_fwavend
               m  = m_fwavdex(mm)
               mr = m_offalp(mm)
               mc = 2*mr
c               mck = (k-1)*psp + mc
               mck = (k-1)*m_nspect2 + mc
               do n=1,m_lenalp(mm),2
                  ir = mck + 2*n - 1
                  ii = ir + 1
c
                  tmpr = d(ir)*alpn(mr+n)
                  tmpi = d(ii)*alpn(mr+n)
                  u3(2*m-1,k,lathm,2) = u3(2*m-1,k,lathm,2) + tmpi
                  u3(2*m  ,k,lathm,2) = u3(2*m  ,k,lathm,2) - tmpr
c
                  tmpr = d(ir)*dalpn(mr+n)
                  tmpi = d(ii)*dalpn(mr+n)
                  v3(2*m-1,k,lathp,1) = v3(2*m-1,k,lathp,1) - tmpr
                  v3(2*m  ,k,lathp,1) = v3(2*m  ,k,lathp,1) - tmpi
c
                  tmpr = vz(ir)*dalpn(mr+n)
                  tmpi = vz(ii)*dalpn(mr+n)
                  u3(2*m-1,k,lathp,1) = u3(2*m-1,k,lathp,1) + tmpr
                  u3(2*m  ,k,lathp,1) = u3(2*m  ,k,lathp,1) + tmpi
c
                  tmpr = vz(ir)*alpn(mr+n)
                  tmpi = vz(ii)*alpn(mr+n)
                  v3(2*m-1,k,lathm,2) = v3(2*m-1,k,lathm,2) + tmpi
                  v3(2*m  ,k,lathm,2) = v3(2*m  ,k,lathm,2) - tmpr
c
                  tmpr = t(ir)*alp(mr+n,irow)
                  tmpi = t(ii)*alp(mr+n,irow)
                  t3(2*m-1,k,lathm,2) = t3(2*m-1,k,lathm,2) + tmpr
                  t3(2*m  ,k,lathm,2) = t3(2*m  ,k,lathm,2) + tmpi
c
                  tmpr = q(ir)*alp(mr+n,irow)
                  tmpi = q(ii)*alp(mr+n,irow)
                  q3(2*m-1,k,lathm,2) = q3(2*m-1,k,lathm,2) + tmpr
                  q3(2*m  ,k,lathm,2) = q3(2*m  ,k,lathm,2) + tmpi
c
                  tmpr = d(ir)*alp(mr+n,irow)
                  tmpi = d(ii)*alp(mr+n,irow)
                  div(2*m-1,k,latm) = div(2*m-1,k,latm) + tmpr
                  div(2*m  ,k,latm) = div(2*m  ,k,latm) + tmpi
c
                  tmpr = vz(ir)*alp(mr+n,irow)
                  tmpi = vz(ii)*alp(mr+n,irow)
                  vort(2*m-1,k,latm) = vort(2*m-1,k,latm) + tmpr
                  vort(2*m  ,k,latm) = vort(2*m  ,k,latm) + tmpi
               end do
            end do
#else
            do m=1,pmmax
               mr = nstart(m)
               mc = 2*mr
               mck = (k-1)*psp + mc
               do n=1,nlen(m),2
                  ir = mck + 2*n - 1
                  ii = ir + 1
c
                  tmpr = d(ir)*alpn(mr+n)
                  tmpi = d(ii)*alpn(mr+n)
                  u3(2*m-1,k,latm) = u3(2*m-1,k,latm) + tmpi
                  u3(2*m  ,k,latm) = u3(2*m  ,k,latm) - tmpr
c
                  tmpr = d(ir)*dalpn(mr+n)
                  tmpi = d(ii)*dalpn(mr+n)
                  v3(2*m-1,k,latp) = v3(2*m-1,k,latp) - tmpr
                  v3(2*m  ,k,latp) = v3(2*m  ,k,latp) - tmpi
c
                  tmpr = vz(ir)*dalpn(mr+n)
                  tmpi = vz(ii)*dalpn(mr+n)
                  u3(2*m-1,k,latp) = u3(2*m-1,k,latp) + tmpr
                  u3(2*m  ,k,latp) = u3(2*m  ,k,latp) + tmpi
c
                  tmpr = vz(ir)*alpn(mr+n)
                  tmpi = vz(ii)*alpn(mr+n)
                  v3(2*m-1,k,latm) = v3(2*m-1,k,latm) + tmpi
                  v3(2*m  ,k,latm) = v3(2*m  ,k,latm) - tmpr
c
                  tmpr = t(ir)*alp(mr+n,irow)
                  tmpi = t(ii)*alp(mr+n,irow)
                  t3(2*m-1,k,latm) = t3(2*m-1,k,latm) + tmpr
                  t3(2*m  ,k,latm) = t3(2*m  ,k,latm) + tmpi
c
                  tmpr = q(ir)*alp(mr+n,irow)
                  tmpi = q(ii)*alp(mr+n,irow)
                  q3(2*m-1,k,latm) = q3(2*m-1,k,latm) + tmpr
                  q3(2*m  ,k,latm) = q3(2*m  ,k,latm) + tmpi
c
                  tmpr = d(ir)*alp(mr+n,irow)
                  tmpi = d(ii)*alp(mr+n,irow)
                  div(2*m-1,k,latm) = div(2*m-1,k,latm) + tmpr
                  div(2*m  ,k,latm) = div(2*m  ,k,latm) + tmpi
c
                  tmpr = vz(ir)*alp(mr+n,irow)
                  tmpi = vz(ii)*alp(mr+n,irow)
                  vort(2*m-1,k,latm) = vort(2*m-1,k,latm) + tmpr
                  vort(2*m  ,k,latm) = vort(2*m  ,k,latm) + tmpi
               end do
            end do
#endif
#ifdef PP_PCCM
            do mm=m_fwavbeg,m_fwavend
               m  = m_fwavdex(mm)
               mr = m_offalp(mm)
               mc = 2*mr
c               mck = (k-1)*psp + mc
               mck = (k-1)*m_nspect2 + mc
               do n=2,m_lenalp(mm),2
                  ir = mck + 2*n - 1
                  ii = ir + 1
c
                  tmpr = d(ir)*alpn(mr+n)
                  tmpi = d(ii)*alpn(mr+n)
                  u3(2*m-1,k,lathp,1) = u3(2*m-1,k,lathp,1) + tmpi
                  u3(2*m  ,k,lathp,1) = u3(2*m  ,k,lathp,1) - tmpr
c
                  tmpr = d(ir)*dalpn(mr+n)
                  tmpi = d(ii)*dalpn(mr+n)
                  v3(2*m-1,k,lathm,2) = v3(2*m-1,k,lathm,2) - tmpr
                  v3(2*m  ,k,lathm,2) = v3(2*m  ,k,lathm,2) - tmpi
c
                  tmpr = vz(ir)*dalpn(mr+n)
                  tmpi = vz(ii)*dalpn(mr+n)
                  u3(2*m-1,k,lathm,2) = u3(2*m-1,k,lathm,2) + tmpr
                  u3(2*m  ,k,lathm,2) = u3(2*m  ,k,lathm,2) + tmpi
c
                  tmpr = vz(ir)*alpn(mr+n)
                  tmpi = vz(ii)*alpn(mr+n)
                  v3(2*m-1,k,lathp,1) = v3(2*m-1,k,lathp,1) + tmpi
                  v3(2*m  ,k,lathp,1) = v3(2*m  ,k,lathp,1) - tmpr
c
                  tmpr = t(ir)*alp(mr+n,irow)
                  tmpi = t(ii)*alp(mr+n,irow)
                  t3(2*m-1,k,lathp,1) = t3(2*m-1,k,lathp,1) + tmpr
                  t3(2*m  ,k,lathp,1) = t3(2*m  ,k,lathp,1) + tmpi
c
                  tmpr = q(ir)*alp(mr+n,irow)
                  tmpi = q(ii)*alp(mr+n,irow)
                  q3(2*m-1,k,lathp,1) = q3(2*m-1,k,lathp,1) + tmpr
                  q3(2*m  ,k,lathp,1) = q3(2*m  ,k,lathp,1) + tmpi
c
                  tmpr = d(ir)*alp(mr+n,irow)
                  tmpi = d(ii)*alp(mr+n,irow)
                  div(2*m-1,k,latp) = div(2*m-1,k,latp) + tmpr
                  div(2*m  ,k,latp) = div(2*m  ,k,latp) + tmpi
c
                  tmpr = vz(ir)*alp(mr+n,irow)
                  tmpi = vz(ii)*alp(mr+n,irow)
                  vort(2*m-1,k,latp) = vort(2*m-1,k,latp) + tmpr
                  vort(2*m  ,k,latp) = vort(2*m  ,k,latp) + tmpi
               end do
            end do
#else
            do m=1,pmmax
               mr = nstart(m)
               mc = 2*mr
               mck = (k-1)*psp + mc
               do n=2,nlen(m),2
                  ir = mck + 2*n - 1
                  ii = ir + 1
c
                  tmpr = d(ir)*alpn(mr+n)
                  tmpi = d(ii)*alpn(mr+n)
                  u3(2*m-1,k,latp) = u3(2*m-1,k,latp) + tmpi
                  u3(2*m  ,k,latp) = u3(2*m  ,k,latp) - tmpr
c
                  tmpr = d(ir)*dalpn(mr+n)
                  tmpi = d(ii)*dalpn(mr+n)
                  v3(2*m-1,k,latm) = v3(2*m-1,k,latm) - tmpr
                  v3(2*m  ,k,latm) = v3(2*m  ,k,latm) - tmpi
c
                  tmpr = vz(ir)*dalpn(mr+n)
                  tmpi = vz(ii)*dalpn(mr+n)
                  u3(2*m-1,k,latm) = u3(2*m-1,k,latm) + tmpr
                  u3(2*m  ,k,latm) = u3(2*m  ,k,latm) + tmpi
c
                  tmpr = vz(ir)*alpn(mr+n)
                  tmpi = vz(ii)*alpn(mr+n)
                  v3(2*m-1,k,latp) = v3(2*m-1,k,latp) + tmpi
                  v3(2*m  ,k,latp) = v3(2*m  ,k,latp) - tmpr
c
                  tmpr = t(ir)*alp(mr+n,irow)
                  tmpi = t(ii)*alp(mr+n,irow)
                  t3(2*m-1,k,latp) = t3(2*m-1,k,latp) + tmpr
                  t3(2*m  ,k,latp) = t3(2*m  ,k,latp) + tmpi
c
                  tmpr = q(ir)*alp(mr+n,irow)
                  tmpi = q(ii)*alp(mr+n,irow)
                  q3(2*m-1,k,latp) = q3(2*m-1,k,latp) + tmpr
                  q3(2*m  ,k,latp) = q3(2*m  ,k,latp) + tmpi
c
                  tmpr = d(ir)*alp(mr+n,irow)
                  tmpi = d(ii)*alp(mr+n,irow)
                  div(2*m-1,k,latp) = div(2*m-1,k,latp) + tmpr
                  div(2*m  ,k,latp) = div(2*m  ,k,latp) + tmpi
c
                  tmpr = vz(ir)*alp(mr+n,irow)
                  tmpi = vz(ii)*alp(mr+n,irow)
                  vort(2*m-1,k,latp) = vort(2*m-1,k,latp) + tmpr
                  vort(2*m  ,k,latp) = vort(2*m  ,k,latp) + tmpi
               end do
            end do
#endif
c
c Correction to get the absolute vorticity.
c
#ifdef PP_PCCM
            if ( l_pwave0 ) then
                vort(m_pwave0_dex,k,latp) = 
     &            vort(m_pwave0_dex,k,latp) + zcor
            endif
#else
            vort(1,k,latp) = vort(1,k,latp) + zcor
#endif
  270    continue
c
c Recompute real fields from symmetric and antisymmetric parts
c
CDIR$ IVDEP
         do 280 i=1,plonp2
            tmp1 = phis(i,latm) + phis(i,latp)
            tmp2 = phis(i,latm) - phis(i,latp)
            phis(i,latm) = tmp1
            phis(i,latp) = tmp2
c
            tmp1 = ps(i,latm) + ps(i,latp)
            tmp2 = ps(i,latm) - ps(i,latp)
            ps(i,latm) = tmp1
            ps(i,latp) = tmp2
c
            tmp1 = pl(i,latm) + pl(i,latp)
            tmp2 = pl(i,latm) - pl(i,latp)
            pl(i,latm) = tmp1
            pl(i,latp) = tmp2
c
            tmp1 = pm(i,latm) + pm(i,latp)
            tmp2 = pm(i,latm) - pm(i,latp)
            pm(i,latm) = tmp1
            pm(i,latp) = tmp2
  280    continue
CDIR$ IVDEP
         do 285 k=1,plev
            do 284 i=1,plonp2
#ifdef PP_PCCM
               tmp1 = u3(i,k,lathm,2) + u3(i,k,lathp,1)
               tmp2 = u3(i,k,lathm,2) - u3(i,k,lathp,1)
               u3(i,k,lathm,2) = tmp1
               u3(i,k,lathp,1) = tmp2
c
               tmp1 = v3(i,k,lathm,2) + v3(i,k,lathp,1)
               tmp2 = v3(i,k,lathm,2) - v3(i,k,lathp,1)
               v3(i,k,lathm,2) = tmp1
               v3(i,k,lathp,1) = tmp2
c
               tmp1 = t3(i,k,lathm,2) + t3(i,k,lathp,1)
               tmp2 = t3(i,k,lathm,2) - t3(i,k,lathp,1)
               t3(i,k,lathm,2) = tmp1
               t3(i,k,lathp,1) = tmp2
c
               tmp1 = q3(i,k,lathm,2) + q3(i,k,lathp,1)
               tmp2 = q3(i,k,lathm,2) - q3(i,k,lathp,1)
               q3(i,k,lathm,2) = tmp1
               q3(i,k,lathp,1) = tmp2
c
#else
c
               tmp1 = u3(i,k,latm) + u3(i,k,latp)
               tmp2 = u3(i,k,latm) - u3(i,k,latp)
               u3(i,k,latm) = tmp1
               u3(i,k,latp) = tmp2
c
               tmp1 = v3(i,k,latm) + v3(i,k,latp)
               tmp2 = v3(i,k,latm) - v3(i,k,latp)
               v3(i,k,latm) = tmp1
               v3(i,k,latp) = tmp2
c
               tmp1 = t3(i,k,latm) + t3(i,k,latp)
               tmp2 = t3(i,k,latm) - t3(i,k,latp)
               t3(i,k,latm) = tmp1
               t3(i,k,latp) = tmp2
c
               tmp1 = q3(i,k,latm) + q3(i,k,latp)
               tmp2 = q3(i,k,latm) - q3(i,k,latp)
               q3(i,k,latm) = tmp1
               q3(i,k,latp) = tmp2
c
#endif
c
               tmp1 = vort(i,k,latm) + vort(i,k,latp)
               tmp2 = vort(i,k,latm) - vort(i,k,latp)
               vort(i,k,latm) = tmp1
               vort(i,k,latp) = tmp2
c
               tmp1 = div(i,k,latm) + div(i,k,latp)
               tmp2 = div(i,k,latm) - div(i,k,latp)
               div(i,k,latm) = tmp1
               div(i,k,latp) = tmp2
  284       continue
  285    continue
  360 continue
      return
      end

