#include "params.h"

      subroutine priver(runoff,orunoff)
c
c-----------------------------------------------------------------------
      implicit none
c-----------------------------------------------------------------------
#include "pmgrid.com"
c-----------------------------------------------------------------------
#include "pagrid.com"
c-----------------------------------------------------------------------
#include "pspect.com"
c-----------------------------------------------------------------------
#include "comtim.com"
c-----------------------------------------------------------------------
#include "pcoupled.com"
c-----------------------------------------------------------------------
#include "priver.com"
c-----------------------------------------------------------------------
#include "pcplrstr.com"
c-----------------------------------------------------------------------
#include "comparll.com"
c-----------------------------------------------------------------------
c
c  Route the runoff from the land surface to the ocean via a
c  simple river model.
c  Base on work of Vorosmarty et al. and Miller et al. 1994.
c  
c---------------------------Code history--------------------------------
c  Original code:  Mike Coe
c  Ported to coupled model by:  Robert Jacob, Feb 1995
c  
c
c ----------------------------------------------------
c
crlj  uwlat: 1 = South Pole  128 = North Pole
      real delt,epsilon
c
c ----------------------------------------------------
c
c
c input arguments
c
c
      real runoff(uwlon,uwlat)
c
c  output arguments
      real orunoff(uwlon,uwlat)
c
c ----------------------------------------------------
c  runoff = basin sfc runoff 
c  sfluxin = sum of fluxes from other gridboxes.
c  fluxout = total flux out of gridbox (only one).
c ----------------------------------------------------

      integer ixx,ixc,ic2,iy,ix,milliclock,hold_time,ierr

c
c ----------------------------------------------------
c  dvolrdt = change volume in gridbox, volr = total volume in 
c  gridbox, effvel = effective velocity (u), dist = distance
c  in x direction in each gridbox, ddist = distance to
c  downstream box, 
c
      real dvolrdt(uwlon,uwlat),
     $     runb(uwlon,uwlat),
     $     dist(uwlon,uwlat)

c
c ----------------------------------------------------
c  circ = global circumfereuwlone,
c  uwlon = no. of gridboxes in x direction
c  io = refereuwlone gradient used in effvel (u) calc.
c  ic = calc grad. Z(I)-Z(I+1)/dist
c  delt = timestep in seconds
c  nyrs = no. of years to run program
c
      real pi,rad,circ,dx,io,rho
      real lune
      real*8 radius
c
c ----------------------------------------------------
c  basic constants in meters, circumfereuwlone of globe,
c  distauwlone between latititudes, radians conversion,
c  io = refereuwlone gradient per Miller at al.
c  rho = density of water (kg/m3)
c
      parameter (circ=4.0024E+7)
      parameter (pi = 3.1415926536, rad = pi/180.)
      parameter (io = 5.0e-5, rho = 1000.)
      parameter (epsilon = 1.e-9, lune = 2.8125)
      parameter (radius = 6.378E+6)
c
c ----------------------------------------------------
c  basic integer values for mechanics
c
      integer i,j,k,m,i2,j2,i3,imon1
c ----------------------------------------------------
c
c ----------------------------------------------------
      integer row,wordz,ic,jc

      real passwlon(puwlat)
      real passelon(puwlat)
      real recvwlon(puwlat)
      real recvelon(puwlat)

c since lats are folded over at the equator
c its easier to refer to equator and poleward
c instead of just north and south
      real passplat(2*(puwlon+2))
      real passelat(2*(puwlon+2))
      real recvplat(2*(puwlon+2))
      real recvelat(2*(puwlon+2))

#ifdef WORD4
      wordz=4
#else
      wordz=8
#endif

      row = uwlat/p_nproc
      delt = dtime
#ifdef DIAGNOSTICS
c      write(6,*) rdirc(1,1),effvel(1,1)
#endif

      do m=1,2
      do j = plats(m),plate(m)

       do i = plons,plone

         if(uwmask(i,j) .eq. 1)then

c  convert from m to  m^3/sec 
          runb(i,j) = runoff(i,j)*reuwarea(j)/delt
         else
          runb(i,j) = 0.
         endif
c
       enddo
      enddo
      enddo
c
      do 121 m=1,2
      do 120 j = plats(m),plate(m)
       do 110 i = plons,plone
c
       if(uwmask(i,j) .eq. 1)then
c
         dvolrdt(i,j) = runb(i,j)+
     *             (sfluxin(i,j) -fluxout(i,j))
c
         volr(i,j) = max(volr(i,j)+dvolrdt(i,j)*delt,0.)
c
         if(volr(i,j) .lt. epsilon)then
          volr(i,j) = 0.
         endif

c
c Calculation of fluxout. 
c expression used by Vorosmarty et al., Huggins and Burney,
c Flux is a function of the total volume and the effective
c velocity (effvel) which is an arbitrary function of the 
c downstream gradient taken from Miller et al.
c
c calculate fluxout (a function of slope)
c
          fluxout(i,j) = volr(i,j)*(effvel(i,j)/ddist(i,j))
c
        endif

	if(uwmask(i,j) .eq. 0) then
	 orunoff(i,j) = sfluxin(i,j)/reuwarea(j)

c make sure fluxout is 0 over ocean
	 fluxout(i,j) = 0.
        endif
c
 110    continue
 120   continue
 121   continue
c
#ifdef TIME_ON
       hold_time=milliclock()
#endif

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c  Update fluxout
ccccccccccccccccccccccccccccccccccccccccccccccccccc

#ifdef UPSHOT
      call log_event(21,m_pin,'StrtGath',8)
#endif

c     call gather0(fluxout(plons,plats),fluxout,row*uwlon)
cccccccccccccccccccccccccccccccccccccc
c  Do the East-West comm
cccccccccccccccccccccccccccccccccccccc
c everybody load the left and right longitude into array
      jc=1
      do m=1,2
      do j = plats(m),plate(m)
      passwlon(jc)=fluxout(plons,j)
      passelon(jc)=fluxout(plone,j)
      jc = jc+1
      enddo
      enddo
    
      if(mod(m_pin_ln,2) .eq. 0) then
	call send0(passwlon,puwlat*wordz,800,m_pin_lft)
	call recv0(recvelon,puwlat*wordz,800)

	call send0(passelon,puwlat*wordz,820,m_pin_rht)
	call recv0(recvwlon,puwlat*wordz,820)
      else
	call recv0(recvelon,puwlat*wordz,800)
	call send0(passwlon,puwlat*wordz,800,m_pin_lft)

	call recv0(recvwlon,puwlat*wordz,820)
	call send0(passelon,puwlat*wordz,820,m_pin_rht)
      endif

      jc=1
      do m=1,2
      do j = plats(m),plate(m)
       if( m_pin_ln .eq. (p_nproc_ln -1 )) then
        fluxout(1,j) = recvelon(jc)
       else
        fluxout(plone+1,j) = recvelon(jc)
       endif

       if( m_pin_ln .eq. 0) then
        fluxout(uwlon,j) = recvwlon(jc)
       else
        fluxout(plons-1,j) = recvwlon(jc)
       endif

      jc = jc+1
      enddo
      enddo

cccccccccccccccccccccccccccccccccccccc
c  Do the North-South comm
cccccccccccccccccccccccccccccccccccccc

      if( (m_pin_ln .ne. 0) .and. 
     $ (m_pin_ln .ne. (p_nproc_ln-1))) then
      ic = 1
      do i=plons-1,plone+1
       passelat(ic)=fluxout(i,plate(1))
       passelat(puwlon+2+ic)=fluxout(i,plats(2))
       passplat(ic)=fluxout(i,plats(1))
       passplat(puwlon+2+ic)=fluxout(i,plate(2))
       ic = ic +1
      enddo
      endif

      if(m_pin_ln .eq. 0) then
       passelat(1)=fluxout(uwlon,plate(1))
       passelat(puwlon+2+1)=fluxout(uwlon,plats(2))
       passplat(1)=fluxout(uwlon,plats(1))
       passplat(puwlon+2+1)=fluxout(uwlon,plate(2))
       ic = 2
       do i=plons,plone+1
        passelat(ic)=fluxout(i,plate(1))
        passelat(puwlon+2+ic)=fluxout(i,plats(2))
        passplat(ic)=fluxout(i,plats(1))
        passplat(puwlon+2+ic)=fluxout(i,plate(2))
        ic = ic +1
       enddo
      endif

      if(m_pin_ln .eq. (p_nproc_ln -1)) then
       ic = 1
       do i=plons-1,plone
        passelat(ic)=fluxout(i,plate(1))
        passelat(puwlon+2+ic)=fluxout(i,plats(2))
        passplat(ic)=fluxout(i,plats(1))
        passplat(puwlon+2+ic)=fluxout(i,plate(2))
        ic = ic +1
       enddo
       passelat(ic)=fluxout(1,plate(1))
       passelat(puwlon+2+ic)=fluxout(1,plats(2))
       passplat(ic)=fluxout(1,plats(1))
       passplat(puwlon+2+ic)=fluxout(1,plate(2))
      endif

      if(m_pin_lt .eq. 0)then
       call send0(passelat,2*(puwlon+2)*wordz,700,m_pin_up)
       call recv0(recvelat,2*(puwlon+2)*wordz,730)
      else if (m_pin_lt .eq. (p_nproc_lt-1)) then
       call recv0(recvplat,2*(puwlon+2)*wordz,700)
       call send0(passplat,2*(puwlon+2)*wordz,730,m_pin_dwn)
      else
	if(mod(m_pin_lt,2) .eq. 0) then
         call send0(passelat,2*(puwlon+2)*wordz,700,m_pin_up)
         call recv0(recvplat,2*(puwlon+2)*wordz,700)
         call send0(passplat,2*(puwlon+2)*wordz,730,m_pin_dwn)
         call recv0(recvelat,2*(puwlon+2)*wordz,730)
	else
         call recv0(recvplat,2*(puwlon+2)*wordz,700)
         call send0(passelat,2*(puwlon+2)*wordz,700,m_pin_up)
         call recv0(recvelat,2*(puwlon+2)*wordz,730)
         call send0(passplat,2*(puwlon+2)*wordz,730,m_pin_dwn)
	endif
      endif 

c unpack the lats received from the poleward direction
      if(m_pin_lt .ne. 0) then

       if( (m_pin_ln .ne. 0) .and. 
     $  (m_pin_ln .ne. (p_nproc_ln-1))) then
          ic = 1
          do i=plons-1,plone+1
           fluxout(i,plats(1)-1)=recvplat(ic)
	   fluxout(i,plate(2)+1)=recvplat(puwlon+2+ic)
	   ic = ic +1
          enddo

        else if(m_pin_ln .eq. 0) then
	  fluxout(uwlon,plats(1)-1)=recvplat(1)
	  fluxout(uwlon,plate(2)+1)=recvplat(puwlon+2+1)
	  ic=2
          do i=plons,plone+1
           fluxout(i,plats(1)-1)=recvplat(ic)
	   fluxout(i,plate(2)+1)=recvplat(puwlon+2+ic)
	   ic = ic + 1
          enddo

        else if(m_pin_ln .eq. (p_nproc_ln -1)) then
	  ic=1
          do i=plons-1,plone
           fluxout(i,plats(1)-1)=recvplat(ic)
	   fluxout(i,plate(2)+1)=recvplat(puwlon+2+ic)
	   ic = ic +1
          enddo
	  fluxout(1,plats(1)-1)=recvplat(ic)
	  fluxout(1,plate(2)+1)=recvplat(puwlon+2+ic)
        endif
      endif

c unpack the lats received from the equatorward direction
      if(m_pin_lt .ne. (p_nproc_lt-1)) then

       if( (m_pin_ln .ne. 0) .and. 
     $  (m_pin_ln .ne. (p_nproc_ln-1))) then
          ic = 1
          do i=plons-1,plone+1
           fluxout(i,plate(1)+1)=recvelat(ic)
	   fluxout(i,plats(2)-1)=recvelat(puwlon+2+ic)
	   ic = ic + 1
          enddo

        else if(m_pin_ln .eq. 0) then
	  fluxout(uwlon,plate(1)+1)=recvelat(1)
	  fluxout(uwlon,plats(2)-1)=recvelat(puwlon+2+1)
	  ic=2
          do i=plons,plone+1
           fluxout(i,plate(1)+1)=recvelat(ic)
	   fluxout(i,plats(2)-1)=recvelat(puwlon+2+ic)
	   ic = ic + 1
          enddo

        else if(m_pin_ln .eq. (p_nproc_ln -1)) then
	  ic=1
          do i=plons-1,plone
           fluxout(i,plate(1)+1)=recvelat(ic)
	   fluxout(i,plats(2)-1)=recvelat(puwlon+2+ic)
	   ic = ic + 1
          enddo
	  fluxout(1,plate(1)+1)=recvelat(ic)
	  fluxout(1,plats(2)-1)=recvelat(puwlon+2+ic)
        endif
      endif


#ifdef UPSHOT
      call log_event(22,m_pin,'EndGath',7)
#endif

#ifdef TIME_ON
      if(m_pin .eq. 0) then
      write(0,*) 'Time for one gather in river model:',
     $     milliclock()-hold_time
      endif
      hold_time=milliclock()
#endif

c ----------------------------------------------------
c call to sumflux to calculate sfluxin
c

         call psumflux(fluxout,sfluxin)

#ifdef TIME_ON
      if(m_pin .eq. 0) then
      write(0,*) 'Time for call to sumflux in river:',
     $     milliclock()-hold_time
      endif
      hold_time=milliclock()
#endif

c ----------------------------------------------------
c
c
       
      return
      end
c
c ---------------------------------------------------------------------
c
      subroutine psumflux(fluxout,sfluxin)
c
c This subroutine calculates the sum of all fluxes into a 
c  basin. Each gridbox surrounding (8 total) are checked  
c  to see if the river direction (rdirc) is set to flow to
c  that grid box. If so then the fluxout for that box is added 
c  to the sum of fluxes in (sfluxin).
c
      implicit none
c-----------------------------------------------------------------------
#include "pmgrid.com"
c-----------------------------------------------------------------------
#include "pagrid.com"
c-----------------------------------------------------------------------
#include "pcoupled.com"
c-----------------------------------------------------------------------
#include "priver.com"
c-----------------------------------------------------------------------
c
c   input  
      real fluxout(uwlon,uwlat)
c   output
      real sfluxin(uwlon,uwlat)

      integer i,j,m
c
      do m=1,2
      do j = plats(m),plate(m)
       do i = plons,plone
c
        sfluxin(i,j) = 0.
c
        if ((i .ne. 1) .and. (j .ne. uwlat))then
         if(rdirc(i-1,j+1) .eq. 4)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(i-1,j+1)
         endif
        endif
	
        if ((i .eq. 1) .and. (j .ne. uwlat))then
         if(rdirc(uwlon,j+1) .eq. 4)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(uwlon,j+1)
         endif
        endif
c
c
        if(j .ne. uwlat)then
         if(rdirc(i,j+1) .eq. 5)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(i,j+1)
         endif
        endif

        if((j .eq. 1) .and. (i .le. uwlon/2))then
         if(rdirc(i+(uwlon/2),j) .eq. 5)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(i+(uwlon/2),j)
         endif
        endif

        if((j .eq. 1) .and. (i .gt. uwlon/2))then
         if(rdirc(i-(uwlon/2),j) .eq. 5)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(i-(uwlon/2),j)
         endif
        endif
c
        if ((i .ne. uwlon) .and. (j .ne. uwlat))then
         if(rdirc(i+1,j+1) .eq. 6)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(i+1,j+1)
         endif
        endif

        if ((i .eq. uwlon) .and. (j .ne. uwlat))then
         if(rdirc(1,j+1) .eq. 6)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(1,j+1)
         endif
        endif
c
        if (i .ne. 1)then
         if(rdirc(i-1,j) .eq. 3)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(i-1,j)
         endif
        endif
	
        if (i .eq. 1)then
         if(rdirc(uwlon,j) .eq. 3)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(uwlon,j)
         endif
        endif
c
        if (i .ne. uwlon)then
         if(rdirc(i+1,j) .eq. 7)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(i+1,j)
         endif
        endif

        if (i .eq. uwlon)then
         if(rdirc(1,j) .eq. 7)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(1,j)
         endif
        endif
c
        if ((i .ne. 1) .and. (j .ne. 1))then
         if(rdirc(i-1,j-1) .eq. 2)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(i-1,j-1)
         endif
        endif

        if ((i .eq. 1) .and. (j .ne. 1))then
         if(rdirc(uwlon,j-1) .eq. 2)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(uwlon,j-1)
         endif
        endif

c
        if (j .ne. 1)then
         if(rdirc(i,j-1) .eq. 1)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(i,j-1)
         endif
        endif
c
        if ((i .ne. uwlon) .and. (j .ne. 1))then
         if(rdirc(i+1,j-1) .eq. 8)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(i+1,j-1)
         endif
        endif
	
        if ((i .eq. uwlon) .and. (j .ne. 1))then
         if(rdirc(1,j-1) .eq. 8)then
          sfluxin(i,j) = sfluxin(i,j) + fluxout(1,j-1)
         endif
        endif
c
        if(sfluxin(i,j) .lt. 0.)then
         sfluxin(i,j) = 0.
        endif
c
       enddo
      enddo
      enddo
      return
      end
